@@ -36,12 +36,12 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/pixel_cache.h"
-#include "magick/color.h"
-#include "magick/transform.h"
+#include "magick/colormap.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
+#include "magick/pixel_cache.h"
 #include "magick/render.h"
+#include "magick/transform.h"
 #include "magick/utility.h"
 
 /*
@@ -239,7 +239,8 @@ static unsigned int DecodeImage(Image *image,const unsigned long compression,
       }
     if (QuantumTick(y,image->rows))
       if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                  LoadImageText,image->filename))
+                                  LoadImageText,image->filename,
+				  image->columns,image->rows))
         break;
   }
   (void) ReadBlobByte(image);  /* end of line */
@@ -329,7 +330,8 @@ static size_t EncodeImage(Image *image,const unsigned long bytes_per_line,
     *q++=0x00;
     if (QuantumTick(y,image->rows))
       if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                  SaveImageText,image->filename))
+                                  SaveImageText,image->filename,
+				  image->columns,image->rows))
         break;
   }
   /*
@@ -634,7 +636,8 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             {
               status=MagickMonitorFormatted(image->rows-y-1,image->rows,
                                             exception,LoadImageText,
-                                            image->filename);
+                                            image->filename,
+					    image->columns,image->rows);
               if (status == False)
                 break;
             }
@@ -680,7 +683,8 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             {
               status=MagickMonitorFormatted(image->rows-y-1,image->rows,
                                             exception,LoadImageText,
-                                            image->filename);
+                                            image->filename,
+					    image->columns,image->rows);
               if (status == False)
                 break;
             }
@@ -717,7 +721,8 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             {
               status=MagickMonitorFormatted(image->rows-y-1,image->rows,
                                             exception,LoadImageText,
-                                            image->filename);
+                                            image->filename,
+					    image->columns,image->rows);
               if (status == False)
                 break;
             }
@@ -766,7 +771,8 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             {
               status=MagickMonitorFormatted(image->rows-y-1,image->rows,
                                             exception,LoadImageText,
-                                            image->filename);
+                                            image->filename,
+					    image->columns,image->rows);
               if (status == False)
                 break;
             }
@@ -801,7 +807,8 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             {
               status=MagickMonitorFormatted(image->rows-y-1,image->rows,
                                             exception,LoadImageText,
-                                            image->filename);
+                                            image->filename,
+					    image->columns,image->rows);
               if (status == False)
                 break;
             }
@@ -829,6 +836,8 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           DestroyImageList(image);
           return((Image *) NULL);
         }
+      DestroyBlobInfo(flipped_image->blob);
+      flipped_image->blob=ReferenceBlob(image->blob);
       DestroyImage(image);
       image=flipped_image;
     }
@@ -1075,7 +1084,8 @@ static unsigned int WriteDIBImage(const ImageInfo *image_info,Image *image)
        if (image->previous == (Image *) NULL)
          if (QuantumTick(y,image->rows))
            if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                       SaveImageText,image->filename))
+                                       SaveImageText,image->filename,
+				       image->columns,image->rows))
              break;
       }
       break;
@@ -1103,7 +1113,8 @@ static unsigned int WriteDIBImage(const ImageInfo *image_info,Image *image)
         if (image->previous == (Image *) NULL)
           if (QuantumTick(y,image->rows))
             if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                        SaveImageText,image->filename))
+                                        SaveImageText,image->filename,
+					image->columns,image->rows))
               break;
       }
       break;
@@ -1139,7 +1150,8 @@ static unsigned int WriteDIBImage(const ImageInfo *image_info,Image *image)
         if (image->previous == (Image *) NULL)
           if (QuantumTick(y,image->rows))
             if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                        SaveImageText,image->filename))
+                                        SaveImageText,image->filename,
+					image->columns,image->rows))
                break;
       }
       break;
