@@ -37,10 +37,11 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/pixel_cache.h"
+#include "magick/colormap.h"
 #include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
+#include "magick/pixel_cache.h"
 #include "magick/utility.h"
 #include "magick/xwindow.h"
 #if defined(HasDPS)
@@ -329,7 +330,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,
     {
       register unsigned long
         color,
-        index;
+        index_val;
 
       unsigned long
         blue_mask,
@@ -376,18 +377,20 @@ static Image *ReadDPSImage(const ImageInfo *image_info,
           for (x=0; x < (long) image->columns; x++)
           {
             pixel=XGetPixel(dps_image,x,y);
-            index=(pixel >> red_shift) & red_mask;
-            q->red=ScaleShortToQuantum(colors[index].red);
-            index=(pixel >> green_shift) & green_mask;
-            q->green=ScaleShortToQuantum(colors[index].green);
-            index=(pixel >> blue_shift) & blue_mask;
-            q->blue=ScaleShortToQuantum(colors[index].blue);
+            index_val=(pixel >> red_shift) & red_mask;
+            q->red=ScaleShortToQuantum(colors[index_val].red);
+            index_val=(pixel >> green_shift) & green_mask;
+            q->green=ScaleShortToQuantum(colors[index_val].green);
+            index_val=(pixel >> blue_shift) & blue_mask;
+            q->blue=ScaleShortToQuantum(colors[index_val].blue);
             q++;
           }
           if (!SyncImagePixels(image))
             break;
           if (QuantumTick(y,image->rows))
-            if (!MagickMonitor(LoadImageText,y,image->rows,exception))
+            if (!MagickMonitorFormatted(y,image->rows,exception,
+                                        LoadImageText,image->filename,
+					image->columns,image->rows))
               break;
         }
       else
@@ -410,7 +413,9 @@ static Image *ReadDPSImage(const ImageInfo *image_info,
           if (!SyncImagePixels(image))
             break;
           if (QuantumTick(y,image->rows))
-            if (!MagickMonitor(LoadImageText,y,image->rows,exception))
+            if (!MagickMonitorFormatted(y,image->rows,exception,
+                                        LoadImageText,image->filename,
+					image->columns,image->rows))
               break;
         }
       break;
@@ -448,13 +453,15 @@ static Image *ReadDPSImage(const ImageInfo *image_info,
         q=SetImagePixels(image,0,y,image->columns,1);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
+        indexes=AccessMutableIndexes(image);
         for (x=0; x < (long) image->columns; x++)
           indexes[x]=(unsigned short) XGetPixel(dps_image,x,y);
         if (!SyncImagePixels(image))
           break;
         if (QuantumTick(y,image->rows))
-          if (!MagickMonitor(LoadImageText,y,image->rows,exception))
+          if (!MagickMonitorFormatted(y,image->rows,exception,
+                                      LoadImageText,image->filename,
+				      image->columns,image->rows))
             break;
       }
       break;
@@ -557,8 +564,8 @@ ModuleExport void RegisterDPSImage(void)
   entry=SetMagickInfo("DPS");
   entry->decoder=(DecoderHandler) ReadDPSImage;
   entry->blob_support=False;
-  entry->description=AcquireString("Display Postscript Interpreter");
-  entry->module=AcquireString("DPS");
+  entry->description="Display Postscript Interpreter";
+  entry->module="DPS";
   (void) RegisterMagickInfo(entry);
 }
 
