@@ -39,7 +39,6 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/pixel_cache.h"
-#include "magick/color.h"
 #include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
@@ -559,7 +558,8 @@ static Image *ReadJP2Image(const ImageInfo *image_info,
       if (image->previous == (Image *) NULL)
         if (QuantumTick(y,image->rows))
           if (!MagickMonitorFormatted(y,image->rows,exception,LoadImageText,
-                                      image->filename))
+                                      image->filename,
+				      image->columns,image->rows))
             break;
     }
   if (number_components == 1)
@@ -965,7 +965,8 @@ static unsigned int WriteJP2Image(const ImageInfo *image_info,Image *image)
     if (image->previous == (Image *) NULL)
       if (QuantumTick(y,image->rows))
         if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                    SaveImageText,image->filename))
+                                    SaveImageText,image->filename,
+				    image->columns,image->rows))
           break;
   }
   (void) strlcpy(magick,image_info->magick,MaxTextExtent);
