@@ -378,32 +378,32 @@ ModuleExport void RegisterLOCALEImage(void)
   entry->encoder=(EncoderHandler) WriteLOCALEImage;
   entry->adjoin=False;
   entry->stealth=True;
-  entry->description=AcquireString("Locale Message File");
-  entry->module=AcquireString("LOCALE");
+  entry->description="Locale Message File";
+  entry->module="LOCALE";
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("LOCALEMC");
   entry->encoder=(EncoderHandler) WriteLOCALEImage;
   entry->adjoin=False;
   entry->stealth=True;
-  entry->description=AcquireString("Microsoft Message File");
-  entry->module=AcquireString("LOCALE");
+  entry->description="Microsoft Message File";
+  entry->module="LOCALE";
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("LOCALEC");
   entry->encoder=(EncoderHandler) WriteLOCALEImage;
   entry->adjoin=False;
   entry->stealth=True;
-  entry->description=AcquireString("Locale Message File - C code");
-  entry->module=AcquireString("LOCALE");
+  entry->description="Locale Message File - C code";
+  entry->module="LOCALE";
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("LOCALEH");
   entry->encoder=(EncoderHandler) WriteLOCALEImage;
   entry->adjoin=False;
   entry->stealth=True;
-  entry->description=AcquireString("Locale Message File - C header file");
-  entry->module=AcquireString("LOCALE");
+  entry->description="Locale Message File - C header file";
+  entry->module="LOCALE";
   (void) RegisterMagickInfo(entry);
 }
 
@@ -526,6 +526,7 @@ static char *EscapeLocaleString(const char *str)
     return strput;
 }
 
+#if 0
 static void FreeAccumulatedStrings(void *handle)
 {
     struct locale_str *xl = (struct locale_str *)handle;
@@ -536,6 +537,7 @@ static void FreeAccumulatedStrings(void *handle)
     MagickFreeMemory(xl->name);
     MagickFreeMemory(handle);
 }
+#endif
 
 /*  accumulate -- read a line from the file, break it up at the '/'s into
  *                individual subfields and build a tree structure that has
