@@ -38,18 +38,19 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/pixel_cache.h"
 #include "magick/color.h"
+#include "magick/colormap.h"
+#include "magick/constitute.h"
 #include "magick/magick.h"
+#include "magick/magick_endian.h"
 #include "magick/monitor.h"
+#include "magick/pixel_cache.h"
 #include "magick/shear.h"
+#include "magick/tempfile.h"
 #include "magick/transform.h"
 #include "magick/utility.h"
-#include "magick/constitute.h"
-#include "magick/magick_endian.h"
-#include "magick/tempfile.h"
 #if defined(HasZLIB)
- #include "zlib.h"
+#  include "zlib.h"
 #endif
 
 
@@ -294,14 +295,23 @@ unsigned char val = 0;
   *BuffL = val;
 }
 
+#if defined(HasZLIB)
+static voidpf ZLIBAllocFunc(voidpf opaque, uInt items, uInt size)
+{
+  ARG_NOT_USED(opaque);
+  return MagickMallocCleared((size_t) items*size);
+}
+static void ZLIBFreeFunc(voidpf opaque, voidpf address)
+{
+  ARG_NOT_USED(opaque);
+  MagickFree(address);
+}
 
 /** This procedure decompreses an image block for a new MATLAB format. */
 static Image *DecompressBlock(Image *orig, magick_off_t Size, ImageInfo *clone_info, ExceptionInfo *exception)
 {
-#if defined(HasZLIB)
-
 Image *image2;
-void *CacheBlock, *DecompressBlock;
+void *cache_block, *decompress_block;
 z_stream zip_info;
 FILE *mat_file;
 size_t magick_size;
@@ -315,26 +325,26 @@ int status;
     (void) unlink(clone_info->filename);
   }
 
-  CacheBlock = MagickAllocateMemory(unsigned char *,(size_t)((Size<16384)?Size:16384));
-  if(CacheBlock==NULL) return NULL;
-  DecompressBlock = MagickAllocateMemory(unsigned char *,(size_t)(4096));
-  if(DecompressBlock==NULL) 
+  cache_block = MagickAllocateMemory(unsigned char *,(size_t)((Size<16384)?Size:16384));
+  if(cache_block==NULL) return NULL;
+  decompress_block = MagickAllocateMemory(unsigned char *,(size_t)(4096));
+  if(decompress_block==NULL) 
   {
-    MagickFreeMemory(CacheBlock);    
+    MagickFreeMemory(cache_block);    
     return NULL;
   }
 
   mat_file = AcquireTemporaryFileStream(clone_info->filename,BinaryFileIOMode);
   if(!mat_file)
   {
-    MagickFreeMemory(CacheBlock);
-    MagickFreeMemory(DecompressBlock);
+    MagickFreeMemory(cache_block);
+    MagickFreeMemory(decompress_block);
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Gannot create file stream for PS image");
     return NULL;
   }
 
-  zip_info.zalloc = (alloc_func) NULL;
-  zip_info.zfree = (free_func) NULL;
+  zip_info.zalloc = ZLIBAllocFunc;
+  zip_info.zfree = ZLIBFreeFunc;
   zip_info.opaque = (voidpf) NULL;
   inflateInit(&zip_info);
   /* zip_info.next_out = 8*4; */
@@ -343,16 +353,16 @@ int status;
   zip_info.total_out = 0;
   while(Size>0 && !EOFBlob(orig))
   {    
-    magick_size = ReadBlob(orig, (Size<16384)?Size:16384, CacheBlock);
-    zip_info.next_in = CacheBlock;
+    magick_size = ReadBlob(orig, (Size<16384)?Size:16384, cache_block);
+    zip_info.next_in = cache_block;
     zip_info.avail_in = magick_size;    
 
     while(zip_info.avail_in>0)
     {
       zip_info.avail_out = 4096;    
-      zip_info.next_out = DecompressBlock;
+      zip_info.next_out = decompress_block;
       status = inflate(&zip_info,Z_NO_FLUSH);      
-      fwrite(DecompressBlock, 4096-zip_info.avail_out, 1, mat_file);
+      fwrite(decompress_block, 4096-zip_info.avail_out, 1, mat_file);
 
       if(status == Z_STREAM_END) goto DblBreak;
     }
@@ -362,8 +372,8 @@ int status;
 DblBreak:
  
   (void)fclose(mat_file);
-  MagickFreeMemory(CacheBlock);
-  MagickFreeMemory(DecompressBlock);
+  MagickFreeMemory(cache_block);
+  MagickFreeMemory(decompress_block);
 
   if((clone_info->file=fopen(clone_info->filename,"rb"))==NULL) goto UnlinkFile;
   if( (image2 = AllocateImage(clone_info))==NULL ) goto EraseFile;  
@@ -380,10 +390,8 @@ UnlinkFile:
   }
 
   return image2;
-#else
-  return NULL;
-#endif
 }
+#endif /*  defined(HasZLIB) */
 
 
 
@@ -1039,7 +1047,8 @@ static unsigned int WriteMATLABImage(const ImageInfo *image_info,Image *image)
           (void) WriteBlob(image,image->rows,pixels);
           if (QuantumTick(progress_quantum,progress_span))
             if (!MagickMonitorFormatted(progress_quantum,progress_span,&image->exception,
-                                        SaveImageText,image->filename))
+                                        SaveImageText,image->filename,
+					image->columns,image->rows))
               goto BreakAll;
 	}    
       } while(z-- >= 2);
