@@ -39,13 +39,14 @@
 #include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/color.h"
+#include "magick/colormap.h"
 #include "magick/compress.h"
 #include "magick/constitute.h"
 #include "magick/enum_strings.h"
-#include "magick/pixel_cache.h"
 #include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
+#include "magick/pixel_cache.h"
 #include "magick/profile.h"
 #include "magick/utility.h"
 #if defined(HasZLIB)
@@ -599,6 +600,32 @@ static unsigned int PushImageRLEPixels(Image *image,
   return(True);
 }
 
+#if 0
+static void *BZLIBAllocFunc(void *opaque, int items, int size)
+{
+  ARG_NOT_USED(opaque);
+  return MagickMallocCleared((size_t) items*size);
+}
+static void BZLIBFreeFunc(void *opaque, void *address)
+{
+  ARG_NOT_USED(opaque);
+  MagickFree(address);
+}
+#endif
+
+#if defined(HasZLIB)
+static voidpf ZLIBAllocFunc(voidpf opaque, uInt items, uInt size)
+{
+  ARG_NOT_USED(opaque);
+  return MagickMallocCleared((size_t) items*size);
+}
+static void ZLIBFreeFunc(voidpf opaque, voidpf address)
+{
+  ARG_NOT_USED(opaque);
+  MagickFree(address);
+}
+#endif /* defined(HasZLIB) */
+
 static Image *ReadMIFFImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
@@ -1107,8 +1134,9 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
     (void) ReadBlobByte(image);
 
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                          "class=%s compression=%s matte=%s columns=%lu rows=%lu depth=%u",
-                          ClassTypeToString(image->storage_class),
+                          "id=\"%s\" class=%s compression=%s matte=%s "
+			  "columns=%lu rows=%lu depth=%u",
+                          id,ClassTypeToString(image->storage_class),
                           CompressionTypeToString(image->compression),
                           MagickBoolToString(image->matte),
                           image->columns, image->rows, image->depth);
@@ -1309,8 +1337,8 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                 break;
               if (y == 0)
                 {
-                  zip_info.zalloc=(alloc_func) NULL;
-                  zip_info.zfree=(free_func) NULL;
+                  zip_info.zalloc=ZLIBAllocFunc;
+                  zip_info.zfree=ZLIBFreeFunc;
                   zip_info.opaque=(voidpf) NULL;
                   code=inflateInit(&zip_info);
                   status|=code >= 0;
@@ -1345,7 +1373,8 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
                   if (!MagickMonitorFormatted(y,image->rows,exception,
-                                              LoadImageText,image->filename))
+                                              LoadImageText,image->filename,
+					      image->columns,image->rows))
                     break;
             }
           break;
@@ -1397,7 +1426,8 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
                   if (!MagickMonitorFormatted(y,image->rows,exception,
-                                              LoadImageText,image->filename))
+                                              LoadImageText,image->filename,
+					      image->columns,image->rows))
                     break;
             }
           break;
@@ -1426,7 +1456,8 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
                   if (!MagickMonitorFormatted(y,image->rows,exception,
-                                              LoadImageText,image->filename))
+                                              LoadImageText,image->filename,
+					      image->columns,image->rows))
                     break;
 
             }
@@ -1447,7 +1478,8 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
                   if (!MagickMonitorFormatted(y,image->rows,exception,
-                                              LoadImageText,image->filename))
+                                              LoadImageText,image->filename,
+					      image->columns,image->rows))
                     break;
             }
           break;
@@ -2216,8 +2248,8 @@ static unsigned int WriteMIFFImage(const ImageInfo *image_info,Image *image)
         {
           if (y == 0)
             {
-              zip_info.zalloc=(alloc_func) NULL;
-              zip_info.zfree=(free_func) NULL;
+              zip_info.zalloc=ZLIBAllocFunc;
+              zip_info.zfree=ZLIBFreeFunc;
               zip_info.opaque=(voidpf) NULL;
               code=deflateInit(&zip_info,(int) Min(image_info->quality/10,9));
               status|=code >= 0;
@@ -2344,7 +2376,8 @@ static unsigned int WriteMIFFImage(const ImageInfo *image_info,Image *image)
       if (image->previous == (Image *) NULL)
         if (QuantumTick(y,image->rows))
           if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                      SaveImageText,image->filename))
+                                      SaveImageText,image->filename,
+				      image->columns,image->rows))
             break;
     }
     MagickFreeMemory(pixels);
