@@ -39,12 +39,12 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
-#include "magick/pixel_cache.h"
-#include "magick/color.h"
+#include "magick/colormap.h"
 #include "magick/constitute.h"
 #include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/paint.h"
+#include "magick/pixel_cache.h"
 #include "magick/quantize.h"
 #include "magick/utility.h"
 
