@@ -37,10 +37,10 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/pixel_cache.h"
-#include "magick/color.h"
+#include "magick/colormap.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
+#include "magick/pixel_cache.h"
 #include "magick/utility.h"
 
 /*
@@ -302,6 +302,9 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     */
     image->columns=(pcx_info.right-pcx_info.left)+1;
     image->rows=(pcx_info.bottom-pcx_info.top)+1;
+    if ((image->columns == 0) || (image->rows == 0) ||
+        (pcx_info.bits_per_pixel == 0))
+      ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
     image->depth=pcx_info.bits_per_pixel <= 8 ? 8 : QuantumDepth;
     image->units=PixelsPerInchResolution;
     image->x_resolution=pcx_info.horizontal_resolution;
@@ -446,7 +449,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       q=SetImagePixels(image,0,y,image->columns,1);
       if (q == (PixelPacket *) NULL)
         break;
-      indexes=GetIndexes(image);
+      indexes=AccessMutableIndexes(image);
       r=scanline;
       if (image->storage_class == DirectClass)
         for (i=0; i < pcx_info.planes; i++)
@@ -583,7 +586,9 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         break;
       if (image->previous == (Image *) NULL)
         if (QuantumTick(y,image->rows))
-          if (!MagickMonitor(LoadImageText,y,image->rows,exception))
+          if (!MagickMonitorFormatted(y,image->rows,exception,LoadImageText,
+                                      image->filename,
+				      image->columns,image->rows))
             break;
     }
     if (image->storage_class == PseudoClass)
@@ -623,7 +628,8 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        if (!MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image),exception))
+        if (!MagickMonitorFormatted(TellBlob(image),GetBlobSize(image),exception,
+                                    LoadImagesText,image->filename))
           break;
       }
   }
@@ -668,8 +674,8 @@ ModuleExport void RegisterPCXImage(void)
   entry->encoder=(EncoderHandler) WritePCXImage;
   entry->seekable_stream=True;
   entry->magick=(MagickHandler) IsDCX;
-  entry->description=AcquireString("ZSoft IBM PC multi-page Paintbrush");
-  entry->module=AcquireString("PCX");
+  entry->description="ZSoft IBM PC multi-page Paintbrush";
+  entry->module="PCX";
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("PCX");
@@ -678,8 +684,8 @@ ModuleExport void RegisterPCXImage(void)
   entry->magick=(MagickHandler) IsPCX;
   entry->adjoin=False;
   entry->seekable_stream=True;
-  entry->description=AcquireString("ZSoft IBM PC Paintbrush");
-  entry->module=AcquireString("PCX");
+  entry->description="ZSoft IBM PC Paintbrush";
+  entry->module="PCX";
   (void) RegisterMagickInfo(entry);
 }
 
@@ -800,7 +806,7 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
-  register IndexPacket
+  register const IndexPacket
     *indexes;
 
   register long
@@ -1016,7 +1022,9 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
           if (WriteRLEPixels(image,&pcx_info,pcx_pixels) == MagickFail)
             break;
           if (QuantumTick(y,image->rows))
-            if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+            if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                        SaveImageText,image->filename,
+					image->columns,image->rows))
               break;
         }
       }
@@ -1031,7 +1039,7 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          indexes=GetIndexes(image);
+          indexes=AccessImmutableIndexes(image);
           q=pcx_pixels;
           /* For each column ... */
           for (x=0; x < (long) image->columns; x++)
@@ -1040,7 +1048,9 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+              if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                          SaveImageText,image->filename,
+					  image->columns,image->rows))
                 break;
         }
       else
@@ -1063,7 +1073,7 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
             p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
             if (p == (const PixelPacket *) NULL)
               break;
-            indexes=GetIndexes(image);
+            indexes=AccessImmutableIndexes(image);
             bit=0;
             byte=0;
             q=pcx_pixels;
@@ -1088,7 +1098,9 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
             break;
             if (image->previous == (Image *) NULL)
               if (QuantumTick(y,image->rows))
-                if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+                if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                            SaveImageText,image->filename,
+					    image->columns,image->rows))
                   break;
           }
         }
@@ -1100,8 +1112,9 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesText,scene++,GetImageListLength(image),
-      &image->exception);
+    status=MagickMonitorFormatted(scene++,GetImageListLength(image),
+                                  &image->exception,SaveImagesText,
+                                  image->filename);
     if (status == False)
       break;
     if (scene >= 1023)
