@@ -39,6 +39,7 @@
 #include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/color.h"
+#include "magick/colormap.h"
 #include "magick/constitute.h"
 #include "magick/log.h"
 #include "magick/magick.h"
@@ -403,7 +404,8 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 if (image->previous == (Image *) NULL)
                   if (QuantumTick(y,image->rows))
                     if (!MagickMonitorFormatted(y,image->rows,exception,
-                                                LoadImageText,image->filename))
+                                                LoadImageText,image->filename,
+						image->columns,image->rows))
                       break;
                 if (EOFBlob(image))
                   break;
@@ -463,7 +465,8 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 if (image->previous == (Image *) NULL)
                   if (QuantumTick(y,image->rows))
                     if (!MagickMonitorFormatted(y,image->rows,exception,
-                                                LoadImageText,image->filename))
+                                                LoadImageText,image->filename,
+						image->columns,image->rows))
                       break;
               }
             image->is_monochrome=is_monochrome;
@@ -523,7 +526,8 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 if (image->previous == (Image *) NULL)
                   if (QuantumTick(y,image->rows))
                     if (!MagickMonitorFormatted(y,image->rows,exception,
-                                                LoadImageText,image->filename))
+                                                LoadImageText,image->filename,
+						image->columns,image->rows))
                       break;
               }
             image->is_monochrome=is_monochrome;
@@ -537,9 +541,6 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           {
             ImportPixelAreaOptions
               import_options;
-        
-            ImportPixelAreaInfo
-              import_info;
 
             size_t
               bytes_per_row;
@@ -560,7 +561,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             ImportPixelAreaOptionsInit(&import_options);
             import_options.grayscale_miniswhite=MagickTrue;
 #if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp parallel for schedule(dynamic,1) shared(row_count,status)
+#  pragma omp parallel for schedule(static,1) shared(row_count,status)
 #endif
             for (y=0; y < (long) image->rows; y++)
               {
@@ -572,6 +573,9 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
                 unsigned long
                   thread_row_count;
+
+		ImportPixelAreaInfo
+		  import_info;
           
                 thread_status=status;
                 if (thread_status == MagickFail)
@@ -579,7 +583,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
                 pixels=AccessThreadViewData(scanline_set);
 #if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp critical
+#  pragma omp critical (GM_ReadPNMImage)
 #endif
                 {
                   if (ReadBlobZC(image,bytes_per_row,&pixels) != bytes_per_row)
@@ -592,7 +596,8 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     if (QuantumTick(thread_row_count,image->rows))
                       if (!MagickMonitorFormatted(thread_row_count,image->rows,
                                                   exception,LoadImageText,
-                                                  image->filename))
+                                                  image->filename,
+						  image->columns,image->rows))
                         thread_status=MagickFail;
                 }
                 if (thread_status != MagickFail)
@@ -612,7 +617,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 if (thread_status == MagickFail)
                   {
 #if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp critical
+#  pragma omp critical (GM_ReadPNMImage)
 #endif
                     status=MagickFail;
                   }
@@ -635,7 +640,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
               bytes_per_row;
 
             MagickBool
-              is_monochrome;
+              is_monochrome=MagickTrue;
 
             unsigned long
               row_count=0;
@@ -643,7 +648,6 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             ThreadViewDataSet
               *scanline_set;
 
-            is_monochrome=MagickTrue;
             packets=(raw_sample_bits+7)/8;
             bytes_per_row=packets*image->columns;
 
@@ -652,7 +656,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
               ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
 
 #if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp parallel for schedule(dynamic,1) shared(is_monochrome,row_count,status)
+#  pragma omp parallel for schedule(static,1) shared(is_monochrome,row_count,status)
 #endif
             for (y=0; y < (long) image->rows; y++)
               {
@@ -678,13 +682,14 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 if (thread_status == MagickFail)
                   continue;
 
-                thread_is_monochrome=is_monochrome;
                 pixels=AccessThreadViewData(scanline_set);
 
 #if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp critical
+#  pragma omp critical (GM_ReadPNMImage)
 #endif
                 {
+		  thread_is_monochrome=is_monochrome;
+
                   if (ReadBlobZC(image,bytes_per_row,&pixels) != bytes_per_row)
                     thread_status=MagickFail;
 
@@ -695,7 +700,8 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     if (QuantumTick(thread_row_count,image->rows))
                       if (!MagickMonitorFormatted(thread_row_count,image->rows,
                                                   exception,LoadImageText,
-                                                  image->filename))
+                                                  image->filename,
+						  image->columns,image->rows))
                         thread_status=MagickFail;
                 }
 
@@ -730,7 +736,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     thread_status=MagickFail;
 
 #if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp critical
+#  pragma omp critical (GM_ReadPNMImage)
 #endif
                 {
                   if (thread_status == MagickFail)
@@ -778,7 +784,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
               ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
 #if 1
 #if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp parallel for schedule(dynamic,1) shared(is_grayscale,is_monochrome,row_count,status)
+#  pragma omp parallel for schedule(static,1) shared(is_grayscale,is_monochrome,row_count,status)
 #endif
 #endif
             for (y=0; y < (long) image->rows; y++)
@@ -806,14 +812,15 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 if (thread_status == MagickFail)
                   continue;
             
-                thread_is_grayscale=is_grayscale;
-                thread_is_monochrome=is_monochrome;
                 pixels=AccessThreadViewData(scanline_set);
             
 #if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp critical
+#  pragma omp critical (GM_ReadPNMImage)
 #endif
                 {
+		  thread_is_grayscale=is_grayscale;
+		  thread_is_monochrome=is_monochrome;
+
                   if (ReadBlobZC(image,bytes_per_row,&pixels) != bytes_per_row)
                     thread_status=MagickFail;
               
@@ -824,7 +831,8 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     if (QuantumTick(thread_row_count,image->rows))
                       if (!MagickMonitorFormatted(thread_row_count,image->rows,
                                                   exception,LoadImageText,
-                                                  image->filename))
+                                                  image->filename,
+						  image->columns,image->rows))
                         thread_status=MagickFail;
                 }
 
@@ -858,7 +866,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     thread_status=MagickFail;
 
 #if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp critical
+#  pragma omp critical (GM_ReadPNMImage)
 #endif
                 {
                   if (thread_status == MagickFail)
@@ -1178,16 +1186,16 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
     if (attribute != (const ImageAttribute *) NULL)
       {
         register char
-          *p;
+          *av;
 
         /*
           Write comments to file.
         */
         (void) WriteBlobByte(image,'#');
-        for (p=attribute->value; *p != '\0'; p++)
+        for (av=attribute->value; *av != '\0'; av++)
         {
-          (void) WriteBlobByte(image,*p);
-          if ((*p == '\n') && (*(p+1) != '\0'))
+          (void) WriteBlobByte(image,*av);
+          if ((*av == '\n') && (*(av+1) != '\0'))
             (void) WriteBlobByte(image,'#');
         }
         (void) WriteBlobByte(image,'\n');
@@ -1238,7 +1246,8 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                          SaveImageText,image->filename))
+                                          SaveImageText,image->filename,
+					  image->columns,image->rows))
                 break;
         }
         if (i != 0)
@@ -1279,7 +1288,8 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                          SaveImageText,image->filename))
+                                          SaveImageText,image->filename,
+					  image->columns,image->rows))
                 break;
         }
         if (i != 0)
@@ -1321,7 +1331,8 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                          SaveImageText,image->filename))
+                                          SaveImageText,image->filename,
+					  image->columns,image->rows))
                 break;
         }
         if (i != 0)
@@ -1367,7 +1378,8 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
             if (image->previous == (Image *) NULL)
               if (QuantumTick(y,image->rows))
                 if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                            SaveImageText,image->filename))
+                                            SaveImageText,image->filename,
+					    image->columns,image->rows))
                   break;
           }
         MagickFreeMemory(pixels);
@@ -1410,7 +1422,8 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                          SaveImageText,image->filename))
+                                          SaveImageText,image->filename,
+					  image->columns,image->rows))
                 break;
         }
         MagickFreeMemory(pixels);
@@ -1454,7 +1467,8 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                          SaveImageText,image->filename))
+                                          SaveImageText,image->filename,
+					  image->columns,image->rows))
                 break;
         }
         MagickFreeMemory(pixels);
@@ -1569,7 +1583,8 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
             i=0;
           if (QuantumTick(y,image->rows))
             if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                        SaveImageText,image->filename))
+                                        SaveImageText,image->filename,
+					image->columns,image->rows))
               break;
         }
         /*
