@@ -18,7 +18,7 @@
 %                            P      SSSSS  22222                              %
 %                                                                             %
 %                                                                             %
-%                   Read/Write GraphicsMagick Image Format.                   %
+%                     Write Postscript Level II Format.                       %
 %                                                                             %
 %                                                                             %
 %                              Software Design                                %
@@ -38,7 +38,7 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
-#include "magick/cache.h"
+#include "magick/pixel_cache.h"
 #include "magick/color.h"
 #include "magick/constitute.h"
 #include "magick/compress.h"
@@ -137,11 +137,11 @@ static unsigned int Huffman2DEncodeImage(const ImageInfo *image_info,
   huffman_image=CloneImage(image,0,0,True,&image->exception);
   if (huffman_image == (Image *) NULL)
     return(False);
-  SetImageType(huffman_image,BilevelType);
+  (void) SetImageType(huffman_image,BilevelType);
   if(!AcquireTemporaryFileName(filename))
     {
       DestroyImage(huffman_image);
-      ThrowBinaryException(FileOpenError,"UnableToCreateTemporaryFile",
+      ThrowBinaryException(FileOpenError,UnableToCreateTemporaryFile,
         filename);
     }
   FormatString(huffman_image->filename,"tiff:%s",filename);
@@ -155,8 +155,8 @@ static unsigned int Huffman2DEncodeImage(const ImageInfo *image_info,
   tiff=TIFFOpen(filename,"rb");
   if (tiff == (TIFF *) NULL)
     {
-      LiberateTemporaryFile(filename);
-      ThrowBinaryException(FileOpenError,"UnableToOpenFile",
+      (void) LiberateTemporaryFile(filename);
+      ThrowBinaryException(FileOpenError,UnableToOpenFile,
         image_info->filename)
     }
   /*
@@ -167,12 +167,12 @@ static unsigned int Huffman2DEncodeImage(const ImageInfo *image_info,
   for (i=1; i < (long) TIFFNumberOfStrips(tiff); i++)
     if (byte_count[i] > strip_size)
       strip_size=byte_count[i];
-  buffer=(unsigned char *) AcquireMemory(strip_size);
+  buffer=MagickAllocateMemory(unsigned char *,strip_size);
   if (buffer == (unsigned char *) NULL)
     {
       TIFFClose(tiff);
-      LiberateTemporaryFile(filename);
-      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+      (void) LiberateTemporaryFile(filename);
+      ThrowBinaryException(ResourceLimitError,MemoryAllocationFailed,
         (char *) NULL)
     }
   /*
@@ -189,9 +189,9 @@ static unsigned int Huffman2DEncodeImage(const ImageInfo *image_info,
       Ascii85Encode(image,(unsigned long) buffer[j]);
     Ascii85Flush(image);
   }
-  LiberateMemory((void **) &buffer);
+  MagickFreeMemory(buffer);
   TIFFClose(tiff);
-  LiberateTemporaryFile(filename);
+  (void) LiberateTemporaryFile(filename);
   return(True);
 }
 #else
@@ -200,7 +200,7 @@ static unsigned int Huffman2DEncodeImage(const ImageInfo *image_info,
 {
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  ThrowBinaryException(MissingDelegateError,"TIFFLibraryIsNotAvailable",image->filename);
+  ThrowBinaryException(MissingDelegateError,TIFFLibraryIsNotAvailable,image->filename);
 }
 #endif
 
@@ -236,15 +236,17 @@ ModuleExport void RegisterPS2Image(void)
   entry->encoder=(EncoderHandler) WritePS2Image;
   entry->adjoin=False;
   entry->seekable_stream=True;
-  entry->description=AcquireString("Adobe Level II Encapsulated PostScript");
-  entry->module=AcquireString("PS2");
+  entry->description="Adobe Level II Encapsulated PostScript";
+  entry->module="PS2";
+  entry->coder_class=PrimaryCoderClass;
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("PS2");
   entry->encoder=(EncoderHandler) WritePS2Image;
   entry->seekable_stream=True;
-  entry->description=AcquireString("Adobe Level II PostScript");
-  entry->module=AcquireString("PS2");
+  entry->description="Adobe Level II PostScript";
+  entry->module="PS2";
+  entry->coder_class=PrimaryCoderClass;
   (void) RegisterMagickInfo(entry);
 }
 
@@ -506,7 +508,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
-  register IndexPacket
+  register const IndexPacket
     *indexes;
 
   register long
@@ -516,7 +518,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
     i;
 
   SegmentInfo
-    bounds;
+    bounds={0.0,0.0,0.0,0.0};
 
   size_t
     length;
@@ -545,7 +547,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
   assert(image->signature == MagickSignature);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+    ThrowWriterException(FileOpenError,UnableToOpenFile,image);
   compression=image->compression;
   if (image_info->compression != UndefinedCompression)
     compression=image_info->compression;
@@ -555,17 +557,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
     case JPEGCompression:
     {
       compression=RLECompression;
-      ThrowException(&image->exception,MissingDelegateError,
-        "JPEGLibraryIsNotAvailable",image->filename);
-      break;
-    }
-#endif
-#if !defined(HasLZW)
-    case LZWCompression:
-    {
-      compression=RLECompression;
-      ThrowException(&image->exception,MissingDelegateError,
-        "LZWEncodingNotEnabled",image->filename);
+      ThrowException(&image->exception,MissingDelegateError,JPEGLibraryIsNotAvailable,image->filename);
       break;
     }
 #endif
@@ -573,8 +565,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
     case ZipCompression:
     {
       compression=RLECompression;
-      ThrowException(&image->exception,MissingDelegateError,
-        "ZipLibraryIsNotAvailable",image->filename);
+      ThrowException(&image->exception,MissingDelegateError,ZipLibraryIsNotAvailable,image->filename);
       break;
     }
 #endif
@@ -585,6 +576,9 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
   scene=0;
   do
   {
+    ImageCharacteristics
+      characteristics;
+
     /*
       Scale image to size of Postscript page.
     */
@@ -597,7 +591,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
     geometry.y=(long) text_size;
     FormatString(page_geometry,"%lux%lu",image->columns,image->rows);
     if (image_info->page != (char *) NULL)
-      (void) strncpy(page_geometry,image_info->page,MaxTextExtent-1);
+      (void) strlcpy(page_geometry,image_info->page,MaxTextExtent);
     else
       if ((image->page.width != 0) && (image->page.height != 0))
         (void) FormatString(page_geometry,"%lux%lu%+ld%+ld",image->page.width,
@@ -614,13 +608,13 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
     dy_resolution=72.0;
     x_resolution=72.0;
     (void) strcpy(density,PSDensityGeometry);
-    count=sscanf(density,"%lfx%lf",&x_resolution,&y_resolution);
+    count=GetMagickDimension(density,&x_resolution,&y_resolution,NULL,NULL);
     if (count != 2)
       y_resolution=x_resolution;
     if (image_info->density != (char *) NULL)
       {
-        count=sscanf(image_info->density,"%lfx%lf",&x_resolution,
-          &y_resolution);
+        count=GetMagickDimension(image_info->density,&x_resolution,
+          &y_resolution,NULL,NULL);
         if (count != 2)
           y_resolution=x_resolution;
       }
@@ -643,7 +637,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
         (void) WriteBlobString(image,buffer);
         timer=time((time_t *) NULL);
         (void) localtime(&timer);
-        (void) strncpy(date,ctime(&timer),MaxTextExtent-1);
+        (void) strlcpy(date,ctime(&timer),MaxTextExtent);
         date[strlen(date)-1]='\0';
         FormatString(buffer,"%%%%CreationDate: (%.1024s)\n",date);
         (void) WriteBlobString(image,buffer);
@@ -756,14 +750,21 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
         {
           FormatString(buffer,"%.1024s \n",labels[i]);
           (void) WriteBlobString(image,buffer);
-          LiberateMemory((void **) &labels[i]);
+          MagickFreeMemory(labels[i]);
         }
-        LiberateMemory((void **) &labels);
+        MagickFreeMemory(labels);
       }
     number_pixels=image->columns*image->rows;
+
+    /*
+      Analyze image to be written.
+    */
+    (void) GetImageCharacteristics(image,&characteristics,
+                                   (OptimizeType == image_info->type),
+                                   &image->exception);
     if ((compression == FaxCompression) ||
         ((image_info->type != TrueColorType) &&
-         IsGrayImage(image,&image->exception)))
+         (characteristics.grayscale)))
       {
         FormatString(buffer,"%lu %lu\n1\n%d\n",image->columns,image->rows,
           (int) (image->colorspace == CMYKColorspace));
@@ -792,12 +793,12 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
             */
             jpeg_image=CloneImage(image,0,0,True,&image->exception);
             if (jpeg_image == (Image *) NULL)
-              ThrowWriterException(CoderError,image->exception.reason,image);
+              ThrowWriterException2(CoderError,image->exception.reason,image);
             (void) strcpy(jpeg_image->magick,"JPEG");
             blob=ImageToBlob(image_info,jpeg_image,&length,&image->exception);
             (void) WriteBlob(image,length,blob);
             DestroyImage(jpeg_image);
-            LiberateMemory((void **) &blob);
+            MagickFreeMemory(blob);
             break;
           }
           case RLECompression:
@@ -810,9 +811,9 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
               Allocate pixel array.
             */
             length=number_pixels;
-            pixels=(unsigned char *) AcquireMemory(length);
+            pixels=MagickAllocateMemory(unsigned char *,length);
             if (pixels == (unsigned char *) NULL)
-              ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
+              ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,
                 image);
             /*
               Dump Runlength encoded pixels.
@@ -832,8 +833,11 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
                   {
-                    status=MagickMonitor(SaveImageText,y,image->rows,
-                      &image->exception);
+                    status=MagickMonitorFormatted(y,image->rows,
+                                                  &image->exception,
+                                                  SaveImageText,
+                                                  image->filename,
+						  image->columns,image->rows);
                     if (status == False)
                       break;
                   }
@@ -842,7 +846,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
               status=LZWEncodeImage(image,length,pixels);
             else
               status=PackbitsEncodeImage(image,length,pixels);
-            LiberateMemory((void **) &pixels);
+            MagickFreeMemory(pixels);
             if (!status)
               {
                 CloseBlob(image);
@@ -871,8 +875,11 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
                   {
-                    status=MagickMonitor(SaveImageText,y,image->rows,
-                      &image->exception);
+                    status=MagickMonitorFormatted(y,image->rows,
+                                                  &image->exception,
+                                                  SaveImageText,
+                                                  image->filename,
+						  image->columns,image->rows);
                     if (status == False)
                       break;
                   }
@@ -900,12 +907,12 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
               */
               jpeg_image=CloneImage(image,0,0,True,&image->exception);
               if (jpeg_image == (Image *) NULL)
-                ThrowWriterException(CoderError,image->exception.reason,image);
+                ThrowWriterException2(CoderError,image->exception.reason,image);
               (void) strcpy(jpeg_image->magick,"JPEG");
               blob=ImageToBlob(image_info,jpeg_image,&length,&image->exception);
               (void) WriteBlob(image,length,blob);
               DestroyImage(jpeg_image);
-              LiberateMemory((void **) &blob);
+              MagickFreeMemory(blob);
               break;
             }
             case RLECompression:
@@ -919,10 +926,9 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
               */
               length=(image->colorspace == CMYKColorspace ? 4 : 3)*
                 number_pixels;
-              pixels=(unsigned char *) AcquireMemory(length);
+              pixels=MagickAllocateMemory(unsigned char *,length);
               if (pixels == (unsigned char *) NULL)
-                ThrowWriterException(ResourceLimitError,
-                  "MemoryAllocationFailed",image);
+                ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
               /*
                 Dump Packbit encoded pixels.
               */
@@ -960,8 +966,11 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                 if (image->previous == (Image *) NULL)
                   if (QuantumTick(y,image->rows))
                     {
-                      status=MagickMonitor(SaveImageText,y,image->rows,
-                        &image->exception);
+                      status=MagickMonitorFormatted(y,image->rows,
+                                                    &image->exception,
+                                                    SaveImageText,
+                                                    image->filename,
+						    image->columns,image->rows);
                       if (status == False)
                         break;
                     }
@@ -975,7 +984,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                   CloseBlob(image);
                   return(False);
                 }
-              LiberateMemory((void **) &pixels);
+              MagickFreeMemory(pixels);
               break;
             }
             case NoCompression:
@@ -1017,8 +1026,11 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                 if (image->previous == (Image *) NULL)
                   if (QuantumTick(y,image->rows))
                     {
-                      status=MagickMonitor(SaveImageText,y,image->rows,
-                        &image->exception);
+                      status=MagickMonitorFormatted(y,image->rows,
+                                                    &image->exception,
+                                                    SaveImageText,
+                                                    image->filename,
+						    image->columns,image->rows);
                       if (status == False)
                         break;
                     }
@@ -1038,7 +1050,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
           (void) WriteBlobString(image,buffer);
           FormatString(buffer,"%d\n",(int) (compression == NoCompression));
           (void) WriteBlobString(image,buffer);
-          FormatString(buffer,"%lu\n",image->colors);
+          FormatString(buffer,"%u\n",image->colors);
           (void) WriteBlobString(image,buffer);
           for (i=0; i < (long) image->colors; i++)
           {
@@ -1060,10 +1072,9 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                 Allocate pixel array.
               */
               length=number_pixels;
-              pixels=(unsigned char *) AcquireMemory(length);
+              pixels=MagickAllocateMemory(unsigned char *,length);
               if (pixels == (unsigned char *) NULL)
-                ThrowWriterException(ResourceLimitError,
-                  "MemoryAllocationFailed",image);
+                ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
               /*
                 Dump Runlength encoded pixels.
               */
@@ -1074,14 +1085,17 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                   &image->exception);
                 if (p == (const PixelPacket *) NULL)
                   break;
-                indexes=GetIndexes(image);
+                indexes=AccessImmutableIndexes(image);
                 for (x=0; x < (long) image->columns; x++)
                   *q++=indexes[x];
                 if (image->previous == (Image *) NULL)
                   if (QuantumTick(y,image->rows))
                     {
-                      status=MagickMonitor(SaveImageText,y,image->rows,
-                        &image->exception);
+                      status=MagickMonitorFormatted(y,image->rows,
+                                                    &image->exception,
+                                                    SaveImageText,
+                                                    image->filename,
+						    image->columns,image->rows);
                       if (status == False)
                         break;
                     }
@@ -1090,7 +1104,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                 status=LZWEncodeImage(image,length,pixels);
               else
                 status=PackbitsEncodeImage(image,length,pixels);
-              LiberateMemory((void **) &pixels);
+              MagickFreeMemory(pixels);
               if (!status)
                 {
                   CloseBlob(image);
@@ -1110,14 +1124,17 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                   &image->exception);
                 if (p == (const PixelPacket *) NULL)
                   break;
-                indexes=GetIndexes(image);
+                indexes=AccessImmutableIndexes(image);
                 for (x=0; x < (long) image->columns; x++)
                   Ascii85Encode(image,indexes[x]);
                 if (image->previous == (Image *) NULL)
                   if (QuantumTick(y,image->rows))
                     {
-                      status=MagickMonitor(SaveImageText,y,image->rows,
-                        &image->exception);
+                      status=MagickMonitorFormatted(y,image->rows,
+                                                    &image->exception,
+                                                    SaveImageText,
+                                                    image->filename,
+						    image->columns,image->rows);
                       if (status == False)
                         break;
                     }
@@ -1142,8 +1159,9 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesText,scene++,GetImageListLength(image),
-      &image->exception);
+    status=MagickMonitorFormatted(scene++,GetImageListLength(image),
+                                  &image->exception,SaveImagesText,
+                                  image->filename);
     if (status == False)
       break;
   } while (image_info->adjoin);
