@@ -472,7 +472,8 @@ static unsigned int SerializePseudoClassImage(const ImageInfo *image_info,
       if (QuantumTick(y,image->rows))
         {
           status=MagickMonitorFormatted(y,image->rows,&image->exception,
-                                        SaveImageText,image->filename);
+                                        SaveImageText,image->filename,
+					image->columns,image->rows);
           if (status == False)
             break;
         }
@@ -564,7 +565,8 @@ static unsigned int SerializeMultiChannelImage(const ImageInfo *image_info,
       if (QuantumTick(y,image->rows))
         {
           status=MagickMonitorFormatted(y,image->rows,&image->exception,
-                                        SaveImageText,image->filename);
+                                        SaveImageText,image->filename,
+					image->columns,image->rows);
           if (status == False)
             break;
         }
@@ -680,7 +682,8 @@ static unsigned int SerializeSingleChannelImage(const ImageInfo *image_info,
         {
           status=MagickMonitorFormatted(y,image->rows,
                                         &image->exception,SaveImageText,
-                                        image->filename);
+                                        image->filename,
+					image->columns,image->rows);
           if (status == False)
             break;
         }
@@ -1936,6 +1939,18 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
 %
 %
 */
+
+static voidpf ZLIBAllocFunc(voidpf opaque, uInt items, uInt size)
+{
+  ARG_NOT_USED(opaque);
+  return MagickMallocCleared((size_t) items*size);
+}
+static void ZLIBFreeFunc(voidpf opaque, voidpf address)
+{
+  ARG_NOT_USED(opaque);
+  MagickFree(address);
+}
+
 static unsigned int ZLIBEncode2Image(Image *image,const size_t length,
   const unsigned long quality,unsigned char *pixels,WriteByteHook write_byte,
   void *info)
@@ -1966,8 +1981,8 @@ static unsigned int ZLIBEncode2Image(Image *image,const size_t length,
   stream.avail_in=(unsigned int) length;
   stream.next_out=compressed_pixels;
   stream.avail_out=(unsigned int) compressed_packets;
-  stream.zalloc=(alloc_func) NULL;
-  stream.zfree=(free_func) NULL;
+  stream.zalloc=ZLIBAllocFunc;
+  stream.zfree=ZLIBFreeFunc;
   stream.opaque=(voidpf) NULL;
   status=deflateInit(&stream,(int) Min(quality/10,9));
   if (status == Z_OK)
