@@ -39,10 +39,11 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
-#include "magick/pixel_cache.h"
+#include "magick/colormap.h"
 #include "magick/constitute.h"
 #include "magick/log.h"
 #include "magick/magick.h"
+#include "magick/pixel_cache.h"
 #include "magick/profile.h"
 #include "magick/utility.h"
 
@@ -121,7 +122,7 @@ static unsigned int DecodeImage(Image *image,const int channel)
                                (long) (x/image->columns),1,1);
               if (q == (PixelPacket *) NULL)
                 break;
-              indexes=GetIndexes(image);
+              indexes=AccessMutableIndexes(image);
               switch (channel)
                 {
                 case -1:
@@ -184,7 +185,7 @@ static unsigned int DecodeImage(Image *image,const int channel)
                            (long) (x/image->columns),1,1);
           if (q == (PixelPacket *) NULL)
             break;
-          indexes=GetIndexes(image);
+          indexes=AccessMutableIndexes(image);
           switch (channel)
             {
             case -1:
@@ -1156,7 +1157,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                                    layer_info[i].image->columns,1);
                   if (q == (PixelPacket *) NULL)
                     break;
-                  indexes=GetIndexes(layer_info[i].image);
+                  indexes=AccessMutableIndexes(layer_info[i].image);
                   for (x=0; x < (long) layer_info[i].image->columns; x++)
                     {
                       if (packet_size == 1)
@@ -1227,7 +1228,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                                    layer_info[i].image->columns,1);
                   if (q == (PixelPacket *) NULL)
                     break;
-                  indexes=GetIndexes(layer_info[i].image);
+                  indexes=AccessMutableIndexes(layer_info[i].image);
                   for (x=0; x < (long) layer_info[i].image->columns; x++)
                     {
                       q->opacity=(Quantum) ((unsigned long)
@@ -1361,7 +1362,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
               q=GetImagePixels(image,0,y,image->columns,1);
               if (q == (PixelPacket *) NULL)
                 break;
-              indexes=GetIndexes(image);
+              indexes=AccessMutableIndexes(image);
               for (x=0; x < (long) image->columns; x++)
                 {
                   if (packet_size == 1)
@@ -1489,8 +1490,8 @@ ModuleExport void RegisterPSDImage(void)
   entry->decoder=(DecoderHandler) ReadPSDImage;
   entry->encoder=(EncoderHandler) WritePSDImage;
   entry->magick=(MagickHandler) IsPSD;
-  entry->description=AcquireString("Adobe Photoshop bitmap");
-  entry->module=AcquireString("PSD");
+  entry->description="Adobe Photoshop bitmap";
+  entry->module="PSD";
   (void) RegisterMagickInfo(entry);
 }
 
