@@ -18,7 +18,7 @@
 %                            SSSSS   GGG   IIIII                              %
 %                                                                             %
 %                                                                             %
-%                   Read/Write GraphicsMagick Image Format.                   %
+%                     Read/Write Irix RGB Image Format.                       %
 %                                                                             %
 %                                                                             %
 %                              Software Design                                %
@@ -36,37 +36,52 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/attribute.h"
 #include "magick/blob.h"
-#include "magick/cache.h"
-#include "magick/color.h"
+#include "magick/colormap.h"
+#include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
+#include "magick/pixel_cache.h"
 #include "magick/utility.h"
 
 /*
   Typedef declaractions.
 */
+
+/*
+  The SGI file header is 512 bytes in size.
+*/
 typedef struct _SGIInfo
 {
   unsigned short
-    magic;
+    magic;                  /* Identification number (474) */
 
-  unsigned char
-    storage,
-    bytes_per_pixel;
+  magick_int8_t
+    storage,                /* Compression flag */
+    bytes_per_pixel;        /* Bytes per pixel (per bit plane) */
 
-  unsigned short
-    dimension,
-    columns,
-    rows,
-    depth;
+  magick_uint16_t
+    dimension,              /* Number of image dimensions */
+    xsize,                  /* Width of image in pixels */
+    ysize,                  /* Height of image in pixels */
+    zsize;                  /* Number of bit planes */
 
-  unsigned long
-    minimum_value,
-    maximum_value;
+  magick_uint32_t
+    pix_min,                /* Smallest pixel component value */
+    pix_max;                /* Largest pixel component value */
 
-  unsigned char
-    filler[492];
+  char
+    dummy1[4];              /* Not used */
+
+  char
+    image_name[80];         /* Name of image (null terminated) */
+
+  magick_uint32_t
+    color_map;              /* Format of pixel data */
+
+  char
+    dummy2[404];            /* Not used */
 } SGIInfo;
 
 /*
@@ -144,17 +159,18 @@ static unsigned int IsSGI(const unsigned char *magick,const size_t length)
 %
 */
 
-static void SGIDecode(const unsigned long bytes_per_pixel,
-  unsigned char *max_packets,unsigned char *pixels)
+static int SGIDecode(const unsigned long bytes_per_pixel,
+  unsigned char *max_packets,unsigned char *pixels,
+  unsigned long npackets,unsigned long npixels)
 {
-  long
+  unsigned long
     count;
 
   register unsigned char
     *p,
     *q;
 
-  unsigned long
+  unsigned int
     pixel;
 
   p=max_packets;
@@ -163,46 +179,64 @@ static void SGIDecode(const unsigned long bytes_per_pixel,
     {
       for ( ; ; )
       {
-        pixel=(*p++) << 8;
+         if (npackets-- == 0)
+           return -1;
+        pixel=(*p++) << 8U;
         pixel|=(*p++);
-        count=(long) (pixel & 0x7f);
+        count=(pixel & 0x7fU);
         if (count == 0)
           break;
-        if (pixel & 0x80)
-          for ( ; count != 0; count--)
+        if (count > npixels)
+          return -1;
+        npixels -= count;
+        if (pixel & 0x80U)
+          for ( ; count != 0U; count--)
           {
+            if (npackets-- == 0)
+              return -1;
             *q=(*p++);
             *(q+1)=(*p++);
-            q+=8;
+            q+=8U;
           }
         else
           {
-            pixel=(*p++) << 8;
+            if (npackets-- == 0)
+              return -1;
+            pixel=(*p++) << 8U;
             pixel|=(*p++);
             for ( ; count != 0; count--)
             {
-              *q=(unsigned char) (pixel >> 8);
+              *q=(unsigned char) (pixel >> 8U);
               *(q+1)=(unsigned char) pixel;
-              q+=8;
+              q+=8U;
             }
           }
       }
-      return;
+      return 0;
     }
   for ( ; ; )
   {
+    if (npackets-- == 0)
+      return -1;
     pixel=(*p++);
-    count=(long) (pixel & 0x7f);
+    count= (pixel & 0x7fU);
     if (count == 0)
       break;
+    if (count > npixels)
+      return -1;
+    npixels -= count;
     if (pixel & 0x80)
       for ( ; count != 0; count--)
       {
+        if (npackets-- == 0)
+          return -1;
         *q=(*p++);
         q+=4;
       }
     else
       {
+        if (npackets-- == 0)
+          return -1;
         pixel=(*p++);
         for ( ; count != 0; count--)
         {
@@ -211,6 +245,7 @@ static void SGIDecode(const unsigned long bytes_per_pixel,
         }
       }
   }
+  return 0;
 }
 
 static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
@@ -258,7 +293,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+    ThrowReaderException(FileOpenError,UnableToOpenFile,image);
   /*
     Read SGI raster header.
   */
@@ -269,37 +304,179 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Verify SGI identifier.
     */
     if (iris_info.magic != 0x01DA)
-      ThrowReaderException(CorruptImageError,"NotASGIImage",image);
+      ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
     iris_info.storage=ReadBlobByte(image);
     iris_info.bytes_per_pixel=ReadBlobByte(image);
     iris_info.dimension=ReadBlobMSBShort(image);
-    iris_info.columns=ReadBlobMSBShort(image);
-    iris_info.rows=ReadBlobMSBShort(image);
-    iris_info.depth=ReadBlobMSBShort(image);
-    image->columns=iris_info.columns;
-    image->rows=iris_info.rows;
-    image->depth=iris_info.depth <= 8 ? 8 : QuantumDepth;
-    if (iris_info.depth < 3)
+    iris_info.xsize=ReadBlobMSBShort(image);
+    iris_info.ysize=ReadBlobMSBShort(image);
+    iris_info.zsize=ReadBlobMSBShort(image);
+    iris_info.pix_min=ReadBlobMSBLong(image);
+    iris_info.pix_max=ReadBlobMSBLong(image);
+
+    (void) ReadBlob(image,(unsigned int) sizeof(iris_info.dummy1),
+                    iris_info.dummy1);
+    (void) ReadBlob(image,(unsigned int) sizeof(iris_info.image_name),
+                    iris_info.image_name);
+    iris_info.image_name[sizeof(iris_info.image_name)-1]=0;
+    iris_info.color_map=ReadBlobMSBLong(image);
+    (void) ReadBlob(image,(unsigned int) sizeof(iris_info.dummy2),
+                    iris_info.dummy2);
+
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "    Header: Storage=%u, BPC=%u, Dimension=%u, XSize=%u, YSize=%u, ZSize=%u, PixMin=%u, PixMax=%u, image_name=\"%.79s\", color_map=%u",
+                          (unsigned int) iris_info.storage,
+                          (unsigned int) iris_info.bytes_per_pixel,
+                          (unsigned int) iris_info.dimension,
+                          (unsigned int) iris_info.xsize,
+                          (unsigned int) iris_info.ysize,
+                          (unsigned int) iris_info.zsize,
+                          iris_info.pix_min,
+                          iris_info.pix_max,
+                          iris_info.image_name,
+                          iris_info.color_map);
+
+    /*
+      Validate image header and set image attributes.
+    */
+    if (iris_info.storage == 0)
       {
-        image->storage_class=PseudoClass;
-        image->colors=256;
+        /* Uncompressed */
+        image->compression=NoCompression;
       }
+    else if (iris_info.storage == 1)
+      {
+        /* RLE compressed */
+        image->compression=RLECompression;
+      }
+    else
+      {
+        /* Error */
+        ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+      }
+
+    if (iris_info.color_map != 0)
+      {
+        /* We only support images with normal (no) colormap */
+        ThrowReaderException(CorruptImageError,ImageTypeNotSupported,image);
+      }
+
+    if (iris_info.bytes_per_pixel == 1)
+      {
+        /* 8 bits per sample */
+        image->depth=8;
+      }
+    else if (iris_info.bytes_per_pixel == 2)
+      {
+        /* 16 bits per sample */
+        image->depth=Min(16,QuantumDepth);
+      }
+    else
+      {
+        /* Error */
+        ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+      }
+
+    if (iris_info.dimension == 1)
+      {
+        /*
+          Image contains one channel and one scanline, with scanline
+          length specified by xsize.
+        */
+        image->columns=iris_info.xsize;
+        image->rows=1;
+        image->is_grayscale=MagickTrue;
+        if (iris_info.bytes_per_pixel == 1)
+          {
+            /* Use a grayscale colormap */
+            image->storage_class=PseudoClass;
+            image->colors=256;
+          }
+      }
+    else if (iris_info.dimension == 2)
+      {
+        /*
+          One channel with a number of scan lines.  Width and height
+          of image are given by the values of xsize and ysize.
+        */
+        image->columns=iris_info.xsize;
+        image->rows=iris_info.ysize;
+        image->is_grayscale=MagickTrue;
+        if (iris_info.bytes_per_pixel == 1)
+          {
+            /* Use a grayscale colormap */
+            image->storage_class=PseudoClass;
+            image->colors=256;
+          }
+      }
+    else if (iris_info.dimension == 3)
+      {
+        /*
+          A number of channels.  Width and height of image are given
+          by the values of xsize and ysize. The number of channels is
+          specified by zsize.
+
+          B/W images have a zsize of 1.  RGB images have a zsize of 3.
+          RGB images with an alpha channel have a zsize of 4. Images
+          may have more than four channels but the meaning of
+          additional channels is implementation dependent.
+        */
+
+        image->columns=iris_info.xsize;
+        image->rows=iris_info.ysize;
+
+        if (iris_info.zsize == 1)
+          {
+            /* B/W image */
+            image->matte = MagickFalse;
+            image->is_grayscale=MagickTrue;
+            if (iris_info.bytes_per_pixel == 1)
+              {
+                /* Use a grayscale colormap */
+                image->storage_class=PseudoClass;
+                image->colors=256;
+              }
+          }
+        else if (iris_info.zsize == 3)
+          {
+            /* RGB */
+            image->matte=MagickFalse;
+          }
+        else if (iris_info.zsize == 4)
+          {
+            /* RGBA  */
+            image->matte=MagickTrue;
+          }
+        else
+          {
+            /* Error */
+            ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+          }
+      }
+    else
+      {
+        /* Error */
+        ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+      }
+
+    if (iris_info.image_name[0])
+      (void) SetImageAttribute(image,"comment",iris_info.image_name);
+
     if (image_info->ping && (image_info->subrange != 0))
       if (image->scene >= (image_info->subimage+image_info->subrange-1))
         break;
-    iris_info.minimum_value=ReadBlobMSBLong(image);
-    iris_info.maximum_value=ReadBlobMSBLong(image);
-    (void) ReadBlob(image,(unsigned int) sizeof(iris_info.filler),
-      (char *) iris_info.filler);
+
     /*
       Allocate SGI pixels.
     */
     bytes_per_pixel=iris_info.bytes_per_pixel;
-    number_pixels=iris_info.columns*iris_info.rows;
-    iris_pixels=(unsigned char *)
-      AcquireMemory(4*bytes_per_pixel*number_pixels);
+    number_pixels=iris_info.xsize*iris_info.ysize;
+    if (4*bytes_per_pixel*number_pixels < number_pixels) /* Overflow? */
+      ThrowReaderException(CorruptImageError,InsufficientImageDataInFile,image);
+    iris_pixels=MagickAllocateMemory(unsigned char *,
+      4*bytes_per_pixel*number_pixels);
     if (iris_pixels == (unsigned char *) NULL)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
     if (iris_info.storage != 0x01)
       {
         unsigned char
@@ -308,34 +485,44 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Read standard image format.
         */
-        scanline=(unsigned char *)
-          AcquireMemory(bytes_per_pixel*iris_info.columns);
+        scanline=MagickAllocateMemory(unsigned char *,
+          bytes_per_pixel*iris_info.xsize);
         if (scanline == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
+          ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
             image);
-        for (z=0; z < (int) iris_info.depth; z++)
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+            "   Reading SGI scanlines"); 
+        for (z=0; z < (int) iris_info.zsize; z++)
         {
           p=iris_pixels+bytes_per_pixel*z;
-          for (y=0; y < (long) iris_info.rows; y++)
+          for (y=0; y < (long) iris_info.ysize; y++)
           {
-            (void) ReadBlob(image,bytes_per_pixel*iris_info.columns,
+            (void) ReadBlob(image,bytes_per_pixel*iris_info.xsize,
               (char *) scanline);
+            if (EOFBlob(image))
+              {
+                ThrowReaderException(CorruptImageError,
+                  UnexpectedEndOfFile, image);
+                break;
+              }
             if (bytes_per_pixel == 2)
-              for (x=0; x < (long) iris_info.columns; x++)
+              for (x=0; x < (long) iris_info.xsize; x++)
               {
                 *p=scanline[2*x];
                 *(p+1)=scanline[2*x+1];
                 p+=8;
               }
             else
-              for (x=0; x < (long) iris_info.columns; x++)
+              for (x=0; x < (long) iris_info.xsize; x++)
               {
                 *p=scanline[x];
                 p+=4;
               }
           }
+          if (EOFBlob(image))
+            break;
         }
-        LiberateMemory((void **) &scanline);
+        MagickFreeMemory(scanline);
       }
     else
       {
@@ -353,83 +540,106 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Read runlength-encoded image format.
         */
-        offsets=(unsigned long *) AcquireMemory(iris_info.rows*
-          iris_info.depth*sizeof(unsigned long));
-        max_packets=(unsigned char *) AcquireMemory(4*iris_info.columns+10);
-        runlength=(unsigned long *) AcquireMemory(iris_info.rows*
-          iris_info.depth*sizeof(unsigned long));
+        offsets=MagickAllocateMemory(unsigned long *,iris_info.ysize*
+          iris_info.zsize*sizeof(unsigned long));
+        max_packets=MagickAllocateArray(unsigned char *,iris_info.xsize+10,4);
+        runlength=MagickAllocateMemory(unsigned long *,iris_info.ysize*
+          iris_info.zsize*sizeof(unsigned long));
         if ((offsets == (unsigned long *) NULL) ||
             (max_packets == (unsigned char *) NULL) ||
             (runlength == (unsigned long *) NULL))
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
+          ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
             image);
-        for (i=0; i < (int) (iris_info.rows*iris_info.depth); i++)
+        for (i=0; i < (int) (iris_info.ysize*iris_info.zsize); i++)
           offsets[i]=ReadBlobMSBLong(image);
-        for (i=0; i < (int) (iris_info.rows*iris_info.depth); i++)
+        for (i=0; i < (int) (iris_info.ysize*iris_info.zsize); i++)
+        {
           runlength[i]=ReadBlobMSBLong(image);
+          if (EOFBlob(image))
+            ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
+          if (runlength[i] > ((unsigned long) 4*iris_info.xsize+10))
+            ThrowReaderException(CorruptImageError,UnableToRunlengthDecodeImage,image);
+        }
         /*
           Check data order.
         */
         offset=0;
         data_order=0;
-        for (y=0; ((y < (long) iris_info.rows) && !data_order); y++)
-          for (z=0; ((z < (int) iris_info.depth) && !data_order); z++)
+        for (y=0; ((y < (long) iris_info.ysize) && !data_order); y++)
+          for (z=0; ((z < (int) iris_info.zsize) && !data_order); z++)
           {
-            if (offsets[y+z*iris_info.rows] < offset)
+            if (offsets[y+z*iris_info.ysize] < offset)
               data_order=1;
-            offset=offsets[y+z*iris_info.rows];
+            offset=offsets[y+z*iris_info.ysize];
           }
-        offset=512+4*bytes_per_pixel*2*(iris_info.rows*iris_info.depth);
+        offset=TellBlob(image);
         if (data_order == 1)
           {
-            for (z=0; z < (int) iris_info.depth; z++)
+            for (z=0; z < (int) iris_info.zsize; z++)
             {
               p=iris_pixels;
-              for (y=0; y < (long) iris_info.rows; y++)
+              for (y=0; y < (long) iris_info.ysize; y++)
               {
-                if (offset != offsets[y+z*iris_info.rows])
+                if (offset != offsets[y+z*iris_info.ysize])
                   {
-                    offset=offsets[y+z*iris_info.rows];
+                    offset=offsets[y+z*iris_info.ysize];
                     (void) SeekBlob(image,(long) offset,SEEK_SET);
                   }
-                (void) ReadBlob(image,runlength[y+z*iris_info.rows],
+                (void) ReadBlob(image,runlength[y+z*iris_info.ysize],
                   (char *) max_packets);
-                offset+=runlength[y+z*iris_info.rows];
-                SGIDecode(bytes_per_pixel,max_packets,p+bytes_per_pixel*z);
-                p+=(iris_info.columns*4*bytes_per_pixel);
+                if (EOFBlob(image))
+                  {
+                    ThrowReaderException(CorruptImageError,
+                      UnexpectedEndOfFile, image);
+                    break;
+                  }
+                offset+=runlength[y+z*iris_info.ysize];
+                if (SGIDecode(bytes_per_pixel,max_packets,p+bytes_per_pixel*z,
+		              runlength[y+z*iris_info.ysize]/bytes_per_pixel,
+                              iris_info.xsize) == -1)
+	          ThrowReaderException(CorruptImageError,
+                                       UnableToRunlengthDecodeImage,image); 
+                p+=(iris_info.xsize*4*bytes_per_pixel);
               }
             }
           }
         else
           {
             p=iris_pixels;
-            for (y=0; y < (long) iris_info.rows; y++)
+            for (y=0; y < (long) iris_info.ysize; y++)
             {
-              for (z=0; z < (int) iris_info.depth; z++)
+              for (z=0; z < (int) iris_info.zsize; z++)
               {
-                if (offset != offsets[y+z*iris_info.rows])
+                if (offset != offsets[y+z*iris_info.ysize])
                   {
-                    offset=offsets[y+z*iris_info.rows];
+                    offset=offsets[y+z*iris_info.ysize];
                     (void) SeekBlob(image,(long) offset,SEEK_SET);
                   }
-                (void) ReadBlob(image,runlength[y+z*iris_info.rows],
+                (void) ReadBlob(image,runlength[y+z*iris_info.ysize],
                   (char *) max_packets);
-                offset+=runlength[y+z*iris_info.rows];
-                SGIDecode(bytes_per_pixel,max_packets,p+bytes_per_pixel*z);
+                if (EOFBlob(image))
+                  {
+                    ThrowReaderException(CorruptImageError,
+                      UnexpectedEndOfFile, image);
+                    break;
+                  }
+                offset+=runlength[y+z*iris_info.ysize];
+                if (SGIDecode(bytes_per_pixel,max_packets,p+bytes_per_pixel*z,
+		              runlength[y+z*iris_info.ysize]/bytes_per_pixel,
+                              iris_info.xsize) == -1)
+	          ThrowReaderException(CorruptImageError,
+                                       UnableToRunlengthDecodeImage,image); 
               }
-              p+=(iris_info.columns*4*bytes_per_pixel);
+              p+=(iris_info.xsize*4*bytes_per_pixel);
+              if (EOFBlob(image))
+                break;
             }
           }
-        LiberateMemory((void **) &runlength);
-        LiberateMemory((void **) &max_packets);
-        LiberateMemory((void **) &offsets);
+        MagickFreeMemory(runlength);
+        MagickFreeMemory(max_packets);
+        MagickFreeMemory(offsets);
       }
-    /*
-      Initialize image structure.
-    */
-    image->matte=iris_info.depth == 4;
-    image->columns=iris_info.columns;
-    image->rows=iris_info.rows;
+
     /*
       Convert SGI raster image to pixel packets.
     */
@@ -451,8 +661,11 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 q->red=ScaleShortToQuantum((*(p+0) << 8) | (*(p+1)));
                 q->green=ScaleShortToQuantum((*(p+2) << 8) | (*(p+3)));
                 q->blue=ScaleShortToQuantum((*(p+4) << 8) | (*(p+5)));
-                q->opacity=(Quantum)
-                  (MaxRGB-ScaleShortToQuantum((*(p+6) << 8) | (*(p+7))));
+                if (image->matte)
+                  q->opacity=(Quantum)
+                    (MaxRGB-ScaleShortToQuantum((*(p+6) << 8) | (*(p+7))));
+                else
+                  q->opacity=OpaqueOpacity;
                 p+=8;
                 q++;
               }
@@ -460,7 +673,9 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 break;
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
-                  if (!MagickMonitor(LoadImageText,y,image->rows,exception))
+                  if (!MagickMonitorFormatted(y,image->rows,exception,
+                                              LoadImageText,image->filename,
+					      image->columns,image->rows))
                     break;
             }
           }
@@ -476,7 +691,10 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
               q->red=ScaleCharToQuantum(*p);
               q->green=ScaleCharToQuantum(*(p+1));
               q->blue=ScaleCharToQuantum(*(p+2));
-              q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(*(p+3)));
+              if (image->matte)
+                q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(*(p+3)));
+              else
+                q->opacity=OpaqueOpacity;
               p+=4;
               q++;
             }
@@ -484,7 +702,9 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
               break;
             if (image->previous == (Image *) NULL)
               if (QuantumTick(y,image->rows))
-                if (!MagickMonitor(LoadImageText,y,image->rows,exception))
+                if (!MagickMonitorFormatted(y,image->rows,exception,
+                                            LoadImageText,image->filename,
+					    image->columns,image->rows))
                   break;
           }
       }
@@ -494,7 +714,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Create grayscale map.
         */
         if (!AllocateImageColormap(image,image->colors))
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
+          ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
             image);
         /*
           Convert SGI image to PseudoClass pixel packets.
@@ -507,7 +727,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
               q=SetImagePixels(image,0,y,image->columns,1);
               if (q == (PixelPacket *) NULL)
                 break;
-              indexes=GetIndexes(image);
+              indexes=AccessMutableIndexes(image);
               for (x=0; x < (long) image->columns; x++)
               {
                 indexes[x]=(*p << 8);
@@ -519,7 +739,9 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 break;
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
-                  if (!MagickMonitor(LoadImageText,y,image->rows,exception))
+                  if (!MagickMonitorFormatted(y,image->rows,exception,
+                                              LoadImageText,image->filename,
+					      image->columns,image->rows))
                     break;
             }
           }
@@ -530,7 +752,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
             q=SetImagePixels(image,0,y,image->columns,1);
             if (q == (PixelPacket *) NULL)
               break;
-            indexes=GetIndexes(image);
+            indexes=AccessMutableIndexes(image);
             for (x=0; x < (long) image->columns; x++)
             {
               indexes[x]=(*p);
@@ -541,16 +763,18 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
               break;
             if (image->previous == (Image *) NULL)
               if (QuantumTick(y,image->rows))
-                if (!MagickMonitor(LoadImageText,y,image->rows,exception))
+                if (!MagickMonitorFormatted(y,image->rows,exception,
+                                            LoadImageText,image->filename,
+					    image->columns,image->rows))
                   break;
           }
-        SyncImage(image);
+        (void) SyncImage(image);
       }
-    LiberateMemory((void **) &iris_pixels);
+    MagickFreeMemory(iris_pixels);
     if (EOFBlob(image))
       {
-        ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
-          image->filename);
+        ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,
+          image);
         break;
       }
     /*
@@ -572,7 +796,8 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        if (!MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image),exception))
+        if (!MagickMonitorFormatted(TellBlob(image),GetBlobSize(image),exception,
+                                    LoadImagesText,image->filename))
           break;
       }
   } while (iris_info.magic == 0x01DA);
@@ -614,8 +839,8 @@ ModuleExport void RegisterSGIImage(void)
   entry->decoder=(DecoderHandler) ReadSGIImage;
   entry->encoder=(EncoderHandler) WriteSGIImage;
   entry->magick=(MagickHandler) IsSGI;
-  entry->description=AcquireString("Irix RGB image");
-  entry->module=AcquireString("SGI");
+  entry->description="Irix RGB image";
+  entry->module="SGI";
   entry->seekable_stream=True;
   (void) RegisterMagickInfo(entry);
 }
@@ -765,17 +990,29 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if ((image->columns > 65535L) || (image->rows > 65535L))
-    ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit",image);
+    ThrowWriterException(ImageError,WidthOrHeightExceedsLimit,image);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFail)
+    ThrowWriterException(FileOpenError,UnableToOpenFile,image);
   scene=0;
   do
   {
+    ImageCharacteristics
+      characteristics;
+
+    /*
+      Ensure that image is in an RGB space.
+    */
+    (void) TransformColorspace(image,RGBColorspace);
+    /*
+      Analyze image to be written.
+    */
+    (void) GetImageCharacteristics(image,&characteristics,
+                                   (OptimizeType == image_info->type),
+                                   &image->exception);
     /*
       Initialize SGI raster file header.
     */
-    TransformColorspace(image,RGBColorspace);
     iris_info.magic=0x01DA;
     if (image_info->compression == NoCompression)
       iris_info.storage=0x00;
@@ -783,19 +1020,36 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
       iris_info.storage=0x01;
     iris_info.bytes_per_pixel=1;  /* one byte per pixel */
     iris_info.dimension=3;
-    iris_info.columns=(unsigned short) image->columns;
-    iris_info.rows=(unsigned short) image->rows;
-    iris_info.depth=image->matte ? 4 : 3;
-    if ((image_info->type != TrueColorType) &&
-        IsGrayImage(image,&image->exception))
+    iris_info.xsize=(unsigned short) image->columns;
+    iris_info.ysize=(unsigned short) image->rows;
+    if (image->matte != MagickFalse)
+      iris_info.zsize=4;
+    else
       {
-        iris_info.dimension=2;
-        iris_info.depth=1;
+        if ((image_info->type != TrueColorType) &&
+            (characteristics.grayscale))
+          {
+            iris_info.dimension=2;
+            iris_info.zsize=1;
+          }
+        else
+          iris_info.zsize=3;
       }
-    iris_info.minimum_value=0;
-    iris_info.maximum_value=ScaleQuantumToChar(MaxRGB);
-    for (i=0; i < (int) sizeof(iris_info.filler); i++)
-      iris_info.filler[i]=0;
+    iris_info.pix_min=0;
+    iris_info.pix_max=ScaleQuantumToChar(MaxRGB);
+
+    (void) memset(iris_info.dummy1,0,sizeof(iris_info.dummy1));
+    {
+      const ImageAttribute
+        *attribute;
+
+      (void) memset(iris_info.image_name,0,sizeof(iris_info.image_name));
+      if ((attribute=GetImageAttribute(image,"comment")))
+        (void) strlcpy(iris_info.image_name,attribute->value,sizeof(iris_info.image_name));
+    }
+    iris_info.color_map=0;
+    (void) memset(iris_info.dummy2,0,sizeof(iris_info.dummy2));
+
     /*
       Write SGI header.
     */
@@ -803,20 +1057,25 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobByte(image,iris_info.storage);
     (void) WriteBlobByte(image,iris_info.bytes_per_pixel);
     (void) WriteBlobMSBShort(image,iris_info.dimension);
-    (void) WriteBlobMSBShort(image,iris_info.columns);
-    (void) WriteBlobMSBShort(image,iris_info.rows);
-    (void) WriteBlobMSBShort(image,iris_info.depth);
-    (void) WriteBlobMSBLong(image,iris_info.minimum_value);
-    (void) WriteBlobMSBLong(image,iris_info.maximum_value);
-    (void) WriteBlob(image,sizeof(iris_info.filler),
-      (char *) iris_info.filler);
+    (void) WriteBlobMSBShort(image,iris_info.xsize);
+    (void) WriteBlobMSBShort(image,iris_info.ysize);
+    (void) WriteBlobMSBShort(image,iris_info.zsize);
+    (void) WriteBlobMSBLong(image,iris_info.pix_min);
+    (void) WriteBlobMSBLong(image,iris_info.pix_max);
+    (void) WriteBlob(image,sizeof(iris_info.dummy1),
+                     (char *) iris_info.dummy1);
+    (void) WriteBlob(image,sizeof(iris_info.image_name),
+                     (char *) iris_info.image_name);
+    (void) WriteBlobMSBLong(image,iris_info.color_map);
+    (void) WriteBlob(image,sizeof(iris_info.dummy2),
+      (char *) iris_info.dummy2);
     /*
       Allocate SGI pixels.
     */
     number_pixels=image->columns*image->rows;
-    iris_pixels=(unsigned char *) AcquireMemory(4*number_pixels);
+    iris_pixels=MagickAllocateMemory(unsigned char *,4*number_pixels);
     if (iris_pixels == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
     /*
       Convert image pixels to uncompressed SGI pixels.
     */
@@ -825,7 +1084,7 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
       p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
       if (p == (const PixelPacket *) NULL)
         break;
-      q=iris_pixels+((iris_info.rows-1)-y)*(iris_info.columns*4);
+      q=iris_pixels+((iris_info.ysize-1)-y)*(iris_info.xsize*4);
       for (x=0; x < (long) image->columns; x++)
       {
         *q++=ScaleQuantumToChar(p->red);
@@ -836,7 +1095,9 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
       }
       if (image->previous == (Image *) NULL)
         if (QuantumTick(y,image->rows))
-          if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+          if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                      SaveImageText,image->filename,
+				      image->columns,image->rows))
             break;
     }
     if (image_info->compression == NoCompression)
@@ -847,24 +1108,24 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
         /*
           Write uncompressed SGI pixels.
         */
-        scanline=(unsigned char *) AcquireMemory(iris_info.columns);
+        scanline=MagickAllocateMemory(unsigned char *,iris_info.xsize);
         if (scanline == (unsigned char *) NULL)
-          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
+          ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,
             image);
-        for (z=0; z < (int) iris_info.depth; z++)
+        for (z=0; z < (int) iris_info.zsize; z++)
         {
           q=iris_pixels+z;
-          for (y=0; y < (long) iris_info.rows; y++)
+          for (y=0; y < (long) iris_info.ysize; y++)
           {
-            for (x=0; x < (long) iris_info.columns; x++)
+            for (x=0; x < (long) iris_info.xsize; x++)
             {
               scanline[x]=(*q);
               q+=4;
             }
-            (void) WriteBlob(image,iris_info.columns,(char *) scanline);
+            (void) WriteBlob(image,iris_info.xsize,(char *) scanline);
           }
         }
-        LiberateMemory((void **) &scanline);
+        MagickFreeMemory(scanline);
       }
     else
       {
@@ -878,53 +1139,55 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
         /*
           Convert SGI uncompressed pixels.
         */
-        offsets=(unsigned long *) AcquireMemory(iris_info.rows*
-          iris_info.depth*sizeof(unsigned long));
-        packets=(unsigned char *)
-          AcquireMemory(4*(2*iris_info.columns+10)*image->rows);
-        runlength=(unsigned long *) AcquireMemory(iris_info.rows*
-          iris_info.depth*sizeof(unsigned long));
+        offsets=MagickAllocateMemory(unsigned long *,iris_info.ysize*
+          iris_info.zsize*sizeof(unsigned long));
+        packets=MagickAllocateMemory(unsigned char *,
+          4*(2*iris_info.xsize+10)*image->rows);
+        runlength=MagickAllocateMemory(unsigned long *,iris_info.ysize*
+          iris_info.zsize*sizeof(unsigned long));
         if ((offsets == (unsigned long *) NULL) ||
             (packets == (unsigned char *) NULL) ||
             (runlength == (unsigned long *) NULL))
-          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
+          ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,
             image);
-        offset=512+4*2*(iris_info.rows*iris_info.depth);
+        offset=512+4*2*(iris_info.ysize*iris_info.zsize);
         number_packets=0;
         q=iris_pixels;
-        for (y=0; y < (long) iris_info.rows; y++)
+        for (y=0; y < (long) iris_info.ysize; y++)
         {
-          for (z=0; z < (int) iris_info.depth; z++)
+          for (z=0; z < (int) iris_info.zsize; z++)
           {
             length=
-              SGIEncode(q+z,(int) iris_info.columns,packets+number_packets);
+              SGIEncode(q+z,(int) iris_info.xsize,packets+number_packets);
             number_packets+=length;
-            offsets[y+z*iris_info.rows]=offset;
-            runlength[y+z*iris_info.rows]=length;
+            offsets[y+z*iris_info.ysize]=offset;
+            runlength[y+z*iris_info.ysize]=length;
             offset+=length;
           }
-          q+=(iris_info.columns*4);
+          q+=(iris_info.xsize*4);
         }
         /*
           Write out line start and length tables and runlength-encoded pixels.
         */
-        for (i=0; i < (int) (iris_info.rows*iris_info.depth); i++)
+        for (i=0; i < (int) (iris_info.ysize*iris_info.zsize); i++)
           (void) WriteBlobMSBLong(image,offsets[i]);
-        for (i=0; i < (int) (iris_info.rows*iris_info.depth); i++)
+        for (i=0; i < (int) (iris_info.ysize*iris_info.zsize); i++)
           (void) WriteBlobMSBLong(image,runlength[i]);
         (void) WriteBlob(image,number_packets,(char *) packets);
         /*
           Free memory.
         */
-        LiberateMemory((void **) &runlength);
-        LiberateMemory((void **) &packets);
-        LiberateMemory((void **) &offsets);
+        MagickFreeMemory(runlength);
+        MagickFreeMemory(packets);
+        MagickFreeMemory(offsets);
       }
-    LiberateMemory((void **) &iris_pixels);
+    MagickFreeMemory(iris_pixels);
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    if (!MagickMonitor(SaveImagesText,scene++,GetImageListLength(image),&image->exception))
+    if (!MagickMonitorFormatted(scene++,GetImageListLength(image),
+                                &image->exception,SaveImagesText,
+                                image->filename))
       break;
   } while (image_info->adjoin);
   if (image_info->adjoin)
