@@ -37,10 +37,10 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/cache.h"
-#include "magick/color.h"
+#include "magick/colormap.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
+#include "magick/pixel_cache.h"
 #include "magick/utility.h"
 
 /*
@@ -146,7 +146,7 @@ static unsigned int DecodeImage(const unsigned char *compressed_pixels,
   while ((size_t) (p-compressed_pixels) < length)
   {
     byte=(*p++);
-    if (byte != 128)
+    if (byte != 128U)
       *q++=byte;
     else
       {
@@ -208,7 +208,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   typedef struct _SUNInfo
   {
-    unsigned long
+    magick_uint32_t
       magic,
       width,
       height,
@@ -273,6 +273,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Read SUN raster header.
   */
+  (void) memset(&sun_info,0,sizeof(sun_info));
   sun_info.magic=ReadBlobMSBLong(image);
   do
   {
@@ -366,18 +367,22 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->matte=(sun_info.depth == 32);
     image->columns=sun_info.width;
     image->rows=sun_info.height;
+    image->depth=8;
+    if (sun_info.depth < 8)
+      image->depth=sun_info.depth;
     if (image_info->ping)
       {
         CloseBlob(image);
         return(image);
       }
-    sun_data=MagickAllocateMemory(unsigned char *,sun_info.length);
+    sun_data=MagickAllocateMemory(unsigned char *,(size_t) sun_info.length);
     if (sun_data == (unsigned char *) NULL)
       ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
     count=ReadBlob(image,sun_info.length,(char *) sun_data);
     if ((count == 0) && (sun_info.type != RT_ENCODED))
       ThrowReaderException(CorruptImageError,UnableToReadImageData,image);
     sun_pixels=sun_data;
+    bytes_per_line=0;
     if (sun_info.type == RT_ENCODED)
       {
         unsigned long
@@ -405,7 +410,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
         q=SetImagePixels(image,0,y,image->columns,1);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
+        indexes=AccessMutableIndexes(image);
         for (x=0; x < ((long) image->columns-7); x+=8)
         {
           for (bit=7; bit >= 0; bit--)
@@ -424,7 +429,9 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
           break;
         if (image->previous == (Image *) NULL)
           if (QuantumTick(y,image->rows))
-            if (!MagickMonitor(LoadImageText,y,image->rows,exception))
+            if (!MagickMonitorFormatted(y,image->rows,exception,
+                                        LoadImageText,image->filename,
+					image->columns,image->rows))
               break;
       }
     else
@@ -439,7 +446,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
             q=SetImagePixels(image,0,y,image->columns,1);
             if (q == (PixelPacket *) NULL)
               break;
-            indexes=GetIndexes(image);
+            indexes=AccessMutableIndexes(image);
             for (x=0; x < (long) image->columns; x++)
               indexes[x]=(*p++);
             if ((image->columns % 2) != 0)
@@ -448,7 +455,9 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
               break;
             if (image->previous == (Image *) NULL)
               if (QuantumTick(y,image->rows))
-                if (!MagickMonitor(LoadImageText,y,image->rows,exception))
+                if (!MagickMonitorFormatted(y,image->rows,exception,
+                                            LoadImageText,image->filename,
+					    image->columns,image->rows))
                   break;
           }
 	}
@@ -494,12 +503,14 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
               break;
             if (image->previous == (Image *) NULL)
               if (QuantumTick(y,image->rows))
-                if (!MagickMonitor(LoadImageText,y,image->rows,exception))
+                if (!MagickMonitorFormatted(y,image->rows,exception,
+                                            LoadImageText,image->filename,
+					    image->columns,image->rows))
                   break;
           }
 	}
     if (image->storage_class == PseudoClass)
-      SyncImage(image);
+      (void) SyncImage(image);
     MagickFreeMemory(sun_pixels);
     if (EOFBlob(image))
       {
@@ -526,7 +537,9 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        if (!MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image),exception))
+        if (!MagickMonitorFormatted(TellBlob(image),GetBlobSize(image),
+                                    exception,LoadImagesText,
+                                    image->filename))
           break;
       }
   } while (sun_info.magic == 0x59a66a95);
@@ -568,14 +581,15 @@ ModuleExport void RegisterSUNImage(void)
   entry->decoder=(DecoderHandler) ReadSUNImage;
   entry->encoder=(EncoderHandler) WriteSUNImage;
   entry->magick=(MagickHandler) IsSUN;
-  entry->description=AcquireString("SUN Rasterfile");
-  entry->module=AcquireString("SUN");
+  entry->description="SUN Rasterfile";
+  entry->module="SUN";
   (void) RegisterMagickInfo(entry);
+
   entry=SetMagickInfo("SUN");
   entry->decoder=(DecoderHandler) ReadSUNImage;
   entry->encoder=(EncoderHandler) WriteSUNImage;
-  entry->description=AcquireString("SUN Rasterfile");
-  entry->module=AcquireString("SUN");
+  entry->description="SUN Rasterfile";
+  entry->module="SUN";
   (void) RegisterMagickInfo(entry);
 }
 
@@ -643,7 +657,7 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
 
   typedef struct _SUNInfo
   {
-    unsigned long
+    magick_uint32_t
       magic,
       width,
       height,
@@ -660,7 +674,7 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
-  register IndexPacket
+  register const IndexPacket
     *indexes;
 
   register long
@@ -692,10 +706,26 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
   scene=0;
   do
   {
+    ImageCharacteristics
+      characteristics;
+
+    /*
+      Ensure that image is in an RGB space.
+    */
+    (void) TransformColorspace(image,RGBColorspace);
+    /*
+      Analyze image to be written.
+    */
+    if (!GetImageCharacteristics(image,&characteristics,
+                                 (OptimizeType == image_info->type),
+                                 &image->exception))
+      {
+        CloseBlob(image);
+        return MagickFail;
+      }
     /*
       Initialize SUN raster file header.
     */
-    TransformColorspace(image,RGBColorspace);
     sun_info.magic=0x59a66a95;
     sun_info.width=image->columns;
     sun_info.height=image->rows;
@@ -709,20 +739,20 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
         /*
           Full color SUN raster.
         */
-        sun_info.depth=(image->matte ? 32 : 24);
-        sun_info.length=(image->matte ? 4 : 3)*number_pixels;
-        sun_info.length+=image->columns & 0x01 ? image->rows : 0;
+        sun_info.depth=(image->matte ? 32U : 24U);
+        sun_info.length=(image->matte ? 4U : 3U)*number_pixels;
+        sun_info.length+=image->columns & 0x01U ? image->rows : 0U;
       }
     else
-      if (IsMonochromeImage(image,&image->exception))
+      if (characteristics.monochrome)
         {
           /*
             Monochrome SUN raster.
           */
           sun_info.depth=1;
-          sun_info.length=((image->columns+7) >> 3)*image->rows;
-          sun_info.length+=((image->columns/8)+(image->columns % 8 ? 1 : 0)) %
-            2 ? image->rows : 0;
+          sun_info.length=((image->columns+7U) >> 3)*image->rows;
+          sun_info.length+=((image->columns/8U)+(image->columns % 8U ? 1U : 0U)) %
+            2U ? image->rows : 0U;
         }
       else
         {
@@ -731,7 +761,7 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
           */
           sun_info.depth=8;
           sun_info.length=number_pixels;
-          sun_info.length+=image->columns & 0x01 ? image->rows : 0;
+          sun_info.length+=image->columns & 0x01U ? image->rows : 0;
           sun_info.maptype=RMT_EQUAL_RGB;
           sun_info.maplength=image->colors*3;
         }
@@ -793,13 +823,15 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
           (void) WriteBlob(image,q-pixels,(char *) pixels);
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+              if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                          SaveImageText,image->filename,
+					  image->columns,image->rows))
                 break;
         }
         MagickFreeMemory(pixels);
       }
     else
-      if (IsMonochromeImage(image,&image->exception))
+      if (characteristics.monochrome)
         {
           register unsigned char
             bit,
@@ -809,7 +841,7 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
           /*
             Convert PseudoClass image to a SUN monochrome image.
           */
-          SetImageType(image,BilevelType);
+          (void) SetImageType(image,BilevelType);
           polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
           if (image->colors == 2)
             polarity=PixelIntensityToQuantum(&image->colormap[0]) <
@@ -819,7 +851,7 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
             p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
             if (p == (const PixelPacket *) NULL)
               break;
-            indexes=GetIndexes(image);
+            indexes=AccessImmutableIndexes(image);
             bit=0;
             byte=0;
             for (x=0; x < (long) image->columns; x++)
@@ -843,7 +875,9 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
               (void) WriteBlobByte(image,0);  /* pad scanline */
             if (image->previous == (Image *) NULL)
               if (QuantumTick(y,image->rows))
-                if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+                if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                            SaveImageText,image->filename,
+					    image->columns,image->rows))
                   break;
           }
         }
@@ -866,7 +900,7 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
             p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
             if (p == (const PixelPacket *) NULL)
               break;
-            indexes=GetIndexes(image);
+            indexes=AccessImmutableIndexes(image);
             for (x=0; x < (long) image->columns; x++)
             {
               (void) WriteBlobByte(image,indexes[x]);
@@ -876,14 +910,18 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
               (void) WriteBlobByte(image,0);  /* pad scanline */
             if (image->previous == (Image *) NULL)
               if (QuantumTick(y,image->rows))
-                if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+                if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                            SaveImageText,image->filename,
+					    image->columns,image->rows))
                   break;
           }
         }
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    if (!MagickMonitor(SaveImagesText,scene++,GetImageListLength(image),&image->exception))
+    if (!MagickMonitorFormatted(scene++,GetImageListLength(image),
+                                &image->exception,SaveImagesText,
+                                image->filename))
       break;
   } while (image_info->adjoin);
   if (image_info->adjoin)
