@@ -18,7 +18,7 @@
 %                          V    IIIII  F      F                               %
 %                                                                             %
 %                                                                             %
-%                   Read/Write GraphicsMagick Image Format.                   %
+%               Read/Write Khoros Visualization Image Format.                 %
 %                                                                             %
 %                                                                             %
 %                              Software Design                                %
@@ -38,10 +38,10 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
-#include "magick/cache.h"
-#include "magick/color.h"
+#include "magick/colormap.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
+#include "magick/pixel_cache.h"
 #include "magick/utility.h"
 
 /*
@@ -247,7 +247,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+    ThrowReaderException(FileOpenError,UnableToOpenFile,image);
   /*
     Read VIFF header (1024 bytes).
   */
@@ -257,8 +257,8 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
     /*
       Verify VIFF identifier.
     */
-    if ((count == 0) || ((unsigned char) viff_info.identifier != 0xab))
-      ThrowReaderException(CorruptImageError,"NotAVIFFImage",image);
+    if ((count == 0) || ((unsigned char) viff_info.identifier != 0xabU))
+      ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
     /*
       Initialize VIFF image.
     */
@@ -330,32 +330,34 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
     */
     number_pixels=viff_info.columns*viff_info.rows;
     if (number_pixels == 0)
-      ThrowReaderException(CoderError,"ImageColumnOrRowSizeIsNotSupported",
+      ThrowReaderException(CoderError,ImageColumnOrRowSizeIsNotSupported,
         image);
+    if (viff_info.number_data_bands < 1 || viff_info.number_data_bands > 4)
+      ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
     if ((viff_info.data_storage_type != VFF_TYP_BIT) &&
         (viff_info.data_storage_type != VFF_TYP_1_BYTE) &&
         (viff_info.data_storage_type != VFF_TYP_2_BYTE) &&
         (viff_info.data_storage_type != VFF_TYP_4_BYTE) &&
         (viff_info.data_storage_type != VFF_TYP_FLOAT) &&
         (viff_info.data_storage_type != VFF_TYP_DOUBLE))
-      ThrowReaderException(CoderError,"DataStorageTypeIsNotSupported",image);
+      ThrowReaderException(CoderError,DataStorageTypeIsNotSupported,image);
     if (viff_info.data_encode_scheme != VFF_DES_RAW)
-      ThrowReaderException(CoderError,"DataEncodingSchemeIsNotSupported",image);
+      ThrowReaderException(CoderError,DataEncodingSchemeIsNotSupported,image);
     if ((viff_info.map_storage_type != VFF_MAPTYP_NONE) &&
         (viff_info.map_storage_type != VFF_MAPTYP_1_BYTE) &&
         (viff_info.map_storage_type != VFF_MAPTYP_2_BYTE) &&
         (viff_info.map_storage_type != VFF_MAPTYP_4_BYTE) &&
         (viff_info.map_storage_type != VFF_MAPTYP_FLOAT) &&
         (viff_info.map_storage_type != VFF_MAPTYP_DOUBLE))
-      ThrowReaderException(CoderError,"MapStorageTypeIsNotSupported",image);
+      ThrowReaderException(CoderError,MapStorageTypeIsNotSupported,image);
     if ((viff_info.color_space_model != VFF_CM_NONE) &&
         (viff_info.color_space_model != VFF_CM_ntscRGB) &&
         (viff_info.color_space_model != VFF_CM_genericRGB))
-      ThrowReaderException(CoderError,"ColorspaceModelIsNotSupported",image);
+      ThrowReaderException(CoderError,ColorspaceModelIsNotSupported,image);
     if (viff_info.location_type != VFF_LOC_IMPLICIT)
-      ThrowReaderException(CoderError,"LocationTypeIsNotSupported",image);
+      ThrowReaderException(CoderError,LocationTypeIsNotSupported,image);
     if (viff_info.number_of_images != 1)
-      ThrowReaderException(CoderError,"NumberOfImagesIsNotSupported",image);
+      ThrowReaderException(CoderError,NumberOfImagesIsNotSupported,image);
     if (viff_info.map_rows == 0)
       viff_info.map_scheme=VFF_MS_NONE;
     switch ((int) viff_info.map_scheme)
@@ -371,7 +373,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
             if (viff_info.data_storage_type == VFF_TYP_BIT)
               image->colors=2;
             if (!AllocateImageColormap(image,image->colors))
-              ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
+              ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
                 image);
           }
         break;
@@ -396,12 +398,12 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
         }
         image->colors=viff_info.map_columns;
         if (!AllocateImageColormap(image,image->colors))
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
+          ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
             image);
-        viff_colormap=(unsigned char *)
-          AcquireMemory(bytes_per_pixel*image->colors*viff_info.map_rows);
+        viff_colormap=MagickAllocateMemory(unsigned char *,
+          bytes_per_pixel*image->colors*viff_info.map_rows);
         if (viff_colormap == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
+          ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
             image);
         /*
           Read VIFF raster colormap.
@@ -454,11 +456,11 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
                 image->colormap[i % image->colors].blue=
                   ScaleCharToQuantum((unsigned long) value);
         }
-        LiberateMemory((void **) &viff_colormap);
+        MagickFreeMemory(viff_colormap);
         break;
       }
       default:
-        ThrowReaderException(CoderError,"ColormapTypeIsNotSupported",image)
+        ThrowReaderException(CoderError,ColormapTypeNotSupported,image)
     }
     /*
       Initialize image structure.
@@ -486,10 +488,10 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
       max_packets=((image->columns+7) >> 3)*image->rows;
     else
       max_packets=number_pixels*viff_info.number_data_bands;
-    viff_pixels=(unsigned char *)
-      AcquireMemory(bytes_per_pixel*max_packets*sizeof(Quantum));
+    viff_pixels=MagickAllocateMemory(unsigned char *,
+      bytes_per_pixel*max_packets*sizeof(Quantum));
     if (viff_pixels == (unsigned char *) NULL)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
     (void) ReadBlob(image,bytes_per_pixel*max_packets,(char *) viff_pixels);
     lsb_first=1;
     if (*(char *) &lsb_first &&
@@ -596,7 +598,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
         /*
           Convert bitmap scanline.
         */
-        SetImageType(image,BilevelType);
+        (void) SetImageType(image,BilevelType);
         polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
         if (image->colors >= 2)
           polarity=PixelIntensityToQuantum(&image->colormap[0]) >
@@ -606,7 +608,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          indexes=GetIndexes(image);
+          indexes=AccessMutableIndexes(image);
           for (x=0; x < (long) (image->columns-7); x+=8)
           {
             for (bit=0; bit < 8; bit++)
@@ -625,7 +627,9 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(LoadImageText,y,image->rows,exception))
+              if (!MagickMonitorFormatted(y,image->rows,exception,
+                                          LoadImageText,image->filename,
+					  image->columns,image->rows))
                 break;
         }
       }
@@ -636,14 +640,16 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          indexes=GetIndexes(image);
+          indexes=AccessMutableIndexes(image);
           for (x=0; x < (long) image->columns; x++)
             indexes[x]=(*p++);
           if (!SyncImagePixels(image))
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(LoadImageText,y,image->rows,exception))
+              if (!MagickMonitorFormatted(y,image->rows,exception,
+                                          LoadImageText,image->filename,
+					  image->columns,image->rows))
                 break;
         }
       else
@@ -677,16 +683,18 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
               break;
             if (image->previous == (Image *) NULL)
               if (QuantumTick(y,image->rows))
-                if (!MagickMonitor(LoadImageText,y,image->rows,exception))
+                if (!MagickMonitorFormatted(y,image->rows,exception,
+                                            LoadImageText,image->filename,
+					    image->columns,image->rows))
                   break;
           }
         }
-    LiberateMemory((void **) &viff_pixels);
+    MagickFreeMemory(viff_pixels);
     if (image->storage_class == PseudoClass)
-      SyncImage(image);
+      (void) SyncImage(image);
     if (EOFBlob(image))
       {
-        ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+        ThrowException(exception,CorruptImageError,UnexpectedEndOfFile,
           image->filename);
         break;
       }
@@ -709,7 +717,9 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        if (!MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image),exception))
+        if (!MagickMonitorFormatted(TellBlob(image),GetBlobSize(image),
+                                    exception,LoadImagesText,
+                                    image->filename))
           break;
       }
   } while ((count != 0) && (viff_info.identifier == 0xab));
@@ -751,15 +761,15 @@ ModuleExport void RegisterVIFFImage(void)
   entry->decoder=(DecoderHandler) ReadVIFFImage;
   entry->encoder=(EncoderHandler) WriteVIFFImage;
   entry->magick=(MagickHandler) IsVIFF;
-  entry->description=AcquireString("Khoros Visualization image");
-  entry->module=AcquireString("VIFF");
+  entry->description="Khoros Visualization image";
+  entry->module="VIFF";
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("XV");
   entry->decoder=(DecoderHandler) ReadVIFFImage;
   entry->encoder=(EncoderHandler) WriteVIFFImage;
-  entry->description=AcquireString("Khoros Visualization image");
-  entry->module=AcquireString("VIFF");
+  entry->description="Khoros Visualization image";
+  entry->module="VIFF";
   (void) RegisterMagickInfo(entry);
 }
 
@@ -817,8 +827,6 @@ ModuleExport void UnregisterVIFFImage(void)
 %
 %
 */
-static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
-{
 #define VFF_CM_genericRGB  15
 #define VFF_CM_NONE  0
 #define VFF_DEP_IEEEORDER  0x2
@@ -830,6 +838,8 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
 #define VFF_MS_ONEPERBAND  1
 #define VFF_TYP_BIT  0
 #define VFF_TYP_1_BYTE  1
+static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
+{
 
   typedef struct _ViffInfo
   {
@@ -881,7 +891,7 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
-  register IndexPacket
+  register const IndexPacket
     *indexes;
 
   register long
@@ -917,15 +927,27 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
   assert(image->signature == MagickSignature);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
-  memset(&viff_info,0,sizeof(ViffInfo));
+    ThrowWriterException(FileOpenError,UnableToOpenFile,image);
+  (void) memset(&viff_info,0,sizeof(ViffInfo));
   scene=0;
   do
   {
+    ImageCharacteristics
+      characteristics;
+
+    /*
+      Ensure that image is in an RGB space.
+    */
+    (void) TransformColorspace(image,RGBColorspace);
+    /*
+      Analyze image to be written.
+    */
+    (void) GetImageCharacteristics(image,&characteristics,
+                                   (OptimizeType == image_info->type),
+                                   &image->exception);
     /*
       Initialize VIFF image structure.
     */
-    TransformColorspace(image,RGBColorspace);
     viff_info.identifier=(char) 0xab;
     viff_info.file_type=1;
     viff_info.release=1;
@@ -934,11 +956,8 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
     *viff_info.comment='\0';
     attribute=GetImageAttribute(image,"comment");
     if (attribute != (const ImageAttribute *) NULL)
-      {
-        (void) strncpy(viff_info.comment,attribute->value,
-          Min(strlen(attribute->value),511));
-        viff_info.comment[Min(strlen(attribute->value),511)]='\0';
-      }
+      (void) strlcpy(viff_info.comment,attribute->value,
+                     sizeof(viff_info.comment));
     viff_info.rows=image->columns;
     viff_info.columns=image->rows;
     viff_info.subrows=0;
@@ -974,7 +993,7 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
         viff_info.color_space_model=VFF_CM_NONE;
         viff_info.data_storage_type=VFF_TYP_1_BYTE;
         packets=number_pixels;
-        if (!IsGrayImage(image,&image->exception))
+        if (!characteristics.grayscale)
           {
             /*
               Colormapped VIFF raster.
@@ -1035,9 +1054,9 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
     /*
       Convert MIFF to VIFF raster pixels.
     */
-    viff_pixels=(unsigned char *) AcquireMemory(packets);
+    viff_pixels=MagickAllocateMemory(unsigned char *,packets);
     if (viff_pixels == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
     q=viff_pixels;
     if (image->storage_class == DirectClass)
       {
@@ -1062,12 +1081,14 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
           }
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+              if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                          SaveImageText,image->filename,
+					  image->columns,image->rows))
                 break;
         }
       }
     else
-      if (!IsGrayImage(image,&image->exception))
+      if (!characteristics.grayscale)
         {
           unsigned char
             *viff_colormap;
@@ -1075,9 +1096,9 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
           /*
             Dump colormap to file.
           */
-          viff_colormap=(unsigned char *) AcquireMemory(3*image->colors);
+          viff_colormap=MagickAllocateMemory(unsigned char *,3*image->colors);
           if (viff_colormap == (unsigned char *) NULL)
-            ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
+            ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,
               image);
           q=viff_colormap;
           for (i=0; i < (long) image->colors; i++)
@@ -1087,7 +1108,7 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
           for (i=0; i < (long) image->colors; i++)
             *q++=ScaleQuantumToChar(image->colormap[i].blue);
           (void) WriteBlob(image,3*image->colors,(char *) viff_colormap);
-          LiberateMemory((void **) &viff_colormap);
+          MagickFreeMemory(viff_colormap);
           /*
             Convert PseudoClass packet to VIFF colormapped pixels.
           */
@@ -1097,12 +1118,14 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
             p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
             if (p == (const PixelPacket *) NULL)
               break;
-            indexes=GetIndexes(image);
+            indexes=AccessImmutableIndexes(image);
             for (x=0; x < (long) image->columns; x++)
               *q++=indexes[x];
             if (image->previous == (Image *) NULL)
               if (QuantumTick(y,image->rows))
-                if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+                if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                            SaveImageText,image->filename,
+					    image->columns,image->rows))
                   break;
           }
         }
@@ -1121,7 +1144,7 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
             /*
               Convert PseudoClass image to a VIFF monochrome image.
             */
-            SetImageType(image,BilevelType);
+            (void) SetImageType(image,BilevelType);
             polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
             if (image->colors == 2)
               polarity=PixelIntensityToQuantum(&image->colormap[0]) <
@@ -1132,7 +1155,7 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              indexes=GetIndexes(image);
+              indexes=AccessImmutableIndexes(image);
               bit=0;
               byte=0;
               for (x=0; x < (long) image->columns; x++)
@@ -1152,7 +1175,9 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
                 *q++=byte >> (8-bit);
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
-                  if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+                  if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                              SaveImageText,image->filename,
+					      image->columns,image->rows))
                     break;
             }
           }
@@ -1174,17 +1199,20 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
               }
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
-                  if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+                  if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+					      SaveImageText,image->filename,
+					      image->columns,image->rows))
                     break;
             }
           }
     (void) WriteBlob(image,packets,(char *) viff_pixels);
-    LiberateMemory((void **) &viff_pixels);
+    MagickFreeMemory(viff_pixels);
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesText,scene++,GetImageListLength(image),
-      &image->exception);
+    status=MagickMonitorFormatted(scene++,GetImageListLength(image),
+                                  &image->exception,SaveImagesText,
+                                  image->filename);
     if (status == False)
       break;
   } while (image_info->adjoin);
