@@ -37,10 +37,11 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/pixel_cache.h"
 #include "magick/color.h"
+#include "magick/colormap.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
+#include "magick/pixel_cache.h"
 #include "magick/quantize.h"
 #include "magick/resize.h"
 #include "magick/utility.h"
@@ -362,7 +363,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             CopyException(exception,&image->exception);
             break;
           }
-        indexes=GetIndexes(image);
+        indexes=AccessMutableIndexes(image);
         for (x=0; x < (long) image->columns; x++)
         {
           (void) strncpy(key,p,width);
@@ -432,8 +433,8 @@ ModuleExport void RegisterXPMImage(void)
   entry->decoder=(DecoderHandler) ReadXPMImage;
   entry->encoder=(EncoderHandler) WritePICONImage;
   entry->adjoin=False;
-  entry->description=AcquireString("Personal Icon");
-  entry->module=AcquireString("XPM");
+  entry->description="Personal Icon";
+  entry->module="XPM";
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("PM");
@@ -441,8 +442,8 @@ ModuleExport void RegisterXPMImage(void)
   entry->encoder=(EncoderHandler) WriteXPMImage;
   entry->adjoin=False;
   entry->stealth=True;
-  entry->description=AcquireString("X Windows system pixmap (color)");
-  entry->module=AcquireString("XPM");
+  entry->description="X Windows system pixmap (color)";
+  entry->module="XPM";
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("XPM");
@@ -450,8 +451,8 @@ ModuleExport void RegisterXPMImage(void)
   entry->encoder=(EncoderHandler) WriteXPMImage;
   entry->magick=(MagickHandler) IsXPM;
   entry->adjoin=False;
-  entry->description=AcquireString("X Windows system pixmap (color)");
-  entry->module=AcquireString("XPM");
+  entry->description="X Windows system pixmap (color)";
+  entry->module="XPM";
   (void) RegisterMagickInfo(entry);
 }
 
@@ -575,9 +576,6 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
-  register IndexPacket
-    *indexes;
-
   register long
     i,
     x;
@@ -676,10 +674,13 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
       MagickReallocMemory(PixelPacket *,picon->colormap,colors*sizeof(PixelPacket));
       for (y=0; y < (long) picon->rows; y++)
       {
+        register IndexPacket
+          *indexes;
+
         q=GetImagePixels(picon,0,y,picon->columns,1);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(picon);
+        indexes=AccessMutableIndexes(picon);
         for (x=0; x < (long) picon->columns; x++)
         {
           if (q->opacity == TransparentOpacity)
@@ -740,10 +741,13 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlobString(image,"/* pixels */\n");
   for (y=0; y < (long) picon->rows; y++)
   {
+    register const IndexPacket
+      *indexes;
+
     p=AcquireImagePixels(picon,0,y,picon->columns,1,&picon->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(picon);
+    indexes=AccessImmutableIndexes(picon);
     (void) WriteBlobString(image,"\"");
     for (x=0; x < (long) picon->columns; x++)
     {
@@ -762,7 +766,9 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
       (y == (long) (picon->rows-1) ? "" : ","));
     (void) WriteBlobString(image,buffer);
     if (QuantumTick(y,picon->rows))
-      if (!MagickMonitor(SaveImageText,y,picon->rows,&image->exception))
+      if (!MagickMonitorFormatted(y,picon->rows,&image->exception,
+                                  SaveImageText,image->filename,
+				  image->columns,image->rows))
         break;
   }
   DestroyImage(picon);
@@ -824,9 +830,6 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
-  register IndexPacket
-    *indexes;
-
   register long
     i,
     x;
@@ -896,10 +899,13 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
       MagickReallocMemory(PixelPacket *,image->colormap,colors*sizeof(PixelPacket));
       for (y=0; y < (long) image->rows; y++)
       {
+        register IndexPacket
+          *indexes;
+
         q=GetImagePixels(image,0,y,image->columns,1);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
+        indexes=AccessMutableIndexes(image);
         for (x=0; x < (long) image->columns; x++)
         {
           if (q->opacity == TransparentOpacity)
@@ -960,10 +966,13 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlobString(image,"/* pixels */\n");
   for (y=0; y < (long) image->rows; y++)
   {
+    register const IndexPacket
+      *indexes;
+
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(image);
+    indexes=AccessImmutableIndexes(image);
     (void) WriteBlobString(image,"\"");
     for (x=0; x < (long) image->columns; x++)
     {
@@ -982,7 +991,9 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
       (y == (long) (image->rows-1) ? "" : ","));
     (void) WriteBlobString(image,buffer);
     if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+      if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                  SaveImageText,image->filename,
+				  image->columns,image->rows))
         break;
   }
   (void) WriteBlobString(image,"};\n");
