@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 GraphicsMagick Group
+% Copyright (C) 2003-2009 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -40,6 +40,7 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/profile.h"
 #include "magick/render.h"
 #include "magick/tempfile.h"
 #include "magick/utility.h"
@@ -113,7 +114,8 @@ MagickExport MagickPassFail CloneImageAttributes(Image* clone_image,
           break;
         }
       cloned_attribute->key=AcquireString(attribute->key);
-      cloned_attribute->value=AcquireString(attribute->value);
+      cloned_attribute->length=attribute->length;
+      cloned_attribute->value=MagickAllocateMemory(char *,cloned_attribute->length+1);
       cloned_attribute->previous=(ImageAttribute *) NULL;
       cloned_attribute->next=(ImageAttribute *) NULL;
       if ((cloned_attribute->value == (char *) NULL) ||
@@ -123,6 +125,7 @@ MagickExport MagickPassFail CloneImageAttributes(Image* clone_image,
           status = MagickFail;
           break;
         }
+      strcpy(cloned_attribute->value,attribute->value);
 
       if (cloned_attributes == (ImageAttribute *) NULL)
         {
@@ -243,31 +246,37 @@ static unsigned int GenerateIPTCAttribute(Image *image,const char *key)
   size_t
     length;
 
-  if (image->iptc_profile.length == 0)
+  const unsigned char
+    *profile;
+
+  size_t
+    profile_length;
+
+  if((profile=GetImageProfile(image,"IPTC",&profile_length)) == 0)
     return(False);
   count=sscanf(key,"IPTC:%d:%d",&dataset,&record);
   if (count != 2)
     return(False);
-  for (i=0; i < (long) image->iptc_profile.length; i++)
+  for (i=0; i < (long) profile_length; i++)
   {
-    if (image->iptc_profile.info[i] != 0x1c)
+    if (profile[i] != 0x1cU)
       continue;
-    if (image->iptc_profile.info[i+1] != dataset)
+    if (profile[i+1] != dataset)
       continue;
-    if (image->iptc_profile.info[i+2] != record)
+    if (profile[i+2] != record)
       continue;
-    length=image->iptc_profile.info[i+3] << 8;
-    length|=image->iptc_profile.info[i+4];
+    length=profile[i+3] << 8;
+    length|=profile[i+4];
     attribute=MagickAllocateMemory(char *,length+MaxTextExtent);
     if (attribute == (char *) NULL)
       continue;
-    (void) strncpy(attribute,(char *) image->iptc_profile.info+i+5,length);
+    (void) strlcpy(attribute,(char *) profile+i+5,length);
     attribute[length]='\0';
     (void) SetImageAttribute(image,key,(const char *) attribute);
     MagickFreeMemory(attribute);
     break;
   }
-  return(i < (long) image->iptc_profile.length);
+  return(i < (long) profile_length);
 }
 
 static unsigned char ReadByte(unsigned char **p,size_t *length)
@@ -336,8 +345,12 @@ static int ReadMSBShort(unsigned char **p,size_t *length)
   return(value);
 }
 
+/*
+  FIXME: length is defined as type size_t, and then code incorrectly assumes that size_t is
+  a signed type
+*/
 static char *TracePSClippingPath(unsigned char *blob,size_t length,
-  unsigned long columns,unsigned long rows)
+  unsigned long ARGUNUSED(columns),unsigned long ARGUNUSED(rows))
 {
   char
     *path,
@@ -362,6 +375,7 @@ static char *TracePSClippingPath(unsigned char *blob,size_t length,
   unsigned int
     in_subpath;
 
+  first[0].x=first[0].y=first[1].x=first[1].y=0;
   path=AllocateString((char *) NULL);
   if (path == (char *) NULL)
     return((char *) NULL);
@@ -552,6 +566,10 @@ static char *TracePSClippingPath(unsigned char *blob,size_t length,
   return(path);
 }
 
+/*
+  FIXME: length is defined as type size_t, and then code incorrectly assumes that size_t is
+  a signed type.
+*/
 static char *TraceSVGClippingPath(unsigned char *blob,size_t length,
   unsigned long columns,unsigned long rows)
 {
@@ -578,6 +596,7 @@ static char *TraceSVGClippingPath(unsigned char *blob,size_t length,
   unsigned int
     in_subpath;
 
+  first[0].x=first[0].y=first[1].x=first[1].y=0;
   path=AllocateString((char *) NULL);
   if (path == (char *) NULL)
     return((char *) NULL);
@@ -768,7 +787,13 @@ static int Generate8BIMAttribute(Image *image,const char *key)
   unsigned long
     count;
 
-  if (image->iptc_profile.length == 0)
+  const unsigned char
+    *profile;
+
+  size_t
+    profile_length;
+
+  if ((profile=GetImageProfile(image,"IPTC",&profile_length)) == 0)
     return(False);
 
   /*
@@ -790,8 +815,10 @@ static int Generate8BIMAttribute(Image *image,const char *key)
   resource=(char *) NULL;
 
   status=False;
-  length=image->iptc_profile.length;
-  info=image->iptc_profile.info;
+  length=profile_length;
+  /* FIXME: following cast should not be necessary but info can't be
+     const due to odd function design. */
+  info=(unsigned char *) profile;
 
   while ((length > 0) && (status == False))
   {
@@ -852,7 +879,7 @@ static int Generate8BIMAttribute(Image *image,const char *key)
     attribute=(char *) MagickAllocateMemory(char *,count+MaxTextExtent);
     if (attribute != (char *) NULL)
       {
-        memcpy(attribute,(char *) info,count);
+        (void) memcpy(attribute,(char *) info,count);
         attribute[count]='\0';
         info+=count;
         length-=count;
@@ -927,24 +954,139 @@ static TagInfo
     {  0x115, (char *) "SamplesPerPixel"},
     {  0x116, (char *) "RowsPerStrip"},
     {  0x117, (char *) "StripByteCounts"},
+    {  0x118, (char *) "MinSampleValue"},
+    {  0x119, (char *) "MaxSampleValue"},
     {  0x11A, (char *) "XResolution"},
     {  0x11B, (char *) "YResolution"},
     {  0x11C, (char *) "PlanarConfiguration"},
+    {  0x11D, (char *) "PageName"},
+    {  0x11E, (char *) "XPosition"},
+    {  0x11F, (char *) "YPosition"},
+    {  0x120, (char *) "FreeOffsets"},
+    {  0x121, (char *) "FreeByteCounts"},
+    {  0x122, (char *) "GrayResponseUnit"},
+    {  0x123, (char *) "GrayResponseCurve"},
+    {  0x124, (char *) "T4Options"},
+    {  0x125, (char *) "T6Options"},
     {  0x128, (char *) "ResolutionUnit"},
     {  0x12D, (char *) "TransferFunction"},
     {  0x131, (char *) "Software"},
     {  0x132, (char *) "DateTime"},
     {  0x13B, (char *) "Artist"},
+    {  0x13C, (char *) "HostComputer"},
+    {  0x13D, (char *) "Predictor"},
     {  0x13E, (char *) "WhitePoint"},
     {  0x13F, (char *) "PrimaryChromaticities"},
+    {  0x140, (char *) "ColorMap"},
+    {  0x141, (char *) "HalfToneHints"},
+    {  0x142, (char *) "TileWidth"},
+    {  0x143, (char *) "TileLength"},
+    {  0x144, (char *) "TileOffsets"},
+    {  0x145, (char *) "TileByteCounts"},
+    {  0x14A, (char *) "SubIFD"},
+    {  0x14C, (char *) "InkSet"},
+    {  0x14D, (char *) "InkNames"},
+    {  0x14E, (char *) "NumberOfInks"},
+    {  0x150, (char *) "DotRange"},
+    {  0x151, (char *) "TargetPrinter"},
+    {  0x152, (char *) "ExtraSample"},
+    {  0x153, (char *) "SampleFormat"},
+    {  0x154, (char *) "SMinSampleValue"},
+    {  0x155, (char *) "SMaxSampleValue"},
     {  0x156, (char *) "TransferRange"},
+    {  0x157, (char *) "ClipPath"},
+    {  0x158, (char *) "XClipPathUnits"},
+    {  0x159, (char *) "YClipPathUnits"},
+    {  0x15A, (char *) "Indexed"},
+    {  0x15B, (char *) "JPEGTables"},
+    {  0x15F, (char *) "OPIProxy"},
     {  0x200, (char *) "JPEGProc"},
     {  0x201, (char *) "JPEGInterchangeFormat"},
     {  0x202, (char *) "JPEGInterchangeFormatLength"},
+    {  0x203, (char *) "JPEGRestartInterval"},
+    {  0x205, (char *) "JPEGLosslessPredictors"},
+    {  0x206, (char *) "JPEGPointTransforms"},
+    {  0x207, (char *) "JPEGQTables"},
+    {  0x208, (char *) "JPEGDCTables"},
+    {  0x209, (char *) "JPEGACTables"},
     {  0x211, (char *) "YCbCrCoefficients"},
     {  0x212, (char *) "YCbCrSubSampling"},
     {  0x213, (char *) "YCbCrPositioning"},
     {  0x214, (char *) "ReferenceBlackWhite"},
+    {  0x2BC, (char *) "ExtensibleMetadataPlatform"},
+    {  0x301, (char *) "Gamma"},
+    {  0x302, (char *) "ICCProfileDescriptor"},
+    {  0x303, (char *) "SRGBRenderingIntent"},
+    {  0x320, (char *) "ImageTitle"},
+    {  0x5001, (char *) "ResolutionXUnit"},
+    {  0x5002, (char *) "ResolutionYUnit"},
+    {  0x5003, (char *) "ResolutionXLengthUnit"},
+    {  0x5004, (char *) "ResolutionYLengthUnit"},
+    {  0x5005, (char *) "PrintFlags"},
+    {  0x5006, (char *) "PrintFlagsVersion"},
+    {  0x5007, (char *) "PrintFlagsCrop"},
+    {  0x5008, (char *) "PrintFlagsBleedWidth"},
+    {  0x5009, (char *) "PrintFlagsBleedWidthScale"},
+    {  0x500A, (char *) "HalftoneLPI"},
+    {  0x500B, (char *) "HalftoneLPIUnit"},
+    {  0x500C, (char *) "HalftoneDegree"},
+    {  0x500D, (char *) "HalftoneShape"},
+    {  0x500E, (char *) "HalftoneMisc"},
+    {  0x500F, (char *) "HalftoneScreen"},
+    {  0x5010, (char *) "JPEGQuality"},
+    {  0x5011, (char *) "GridSize"},
+    {  0x5012, (char *) "ThumbnailFormat"},
+    {  0x5013, (char *) "ThumbnailWidth"},
+    {  0x5014, (char *) "ThumbnailHeight"},
+    {  0x5015, (char *) "ThumbnailColorDepth"},
+    {  0x5016, (char *) "ThumbnailPlanes"},
+    {  0x5017, (char *) "ThumbnailRawBytes"},
+    {  0x5018, (char *) "ThumbnailSize"},
+    {  0x5019, (char *) "ThumbnailCompressedSize"},
+    {  0x501A, (char *) "ColorTransferFunction"},
+    {  0x501B, (char *) "ThumbnailData"},
+    {  0x5020, (char *) "ThumbnailImageWidth"},
+    {  0x5021, (char *) "ThumbnailImageHeight"},
+    {  0x5022, (char *) "ThumbnailBitsPerSample"},
+    {  0x5023, (char *) "ThumbnailCompression"},
+    {  0x5024, (char *) "ThumbnailPhotometricInterp"},
+    {  0x5025, (char *) "ThumbnailImageDescription"},
+    {  0x5026, (char *) "ThumbnailEquipMake"},
+    {  0x5027, (char *) "ThumbnailEquipModel"},
+    {  0x5028, (char *) "ThumbnailStripOffsets"},
+    {  0x5029, (char *) "ThumbnailOrientation"},
+    {  0x502A, (char *) "ThumbnailSamplesPerPixel"},
+    {  0x502B, (char *) "ThumbnailRowsPerStrip"},
+    {  0x502C, (char *) "ThumbnailStripBytesCount"},
+    {  0x502D, (char *) "ThumbnailResolutionX"},
+    {  0x502E, (char *) "ThumbnailResolutionY"},
+    {  0x502F, (char *) "ThumbnailPlanarConfig"},
+    {  0x5030, (char *) "ThumbnailResolutionUnit"},
+    {  0x5031, (char *) "ThumbnailTransferFunction"},
+    {  0x5032, (char *) "ThumbnailSoftwareUsed"},
+    {  0x5033, (char *) "ThumbnailDateTime"},
+    {  0x5034, (char *) "ThumbnailArtist"},
+    {  0x5035, (char *) "ThumbnailWhitePoint"},
+    {  0x5036, (char *) "ThumbnailPrimaryChromaticities"},
+    {  0x5037, (char *) "ThumbnailYCbCrCoefficients"},
+    {  0x5038, (char *) "ThumbnailYCbCrSubsampling"},
+    {  0x5039, (char *) "ThumbnailYCbCrPositioning"},
+    {  0x503A, (char *) "ThumbnailRefBlackWhite"},
+    {  0x503B, (char *) "ThumbnailCopyRight"},
+    {  0x5090, (char *) "LuminanceTable"},
+    {  0x5091, (char *) "ChrominanceTable"},
+    {  0x5100, (char *) "FrameDelay"},
+    {  0x5101, (char *) "LoopCount"},
+    {  0x5110, (char *) "PixelUnit"},
+    {  0x5111, (char *) "PixelPerUnitX"},
+    {  0x5112, (char *) "PixelPerUnitY"},
+    {  0x5113, (char *) "PaletteHistogram"},
+    {  0x1000, (char *) "RelatedImageFileFormat"},
+    {  0x800D, (char *) "ImageID"},
+    {  0x80E3, (char *) "Matteing"},
+    {  0x80E4, (char *) "DataType"},
+    {  0x80E5, (char *) "ImageDepth"},
+    {  0x80E6, (char *) "TileDepth"},
     {  0x828D, (char *) "CFARepeatPatternDim"},
     {  0x828E, (char *) "CFAPattern"},
     {  0x828F, (char *) "BatteryLevel"},
@@ -952,6 +1094,9 @@ static TagInfo
     {  0x829A, (char *) "ExposureTime"},
     {  0x829D, (char *) "FNumber"},
     {  0x83BB, (char *) "IPTC/NAA"},
+    {  0x84E3, (char *) "IT8RasterPadding"},
+    {  0x84E5, (char *) "IT8ColorTable"},
+    {  0x8649, (char *) "ImageResourceInformation"},
     {  0x8769, (char *) "ExifOffset"},
     {  0x8773, (char *) "InterColorProfile"},
     {  0x8822, (char *) "ExposureProgram"},
@@ -979,6 +1124,11 @@ static TagInfo
     {  0x9290, (char *) "SubSecTime"},
     {  0x9291, (char *) "SubSecTimeOriginal"},
     {  0x9292, (char *) "SubSecTimeDigitized"},
+    {  0x9C9B, (char *) "WinXP-Title"},     /* Win XP specific, UTF-16 Unicode */
+    {  0x9C9C, (char *) "WinXP-Comments"},  /* Win XP specific, UTF-16 Unicode */
+    {  0x9C9D, (char *) "WinXP-Author"},    /* Win XP specific, UTF-16 Unicode */
+    {  0x9C9E, (char *) "WinXP-Keywords"},  /* Win XP specific, UTF-16 Unicode */
+    {  0x9C9F, (char *) "WinXP-Subject"},   /* Win XP specific, UTF-16 Unicode */
     {  0xA000, (char *) "FlashPixVersion"},
     {  0xA001, (char *) "ColorSpace"},
     {  0xA002, (char *) "ExifImageWidth"},
@@ -986,19 +1136,135 @@ static TagInfo
     {  0xA005, (char *) "InteroperabilityOffset"},
     {  0xA20B, (char *) "FlashEnergy"},
     {  0xA20C, (char *) "SpatialFrequencyResponse"},
+    {  0xA20D, (char *) "Noise"},
     {  0xA20E, (char *) "FocalPlaneXResolution"},
     {  0xA20F, (char *) "FocalPlaneYResolution"},
     {  0xA210, (char *) "FocalPlaneResolutionUnit"},
+    {  0xA211, (char *) "ImageNumber"},
+    {  0xA212, (char *) "SecurityClassification"},
+    {  0xA213, (char *) "ImageHistory"},
     {  0xA214, (char *) "SubjectLocation"},
     {  0xA215, (char *) "ExposureIndex"},
+    {  0xA216, (char *) "TIFF/EPStandardID"},
     {  0xA217, (char *) "SensingMethod"},
     {  0xA300, (char *) "FileSource"},
     {  0xA301, (char *) "SceneType"},
+    {  0xA302, (char *) "CFAPattern"},
+    {  0xA401, (char *) "CustomRendered"},
+    {  0xA402, (char *) "ExposureMode"},
+    {  0xA403, (char *) "WhiteBalance"},
+    {  0xA404, (char *) "DigitalZoomRatio"},
+    {  0xA405, (char *) "FocalLengthIn35mmFilm"},
+    {  0xA406, (char *) "SceneCaptureType"},
+    {  0xA407, (char *) "GainControl"},
+    {  0xA408, (char *) "Contrast"},
+    {  0xA409, (char *) "Saturation"},
+    {  0xA40A, (char *) "Sharpness"},
+    {  0xA40B, (char *) "DeviceSettingDescription"},
+    {  0xA40C, (char *) "SubjectDistanceRange"},
+    {  0xA420, (char *) "ImageUniqueID"},
     {  0x0000, (char *) NULL}
   };
 
+/*
+  Format a tag description.
+
+  An EXIF tag value to be translated, and a buffer of at least
+  MaxTextExtent length are passed as arguments.  For convenience, the
+  converted string is returned.
+*/
+
+static const char *
+EXIFTagDescription(int t, char *tag_description)
+{
+  unsigned int
+    i;
+
+  tag_description[0]='\0';
+  for (i=0; ; i++)
+    {
+      if (tag_table[i].tag == 0)
+	break;
+      if (tag_table[i].tag == t)
+	{
+	  (void) strlcpy(tag_description,tag_table[i].description,MaxTextExtent);
+	  break;
+	}
+    }
+  if (tag_description[0] == '\0')
+    FormatString(tag_description,"0x%04X",t);
+
+  return tag_description;
+}
+
+static const char *
+EXIFFormatDescription(int f)
+{
+  const char
+    *description;
+
+  description="unknown";
+
+  switch (f)
+    {
+    case EXIF_FMT_BYTE:
+      description="BYTE";
+      break;
+    case EXIF_FMT_STRING:
+      description="STRING";
+      break;
+    case EXIF_FMT_USHORT:
+      description="USHORT";
+      break;
+    case EXIF_FMT_ULONG:
+      description="ULONG";
+      break;
+    case EXIF_FMT_URATIONAL:
+      description="URATIONAL";
+      break;
+    case EXIF_FMT_SBYTE:
+      description="SBYTE";
+      break;
+    case EXIF_FMT_UNDEFINED:
+      description="UNDEFINED";
+      break;
+    case EXIF_FMT_SSHORT:
+      description="SSHORT";
+      break;
+    case EXIF_FMT_SLONG:
+      description="SLONG";
+      break;
+    case EXIF_FMT_SRATIONAL:
+      description="SRATIONAL";
+      break;
+    case EXIF_FMT_SINGLE:
+      description="SINGLE";
+      break;
+    case EXIF_FMT_DOUBLE:
+      description="DOUBLE";
+      break;
+    }
+
+  return description;
+}
+
 static int
-  format_bytes[] = {0,1,1,2,4,8,1,1,2,4,8,4,8};
+  format_bytes[] =
+  {
+    0,
+    1, /* BYTE */
+    1, /* STRING / ASCII */
+    2, /* USHORT */
+    4, /* ULONG */
+    8, /* URATIONAL */
+    1, /* SBYTE */
+    1, /* UNDEFINED */
+    2, /* SSHORT */
+    4, /* SLONG */
+    8, /* SRATIONAL */
+    4, /* SINGLE / FLOAT */
+    8  /* DOUBLE */
+  };
 
 static short Read16s(int morder,void *ishort)
 {
@@ -1040,15 +1306,17 @@ static long Read32s(int morder,void *ilong)
 
 static unsigned long Read32u(int morder, void *ilong)
 {
-  return(Read32s(morder,ilong) & 0xffffffff);
+  return(Read32s(morder,ilong) & 0xffffffffUL);
 }
 
+
 static int GenerateEXIFAttribute(Image *image,const char *specification)
 {
   char
+    *final,
     *key,
-    *value,
-    *final;
+    tag_description[MaxTextExtent],
+    *value;
 
   int
     id,
@@ -1057,7 +1325,6 @@ static int GenerateEXIFAttribute(Image *image,const char *specification)
     all;
 
   long
-    index,
     tag;
 
   register long
@@ -1080,22 +1347,36 @@ static int GenerateEXIFAttribute(Image *image,const char *specification)
     destack[DE_STACK_SIZE],
     nde;
 
+  const unsigned char
+    *profile_info;
+  
+  size_t
+    profile_length;
+
+  MagickBool
+    debug=MagickFalse;
+
+  {
+    const char *
+      env_value;
+    
+    /*
+      Allow enabling debug of EXIF tags
+    */
+    if ((env_value=getenv("MAGICK_DEBUG_EXIF")))
+      {
+	if (LocaleCompare(env_value,"TRUE") == 0)
+	  debug=MagickTrue;
+      }
+  }
+
   /*
     Determine if there is any EXIF data available in the image.
   */
   value=(char *) NULL;
   final=AllocateString("");
-  index=(-1);
-  for (i=0; i < (long) image->generic_profiles; i++)
-  {
-    if ((LocaleCompare(image->generic_profile[i].name,"EXIF") == 0) &&
-        (image->generic_profile[i].length != 0))
-      {
-        index=i;
-        break;
-      }
-  }
-  if (index < 0)
+  profile_info=GetImageProfile(image,"EXIF",&profile_length);
+  if (profile_info == 0)
     goto generate_attribute_failure;
   /*
     If EXIF data exists, then try to parse the request for a tag.
@@ -1186,9 +1467,9 @@ static int GenerateEXIFAttribute(Image *image,const char *specification)
     }
   }
   if (tag < 0)
-    goto generate_attribute_failure;
-  length=image->generic_profile[index].length;
-  info=image->generic_profile[index].info;
+    goto generate_attribute_failure;;
+  length=profile_length;
+  info=(unsigned char *) profile_info;
   while (length != 0)
   {
     if (ReadByte(&info,&length) != 0x45)
@@ -1248,14 +1529,18 @@ static int GenerateEXIFAttribute(Image *image,const char *specification)
     nde=Read16u(morder,ifdp);
     for (; de < nde; de++)
     {
-      long
-        n,
+      unsigned int
+        n;
+
+      int
         t,
         f,
         c;
 
       char
-        *pde,
+        *pde;
+
+      unsigned char
         *pval;
 
       pde=(char *) (ifdp+2+(12*de));
@@ -1266,7 +1551,7 @@ static int GenerateEXIFAttribute(Image *image,const char *specification)
       c=(long) Read32u(morder,pde+4); /* get number of components */
       n=c*format_bytes[f];
       if (n <= 4)
-        pval=pde+8;
+        pval=(unsigned char *) pde+8;
       else
         {
           unsigned long
@@ -1278,35 +1563,65 @@ static int GenerateEXIFAttribute(Image *image,const char *specification)
           oval=Read32u(morder,pde+8);
           if ((oval+n) > length)
             continue;
-          pval=(char *)(tiffp+oval);
+          pval=(unsigned char *)(tiffp+oval);
         }
       if (all || (tag == t))
         {
           char
             s[MaxTextExtent];
 
+	  if (debug)
+	    {
+	      fprintf(stderr,"EXIF: TagVal=%d  TagDescr=\"%s\" Format=%d  "
+		      "FormatDescr=\"%s\"  Components=%d\n",t,
+		      EXIFTagDescription(t,tag_description),f,
+		      EXIFFormatDescription(f),c);
+	    }
+
           switch (f)
           {
             case EXIF_FMT_SBYTE:
             {
+              /* 8-bit signed integer */
               FormatString(s,"%ld",(long) (*(char *) pval));
               value=AllocateString(s);
               break;
             }
             case EXIF_FMT_BYTE:
             {
+              /* 8-bit unsigned integer */
+              value=MagickAllocateMemory(char *,n+1);
+              if (value != (char *) NULL)
+                {
+                  unsigned int
+                    a;
+
+                  for (a=0; a < n; a++)
+                  {
+                    value[a]='.';
+                    if (isprint((int) pval[a]) /* || (pval[a] == '\0') */)
+                      value[a]=pval[a];
+                  }
+                  value[a]='\0';
+                  break;
+                }
+#if 0
+              printf("format %u, length %u\n",f,n);
               FormatString(s,"%ld",(long) (*(unsigned char *) pval));
               value=AllocateString(s);
+#endif
               break;
             }
             case EXIF_FMT_SSHORT:
             {
+              /* 16-bit signed integer */
               FormatString(s,"%hd",Read16u(morder,pval));
               value=AllocateString(s);
               break;
             }
             case EXIF_FMT_USHORT:
             {
+              /* 16-bit unsigned integer */
               FormatString(s,"%hu",Read16s(morder,pval));
               value=AllocateString(s);
               break;
@@ -1353,57 +1668,64 @@ static int GenerateEXIFAttribute(Image *image,const char *specification)
             case EXIF_FMT_UNDEFINED:
             case EXIF_FMT_STRING:
             {
-              value=MagickAllocateMemory(char *,n+1);
+	      unsigned int
+		a;
+
+	      size_t
+		allocation_size;
+
+	      MagickBool
+		binary=MagickFalse;
+
+	      allocation_size=n+1;
+	      for (a=0; a < n; a++)
+		if (!(isprint((int) pval[a])))
+		  allocation_size += 3;
+
+              value=MagickAllocateMemory(char *,allocation_size);
               if (value != (char *) NULL)
                 {
-                  long
-                    a;
-
+		  i=0;
                   for (a=0; a < n; a++)
                   {
-                    value[a]='.';
-                    if (isprint((int) pval[a]) || (pval[a] == '\0'))
-                      value[a]=pval[a];
+		    if ((f == EXIF_FMT_STRING) && (pval[a] == '\0'))
+		      break;
+		    if ((isprint((int) pval[a])) ||
+			((pval[a] == '\0') && (a == (n-1) && (!binary))))
+		      {
+			value[i++]=pval[a];
+		      }
+		    else
+		      {
+			i += sprintf(&value[i],"\\%03o",(unsigned int) pval[a]);
+			binary |= MagickTrue;
+		      }
                   }
-                  value[a]='\0';
-                  break;
+                  value[i]='\0';
                 }
               break;
             }
           }
           if (value != (char *) NULL)
             {
-              int
-                i;
-
-              char
+              const char
                 *description;
 
               if (strlen(final) != 0)
                 (void) ConcatenateString(&final,EXIF_DELIMITER);
-              description=(char *) NULL;
+              description=(const char *) NULL;
               switch (all)
               {
                 case 1:
                 {
-                  description=(char *) "unknown";
-                  for (i=0; ; i++)
-                  {
-                    if (tag_table[i].tag == 0)
-                      break;
-                    if (tag_table[i].tag == t)
-                      {
-                        description=tag_table[i].description;
-                        break;
-                      }
-                  }
+		  description=EXIFTagDescription(t,tag_description);
                   FormatString(s,"%.1024s=",description);
                   (void) ConcatenateString(&final,s);
                   break;
                 }
                 case 2:
                 {
-                  FormatString(s,"#%04lx=",t);
+                  FormatString(s,"#%04x=",t);
                   (void) ConcatenateString(&final,s);
                   break;
                 }
@@ -1414,10 +1736,7 @@ static int GenerateEXIFAttribute(Image *image,const char *specification)
         }
         if ((t == TAG_EXIF_OFFSET) || (t == TAG_INTEROP_OFFSET))
           {
-            size_t
-              offset;
-
-            offset=(size_t) Read32u(morder,pval);
+            offset=Read32u(morder,pval);
             if ((offset < length) && (level < (DE_STACK_SIZE-2)))
               {
                 /*
@@ -1440,6 +1759,7 @@ static int GenerateEXIFAttribute(Image *image,const char *specification)
   } while (level > 0);
   if (strlen(final) == 0)
     (void) ConcatenateString(&final,"unknown");
+
   (void) SetImageAttribute(image,specification,(const char *) final);
   MagickFreeMemory(final);
   return(True);
@@ -1448,34 +1768,125 @@ static int GenerateEXIFAttribute(Image *image,const char *specification)
   return False;
 }
 
+/*
+  Generate an aggregate attribute result based on a wildcard
+  specification like "foo:*".
+*/
+static int GenerateWildcardAttribute(Image *image,const char *key)
+{
+  char
+    *result=NULL;
+
+  size_t
+    key_length=0;
+
+  register ImageAttribute
+    *p = (ImageAttribute *) NULL;
+
+  MagickPassFail
+    status=MagickFail;
+
+  /*
+    Support a full "*" wildcard.
+  */
+  if (strcmp("*",key) == 0)
+    {
+      (void) GenerateIPTCAttribute((Image *) image,"IPTC:*");
+      (void) Generate8BIMAttribute((Image *) image,"8BIM:*");
+      (void) GenerateEXIFAttribute((Image *) image,"EXIF:*");
+    }
+
+  key_length=strlen(key)-1;
+  for (p=image->attributes; p != (ImageAttribute *) NULL; p=p->next)
+    if (LocaleNCompare(key,p->key,key_length) == 0)
+      {
+        char
+          s[MaxTextExtent];
+
+        if (result != NULL)
+          (void) ConcatenateString(&result,"\n");
+        FormatString(s,"%.512s=%.1024s",p->key,p->value);
+        (void) ConcatenateString(&result,s);
+      }
+
+  if (result != NULL)
+    {
+      status=SetImageAttribute(image,key,result);
+      MagickFreeMemory(result);
+    }
+  return status;
+}
+
 MagickExport const ImageAttribute *GetImageAttribute(const Image *image,
   const char *key)
 {
   register ImageAttribute
-    *p;
+    *p = (ImageAttribute *) NULL;
+
+  size_t
+    key_length=0;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+
+  /*
+    If key is null, then return a pointer to the attribute list.
+  */
   if (key == (char *) NULL)
     return(image->attributes);
+
+  key_length=strlen(key);
+
+
   for (p=image->attributes; p != (ImageAttribute *) NULL; p=p->next)
     if (LocaleCompare(key,p->key) == 0)
       return(p);
+
   if (LocaleNCompare("IPTC:",key,5) == 0)
     {
+      /*
+        Create an attribute named "IPTC:*" with all matching
+        key=values and return it.
+      */
       if (GenerateIPTCAttribute((Image *) image,key) == True)
         return(GetImageAttribute(image,key));
     }
-  if (LocaleNCompare("8BIM:",key,5) == 0)
+  else if (LocaleNCompare("8BIM:",key,5) == 0)
     {
+      /*
+        Create an attribute named "8BIM:*" with all matching
+        key=values and return it.
+      */
       if (Generate8BIMAttribute((Image *) image,key) == True)
         return(GetImageAttribute(image,key));
     }
-  if (LocaleNCompare("EXIF:",key,5) == 0)
+  else if (LocaleNCompare("EXIF:",key,5) == 0)
     {
+      /*
+        Create an attribute named "EXIF:*" with all matching
+        key=values and return it.
+      */
       if (GenerateEXIFAttribute((Image *) image,key) == True)
         return(GetImageAttribute(image,key));
     }
+  else if ((key_length >=2) && (key[key_length-1] == '*'))
+    {
+      /*
+        Create an attribute named "foo:*" with all matching
+        key=values and return it.
+      */
+      if (GenerateWildcardAttribute((Image *) image,key) == True)
+        return(GetImageAttribute(image,key));
+    }
+  else if ((key_length ==1) && (key[0] == '*'))
+    {
+      /*
+        Create an attribute named "*" with all key=values and return
+        it.
+      */
+      if (GenerateWildcardAttribute((Image *) image,key) == True)
+        return(GetImageAttribute(image,key));
+    }
   return(p);
 }
 
@@ -1559,7 +1970,7 @@ MagickExport const ImageAttribute *GetImageInfoAttribute(
       if (LocaleNCompare("base",key,2) == 0)
         {
           GetPathComponent(image->magick_filename,BasePath,filename);
-          (void) strncpy(attribute,filename,MaxTextExtent-1);
+          (void) strlcpy(attribute,filename,MaxTextExtent);
           break;
         }
       break;
@@ -1568,13 +1979,13 @@ MagickExport const ImageAttribute *GetImageInfoAttribute(
     {
       if (LocaleNCompare("depth",key,2) == 0)
         {
-          FormatString(attribute,"%lu",image->depth);
+          FormatString(attribute,"%u",image->depth);
           break;
         }
       if (LocaleNCompare("directory",key,2) == 0)
         {
           GetPathComponent(image->magick_filename,HeadPath,filename);
-          (void) strncpy(attribute,filename,MaxTextExtent-1);
+          (void) strlcpy(attribute,filename,MaxTextExtent);
           break;
         }
       break;
@@ -1584,7 +1995,7 @@ MagickExport const ImageAttribute *GetImageInfoAttribute(
       if (LocaleNCompare("extension",key,2) == 0)
         {
           GetPathComponent(image->magick_filename,ExtensionPath,filename);
-          (void) strncpy(attribute,filename,MaxTextExtent-1);
+          (void) strlcpy(attribute,filename,MaxTextExtent);
           break;
         }
       break;
@@ -1612,7 +2023,7 @@ MagickExport const ImageAttribute *GetImageInfoAttribute(
     {
       if (LocaleNCompare("input",key,2) == 0)
         {
-          (void) strncpy(attribute,image->filename,MaxTextExtent-1);
+          (void) strlcpy(attribute,image->filename,MaxTextExtent);
           break;
         }
       break;
@@ -1621,7 +2032,7 @@ MagickExport const ImageAttribute *GetImageInfoAttribute(
     {
       if (LocaleNCompare("magick",key,2) == 0)
         {
-          (void) strncpy(attribute,image->magick,MaxTextExtent-1);
+          (void) strlcpy(attribute,image->magick,MaxTextExtent);
           break;
         }
       break;
@@ -1630,7 +2041,7 @@ MagickExport const ImageAttribute *GetImageInfoAttribute(
     {
       if (LocaleNCompare("name",key,2) == 0)
         {
-          (void) strncpy(attribute,filename,MaxTextExtent-1);
+          (void) strlcpy(attribute,filename,MaxTextExtent);
           break;
         }
      break;
@@ -1664,7 +2075,7 @@ MagickExport const ImageAttribute *GetImageInfoAttribute(
     {
       if (LocaleNCompare("output",key,2) == 0)
         {
-          (void) strncpy(attribute,image_info->filename,MaxTextExtent-1);
+          (void) strlcpy(attribute,image_info->filename,MaxTextExtent);
           break;
         }
      break;
@@ -1691,11 +2102,11 @@ MagickExport const ImageAttribute *GetImageInfoAttribute(
     {
       if (LocaleNCompare("unique",key,2) == 0)
         {
-          (void) strncpy(filename,image_info->unique,MaxTextExtent-1);
+          (void) strlcpy(filename,image_info->unique,MaxTextExtent);
           if (*filename == '\0')
             if(!AcquireTemporaryFileName(filename))
               return((ImageAttribute *) NULL);
-          (void) strncpy(attribute,filename,MaxTextExtent-1);
+          (void) strlcpy(attribute,filename,MaxTextExtent);
           break;
         }
       break;
@@ -1732,11 +2143,11 @@ MagickExport const ImageAttribute *GetImageInfoAttribute(
     {
       if (LocaleNCompare("zero",key,2) == 0)
         {
-          (void) strncpy(filename,image_info->zero,MaxTextExtent-1);
+          (void) strlcpy(filename,image_info->zero,MaxTextExtent);
           if (*filename == '\0')
             if(!AcquireTemporaryFileName(filename))
               return((ImageAttribute *) NULL);
-          (void) strncpy(attribute,filename,MaxTextExtent-1);
+          (void) strlcpy(attribute,filename,MaxTextExtent);
           break;
         }
       break;
@@ -1780,7 +2191,7 @@ MagickExport const ImageAttribute *GetImageInfoAttribute(
 %
 %
 */
-MagickExport unsigned int SetImageAttribute(Image *image,const char *key,
+MagickExport MagickPassFail SetImageAttribute(Image *image,const char *key,
   const char *value)
 {
   ImageAttribute
@@ -1795,7 +2206,8 @@ MagickExport unsigned int SetImageAttribute(Image *image,const char *key,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if ((key == (const char *) NULL) || (*key == '\0'))
-    return(False);
+    return(MagickFail);
+
   if (value == (const char *) NULL)
     {
       /*
@@ -1806,10 +2218,6 @@ MagickExport unsigned int SetImageAttribute(Image *image,const char *key,
           break;
       if (p == (ImageAttribute *) NULL)
         return(False);
-      if (p->key != (char *) NULL)
-        MagickFreeMemory(p->key);
-      if (p->value != (char *) NULL)
-        MagickFreeMemory(p->value);
       if (p->previous != (ImageAttribute *) NULL)
         p->previous->next=p->next;
       else
@@ -1821,39 +2229,79 @@ MagickExport unsigned int SetImageAttribute(Image *image,const char *key,
       if (p->next != (ImageAttribute *) NULL)
         p->next->previous=p->previous;
       attribute=p;
-      MagickFreeMemory(attribute);
-      return(True);
+      DestroyImageAttribute(attribute);
+      return(MagickPass);
     }
   if (*value == '\0')
-    return(False);
+    return(MagickFail);
   attribute=MagickAllocateMemory(ImageAttribute *,sizeof(ImageAttribute));
   if (attribute == (ImageAttribute *) NULL)
-    return(False);
+    return(MagickFail);
   attribute->key=AllocateString(key);
-  if ((LocaleNCompare(key,"EXIF",4) == 0) ||
-      (LocaleNCompare(key,"IPTC",4) == 0) ||
-      (LocaleNCompare(key,"[Locale",7) == 0) ||
-      (LocaleNCompare(key,"8BIM",4) == 0))
-    attribute->value=AllocateString(value);
+  attribute->length=0;
+  if ((LocaleNCompare(key,"comment",7) == 0) ||
+      (LocaleNCompare(key,"label",5) == 0))
+    {
+      /*
+        Translate format requests in attribute text.
+
+        This is really gross since it is assumed that the attribute is
+        supplied by the user and the user intends for translation to
+        occur.  However, comment attributes may also come from an
+        image file and may contain arbitrary text.  There does not
+        seem to be any work-around which preserves the already defined
+        interface.
+      */
+      attribute->value=TranslateText((ImageInfo *) NULL,image,value);
+      if (attribute->value != (char *) NULL)
+        attribute->length=strlen(attribute->value);
+    }
   else
-    attribute->value=TranslateText((ImageInfo *) NULL,image,value);
-  attribute->compression=False;
+    {
+      /*
+        Use attribute text as is.
+      */
+      attribute->length=strlen(value);
+      attribute->value=MagickAllocateMemory(char *,attribute->length+1);
+      if (attribute->value != (char *) NULL)
+        (void) strlcpy(attribute->value,value,attribute->length+1);
+    }
+  if ((attribute->value == (char *) NULL) ||
+      (attribute->key == (char *) NULL))
+    {
+      DestroyImageAttribute(attribute);
+      return(MagickFail);
+    }
+
   attribute->previous=(ImageAttribute *) NULL;
   attribute->next=(ImageAttribute *) NULL;
   if (image->attributes == (ImageAttribute *) NULL)
     {
       image->attributes=attribute;
-      return(True);
+      return(MagickPass);
     }
   for (p=image->attributes; p != (ImageAttribute *) NULL; p=p->next)
   {
     if (LocaleCompare(attribute->key,p->key) == 0)
       {
-        (void) ConcatenateString(&p->value,attribute->value);
-        MagickFreeMemory(attribute->value);
-        MagickFreeMemory(attribute->key);
-        MagickFreeMemory(attribute);
-        return(True);
+        size_t
+          min_l,
+          realloc_l;
+
+        /*
+          Extend existing text string.
+        */
+        min_l=p->length+attribute->length+1;
+        for (realloc_l=2; realloc_l <= min_l; realloc_l *= 2);
+        MagickReallocMemory(char *,p->value,realloc_l);
+        if (p->value != (char *) NULL)
+          (void) strcat(p->value+p->length,attribute->value);
+        p->length += attribute->length;
+        DestroyImageAttribute(attribute);
+        if (p->value != (char *) NULL)
+          return(MagickPass);
+        (void) SetImageAttribute(image,key,NULL);
+        return(MagickFail);
       }
     if (p->next == (ImageAttribute *) NULL)
       break;
@@ -1863,5 +2311,5 @@ MagickExport unsigned int SetImageAttribute(Image *image,const char *key,
   */
   attribute->previous=p;
   p->next=attribute;
-  return(True);
+  return(MagickPass);
 }
