@@ -44,7 +44,8 @@
 #include "magick/blob.h"
 #include "magick/constitute.h"
 #include "magick/delegate.h"
-#if defined(WIN32) || defined(__CYGWIN__)
+#include "magick/log.h"
+#if defined(MSWINDOWS) || defined(__CYGWIN__)
 # include "magick/nt_feature.h"
 #endif
 #if defined(POSIX)
@@ -314,6 +315,68 @@ MagickExport const DelegateInfo *GetDelegateInfo(const char *decode,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t P o s t s c r i p t D e l e g a t e I n f o                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  Method GetPostscriptDelegateInfo returns the Postscript delegate which
+%  best supports the image type requested via ImageInfo
+%
+%  The format of the GetPostscriptDelegateInfo method is:
+%
+%      const DelegateInfo *(const ImageInfo *image_info,
+%                           ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o image_info: The 'monochrome' and 'type' fields of image_info are used
+%                  to select the best postscript delegate type.
+%
+%    o antialias: Set to best antialias setting for this delegate based on
+%                 user requested antialias setting, and rendering depth.
+%
+%    o exception: Return any errors or warnings in this structure.
+%
+%
+*/
+MagickExport const DelegateInfo *GetPostscriptDelegateInfo(const ImageInfo *image_info,
+                                                           unsigned int *antialias,
+                                                           ExceptionInfo *exception)
+{
+  char
+    delegate[MaxTextExtent];
+
+  (void) strlcpy(delegate,"gs-color",sizeof(delegate));
+  *antialias=(image_info->antialias ? 4 : 1);
+  if ((image_info->monochrome) || (BilevelType == image_info->type))
+    {
+      (void) strlcpy(delegate,"gs-mono",sizeof(delegate));
+      *antialias=1;
+    }
+  else if (GrayscaleType == image_info->type)
+    {
+      (void) strlcpy(delegate,"gs-gray",sizeof(delegate));
+    }
+  else if (PaletteType == image_info->type)
+    {
+      (void) strlcpy(delegate,"gs-palette",sizeof(delegate));
+    }
+  else if ((GrayscaleMatteType == image_info->type) ||
+           (PaletteMatteType == image_info->type) ||
+           (TrueColorMatteType == image_info->type))
+    {
+      (void) strlcpy(delegate,"gs-color+alpha",sizeof(delegate));
+    }
+  return GetDelegateInfo(delegate,(char *) NULL,exception);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   I n v o k e D e l e g a t e                                               %
 %                                                                             %
 %                                                                             %
@@ -339,6 +402,7 @@ MagickExport const DelegateInfo *GetDelegateInfo(const char *decode,
 %
 %
 */
+#if defined(POSIX)
 static size_t
 UnixShellTextEscape(char *dst, const char *src, const size_t size)
 {
@@ -382,6 +446,7 @@ UnixShellTextEscape(char *dst, const char *src, const size_t size)
 
   return length;
 }
+#endif /* POSIX */
 
 MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
   const char *decode,const char *encode,ExceptionInfo *exception)
@@ -450,7 +515,7 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
         (encode && (delegate_info->decode != (char *) NULL)))
       {
         char
-          filename[MaxTextExtent],
+          decode_filename[MaxTextExtent],
           *magick;
 
         ImageInfo
@@ -497,7 +562,7 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
         (void) strlcpy((char *) clone_info->magick,magick,MaxTextExtent);
         (void) strlcpy(image->magick,magick,MaxTextExtent);
         MagickFreeMemory(magick);
-        (void) strlcpy(filename,image->filename,MaxTextExtent);
+        (void) strlcpy(decode_filename,image->filename,MaxTextExtent);
         FormatString(clone_info->filename,"%.1024s:",delegate_info->decode);
         (void) SetImageInfo(clone_info,True,exception);
         (void) strlcpy(clone_info->filename,image_info->filename,
@@ -505,7 +570,7 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
         for (p=image; p != (Image *) NULL; p=p->next)
         {
           FormatString(p->filename,"%.1024s:%.1024s",delegate_info->decode,
-            filename);
+            decode_filename);
           status=WriteImage(clone_info,p);
           if (status == False)
             {
@@ -673,14 +738,13 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
   MagickFreeMemory(commands);
   return(status != False);
 }
-
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   I n v o k e P o s t s r i p t D e l e g a t e                             %
++   I n v o k e P o s t s c r i p t D e l e g a t e                           %
 %                                                                             %
 %                                                                             %
 %                                                                             %
@@ -691,41 +755,48 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
 %
 %  The format of the InvokePostscriptDelegate method is:
 %
-%      unsigned int InvokePostscriptDelegate(const unsigned int verbose,
-%        const char *command)
+%      MagickPassFail InvokePostscriptDelegate(const unsigned int verbose,
+%        const char *command, ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method InvokePostscriptDelegate returns True is the command
-%      is successfully executed, otherwise False.
+%    o status:  Method InvokePostscriptDelegate returns MagickPass if the command
+%      is successfully executed, otherwise MagickFail.
 %
 %    o verbose: A value other than zero displays the command prior to
 %      executing it.
 %
 %    o command: The address of a character string containing the command to
-%      execute.
+%      execute.  The command is formulated through direct FormatString()
+%      substitutions rather than using TranslateText.
 %
 %
 */
-MagickExport unsigned int InvokePostscriptDelegate(const unsigned int verbose,
-  const char *command)
+MagickExport MagickPassFail
+InvokePostscriptDelegate(const unsigned int verbose,
+			 const char *command,ExceptionInfo *exception)
 {
-#if defined(HasGS) || defined(WIN32)
+  register long
+    i;
+
   char
     **argv;
 
+  int
+    argc;
+
+  int
+    status;
+
+#if defined(HasGS) || defined(MSWINDOWS)
+
   gs_main_instance
     *interpreter;
 
   int
-    argc,
-    code,
-    status;
+    pexit_code;
 
-  register long
-    i;
-
-#if defined(WIN32)
+#if defined(MSWINDOWS)
   const GhostscriptVectors
     *gs_func;
 
@@ -744,47 +815,103 @@ MagickExport unsigned int InvokePostscriptDelegate(const unsigned int verbose,
   gs_func_struct.run_string=gsapi_run_string;
   gs_func_struct.delete_instance=gsapi_delete_instance;
 #endif
-  if (gs_func == (GhostscriptVectors *) NULL)
+  if (gs_func != (GhostscriptVectors *) NULL)
     {
+
+      /*
+	Allocate an interpreter.
+      */
+      interpreter = (gs_main_instance *) NULL;
+      status=(gs_func->new_instance)(&interpreter,(void *) NULL);
+      if (status < 0)
+	{
+	  ThrowException(exception,DelegateError,
+			 FailedToAllocateGhostscriptInterpreter,command);
+	  return(MagickFail);
+	}
+      /*
+	Initialize interpreter with argument list.
+      */
+      argv=StringToArgv(command,&argc);
+      if (argv == (char **) NULL)
+	{
+	  ThrowException(exception,DelegateError,FailedToAllocateArgumentList,
+			 command);
+	  return(MagickFail);
+	}
+
       if (verbose)
-        (void) fputs(command,stdout);
+	{
+	  char
+	    buffer[MaxTextExtent];
+
+#if defined(MSWINDOWS)
+	  (void) NTGhostscriptDLL(buffer,sizeof(buffer));
+#else
+	  (void) strlcpy(buffer,"[ghostscript library]",sizeof(buffer));
+#endif
+	  (void) fputs(buffer,stderr);
+	  for (i=2 ; i < argc ; i++)
+	    (void) fprintf(stderr," \"%s\"",argv[i]);
+	  (void) fflush(stderr);
+	}
+      status=(gs_func->init_with_args)(interpreter,argc-1,argv+1);
+      if (status == 0)
+	{
+	  status=(gs_func->run_string)
+	    (interpreter,"systemdict /start get exec\n",0,&pexit_code);
+	  if ((status == 0) || (status <= -100))
+	    {
+	      char
+		reason[MaxTextExtent];
+
+	      FormatString(reason,"Ghostscript returns status %d, exit code %d",
+			   status,pexit_code);
+	      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"%s",reason);
+	      ThrowException(exception,DelegateError,PostscriptDelegateFailed,command);
+	    }
+	}
+      /*
+	Exit interpreter.
+      */
+      (gs_func->exit)(interpreter);
+      /*
+	Deallocate interpreter.
+      */
+      (gs_func->delete_instance)(interpreter);
+      for (i=0; i < argc; i++)
+	MagickFreeMemory(argv[i]);
+      MagickFreeMemory(argv);
+      if ((status == 0) || (status <= -100))
+	return(MagickFail);
+      return(MagickPass);
+    }
+#endif /* defined(HasGS) || defined(MSWINDOWS) */
+
+  status=MagickFail;
 #if defined(POSIX)
+  {
+    argv = StringToArgv(command,&argc);
+    if (argv == (char **) NULL)
       {
-        int arg_count;
-        char **arg_array;
-        arg_array = StringToArgv(command,&arg_count);
-        return MagickSpawnVP(verbose,arg_array[1],arg_array+1);
+	ThrowException(exception,DelegateError,
+		       FailedToAllocateArgumentList,
+		       command);
       }
+    else
+      {
+	if (MagickSpawnVP(verbose,argv[1],argv+1) == 0)
+	  status=MagickPass;
+	for (i=0; i < argc; i++)
+	  MagickFreeMemory(argv[i]);
+	MagickFreeMemory(argv);
+      }
+  }
 #else
-      return(SystemCommand(verbose,command));
-#endif
-    }
-  if (verbose)
-    {
-      (void) fputs("[ghostscript library]",stdout);
-      (void) fputs(strchr(command,' '),stdout);
-    }
-  status=(gs_func->new_instance)(&interpreter,(void *) NULL);
-  if (status < 0)
-    return(False);
-  argv=StringToArgv(command,&argc);
-  if (argv == (char **) NULL)
-    return(False);
-  status=(gs_func->init_with_args)(interpreter,argc-1,argv+1);
-  if (status == 0)
-    status=(gs_func->run_string)
-     (interpreter,"systemdict /start get exec\n",0,&code);
-  (gs_func->exit)(interpreter);
-  (gs_func->delete_instance)(interpreter);
-  for (i=0; i < argc; i++)
-    MagickFreeMemory(argv[i]);
-  MagickFreeMemory(argv);
-  if ((status == 0) || (status == -101))
-    return(False);
-  return(True);
-#else
-  return(SystemCommand(verbose,command));
+  if (SystemCommand(verbose,command) == 0)
+    status=MagickPass;
 #endif
+  return status;
 }
 
 /*
@@ -845,7 +972,7 @@ MagickExport unsigned int ListDelegateInfo(FILE *file,ExceptionInfo *exception)
       continue;
     *delegate='\0';
     if (p->encode != (char *) NULL)
-      (void) strncpy(delegate,p->encode,MaxTextExtent-1);
+      (void) strlcpy(delegate,p->encode,MaxTextExtent);
     (void) strcat(delegate,"        ");
     delegate[8]='\0';
     commands=StringToList(p->commands);
@@ -873,7 +1000,7 @@ MagickExport unsigned int ListDelegateInfo(FILE *file,ExceptionInfo *exception)
       for (s=commands[0]; length < command_length; s+=formatted_chars)
         {
           if (s != commands[0])
-            fprintf(file,"%*s",command_start_column,"");
+            (void) fprintf(file,"%*s",command_start_column,"");
           strip_length=screen_width-command_start_column;
           if (length+strip_length < command_length)
             {
@@ -885,7 +1012,7 @@ MagickExport unsigned int ListDelegateInfo(FILE *file,ExceptionInfo *exception)
             }
           formatted_chars=fprintf(file,"%.*s",strip_length,s);
           length+=formatted_chars;
-          fprintf(file,"\n");
+          (void) fprintf(file,"\n");
           if (formatted_chars <= 0)
             break;
         }
@@ -930,10 +1057,10 @@ MagickExport unsigned int ListDelegateInfo(FILE *file,ExceptionInfo *exception)
 %
 %
 */
-#if defined(WIN32)
+#if defined(MSWINDOWS)
 static void CatDelegatePath(char *path,
-                               const char *binpath,
-                               const char *command)
+			    const char *binpath,
+			    const char *command)
 {
   strcpy(path,binpath);
   strcat(path,command);
@@ -943,7 +1070,7 @@ static void CatDelegatePath(char *path,
   strcpy(path,command);
   return;
 }
-#endif /* defined(WIN32) */
+#endif /* defined(MSWINDOWS) */
 static unsigned int ReadConfigureFile(const char *basename,
   const unsigned long depth,ExceptionInfo *exception)
 {
@@ -976,7 +1103,7 @@ static unsigned int ReadConfigureFile(const char *basename,
       GetToken(q,&q,token);
       if (*token == '\0')
         break;
-      (void) strncpy(keyword,token,MaxTextExtent-1);
+      (void) strlcpy(keyword,token,MaxTextExtent);
       if (LocaleNCompare(keyword,"<!--",4) == 0)
         {
           /*
@@ -993,7 +1120,7 @@ static unsigned int ReadConfigureFile(const char *basename,
           */
           while ((*token != '>') && (*q != '\0'))
             {
-              (void) strncpy(keyword,token,MaxTextExtent-1);
+              (void) strlcpy(keyword,token,MaxTextExtent);
               GetToken(q,&q,token);
               if (*token != '=')
                 continue;
@@ -1009,9 +1136,8 @@ static unsigned int ReadConfigureFile(const char *basename,
 
                       GetPathComponent(path,HeadPath,filename);
                       if (*filename != '\0')
-                        (void) strcat(filename,DirectorySeparator);
-                      (void) strncat(filename,token,MaxTextExtent-
-                                     strlen(filename)-1);
+                        (void) strlcat(filename,DirectorySeparator,MaxTextExtent);
+                      (void) strlcat(filename,token,MaxTextExtent);
                       (void) ReadConfigureFile(filename,depth+1,exception);
                     }
                   if (delegate_list != (DelegateInfo *) NULL)
@@ -1061,7 +1187,7 @@ static unsigned int ReadConfigureFile(const char *basename,
             if (LocaleCompare((char *) keyword,"command") == 0)
               {
                 delegate_list->commands=AllocateString(token);
-#if defined(WIN32)
+#if defined(MSWINDOWS)
                 if (strchr(delegate_list->commands,'@') != (char *) NULL)
                   {
                     char
@@ -1135,7 +1261,7 @@ static unsigned int ReadConfigureFile(const char *basename,
                     SubstituteString((char **) &delegate_list->commands,
                                      "@HPGLDecodeDelegate@",path);
                   }
-#endif /* defined(WIN32) */
+#endif /* defined(MSWINDOWS) */
               } /* LocaleCompare */
             break;
           }
@@ -1179,11 +1305,6 @@ static unsigned int ReadConfigureFile(const char *basename,
         case 'S':
         case 's':
           {
-            if (LocaleCompare((char *) keyword,"spawn") == 0)
-              {
-                delegate_list->spawn=LocaleCompare(token,"True") == 0;
-                break;
-              }
             if (LocaleCompare((char *) keyword,"stealth") == 0)
               {
                 delegate_list->stealth=LocaleCompare(token,"True") == 0;
