@@ -46,6 +46,84 @@
 #include "magick/pixel_iterator.h"
 #include "magick/monitor.h"
 #include "magick/utility.h"
+
+static MagickPassFail
+BuildChannelHistogramsCB(void *mutable_data,          /* User provided mutable data */
+                         const void *immutable_data,  /* User provided immutable data */
+                         const Image *const_image,    /* Input image */
+                         const PixelPacket *pixels,   /* Pixel row */
+                         const IndexPacket *indexes,  /* Pixel indexes */
+                         const long npixels,          /* Number of pixels in row */
+                         ExceptionInfo *exception     /* Exception report */
+                         )
+{
+  /*
+    Built an image histogram in the supplied table.
+
+    Should be executed by just one thread in order to avoid contention
+    for the histogram table.
+  */
+  DoublePixelPacket
+    *histogram = (DoublePixelPacket *) mutable_data;
+
+  register long
+    i;
+
+  ARG_NOT_USED(immutable_data);
+  ARG_NOT_USED(indexes);
+  ARG_NOT_USED(exception);
+
+  for (i=0; i < npixels; i++)
+    {
+      histogram[ScaleQuantumToMap(pixels[i].red)].red++;
+      histogram[ScaleQuantumToMap(pixels[i].green)].green++;
+      histogram[ScaleQuantumToMap(pixels[i].blue)].blue++;
+      if (const_image->matte)
+        histogram[ScaleQuantumToMap(pixels[i].opacity)].opacity++;
+    }
+
+  return MagickPass;
+}
+
+static DoublePixelPacket *
+BuildChannelHistograms(const Image *image, ExceptionInfo *exception)
+{
+  MagickPassFail
+    status = MagickPass;
+
+  DoublePixelPacket
+    *histogram;
+  
+  histogram=MagickAllocateArray(DoublePixelPacket *,(MaxMap+1),
+                                sizeof(DoublePixelPacket));
+  if (histogram == (DoublePixelPacket *) NULL)
+    {
+      ThrowException(exception,ResourceLimitError,MemoryAllocationFailed,
+                     image->filename);
+      return (DoublePixelPacket *) NULL;
+    }
+
+  (void) memset(histogram,0,(MaxMap+1)*sizeof(DoublePixelPacket));
+
+  {
+    PixelIteratorOptions
+      iterator_options;
+    
+    InitializePixelIteratorOptions(&iterator_options,exception);
+    iterator_options.max_threads=1;
+    status=PixelIterateMonoRead(BuildChannelHistogramsCB,
+                                &iterator_options,
+                                "[%s] Building histogram...",
+                                histogram,NULL,
+                                0,0,image->columns,image->rows,
+                                image,exception);
+  }
+
+  if (status == MagickFail)
+    MagickFreeMemory(histogram);
+  
+  return histogram;
+}
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -85,9 +163,12 @@ ContrastImagePixels(void *mutable_data,         /* User provided mutable data */
   /*
     Modulate pixels contrast.
   */
-  int
-    sign = *((const int *) immutable_data);
+  const double
+    sign = *((const double *) immutable_data);
 
+  static const double
+    alpha=0.5+MagickEpsilon;
+      
   register long
     i;
   
@@ -97,15 +178,31 @@ ContrastImagePixels(void *mutable_data,         /* User provided mutable data */
   ARG_NOT_USED(exception);
 
   for (i=0; i < npixels; i++)
-    Contrast(sign,&pixels[i].red,&pixels[i].green,&pixels[i].blue);
+    {
+      double
+        brightness,
+        hue,
+        saturation;
+
+      TransformHSL(pixels[i].red,pixels[i].green,pixels[i].blue,
+                   &hue,&saturation,&brightness);
+      brightness+=
+        alpha*sign*(alpha*(sin(MagickPI*(brightness-alpha))+1.0)-brightness);
+      if (brightness > 1.0)
+        brightness=1.0;
+      if (brightness < 0.0)
+        brightness=0.0;
+      HSLTransform(hue,saturation,brightness,&pixels[i].red,
+                   &pixels[i].green,&pixels[i].blue);
+    }
 
   return MagickPass;
 }
-#define DullContrastImageText  "[%s] Dulling image contrast..."
-#define SharpenContrastImageText  "[%s] Sharpening image contrast..."
+#define DullContrastImageText "[%s] Dulling contrast..."
+#define SharpenContrastImageText "[%s] Sharpening contrast..."
 MagickExport MagickPassFail ContrastImage(Image *image,const unsigned int sharpen)
 {
-  int
+  double
     sign;
 
   const char
@@ -120,7 +217,7 @@ MagickExport MagickPassFail ContrastImage(Image *image,const unsigned int sharpe
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   is_grayscale=image->is_grayscale;
-  sign=sharpen ? 1 : -1;
+  sign=sharpen ? 1.0 : -1.0;
   progress_message=sharpen ? SharpenContrastImageText : DullContrastImageText;
   if (image->storage_class == PseudoClass)
     {
@@ -219,44 +316,7 @@ ApplyLevels(void *mutable_data,          /* User provided mutable data */
   return MagickPass;
 }
 
-static MagickPassFail
-BuildHistogram(void *mutable_data,          /* User provided mutable data */
-               const void *immutable_data,  /* User provided immutable data */
-               const Image *const_image,    /* Input image */
-               const PixelPacket *pixels,   /* Pixel row */
-               const IndexPacket *indexes,  /* Pixel indexes */
-               const long npixels,          /* Number of pixels in row */
-               ExceptionInfo *exception     /* Exception report */
-               )
-{
-  /*
-    Built an image histogram in the supplied table.
-
-    Should be executed by just one thread in order to avoid contention
-    for the histogram table.
-  */
-  DoublePixelPacket
-    *histogram = (DoublePixelPacket *) mutable_data;
-
-  register long
-    i;
-
-  ARG_NOT_USED(immutable_data);
-  ARG_NOT_USED(indexes);
-  ARG_NOT_USED(exception);
-
-  for (i=0; i < npixels; i++)
-    {
-      histogram[ScaleQuantumToMap(pixels[i].red)].red++;
-      histogram[ScaleQuantumToMap(pixels[i].green)].green++;
-      histogram[ScaleQuantumToMap(pixels[i].blue)].blue++;
-      if (const_image->matte)
-        histogram[ScaleQuantumToMap(pixels[i].opacity)].opacity++;
-    }
-
-  return MagickPass;
-}
-#define EqualizeImageText  "Equalizing image...  "
+#define EqualizeImageText "Equalize...  "
 MagickExport MagickPassFail EqualizeImage(Image *image)
 {
   DoublePixelPacket
@@ -279,37 +339,31 @@ MagickExport MagickPassFail EqualizeImage(Image *image)
     status=MagickPass;
 
   /*
-    Allocate and initialize histogram arrays.
+    Allocate and initialize arrays.
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   is_grayscale=image->is_grayscale;
-  histogram=MagickAllocateMemory(DoublePixelPacket *,
-    (MaxMap+1)*sizeof(DoublePixelPacket));
   map=MagickAllocateMemory(DoublePixelPacket *,(MaxMap+1)*sizeof(DoublePixelPacket));
   levels.map=MagickAllocateMemory(PixelPacket *,(MaxMap+1)*sizeof(PixelPacket));
-  if ((histogram == (DoublePixelPacket *) NULL) ||
-      (map == (DoublePixelPacket *) NULL) ||
+  if ((map == (DoublePixelPacket *) NULL) ||
       (levels.map == (PixelPacket *) NULL))
-    ThrowBinaryException(ResourceLimitError,MemoryAllocationFailed,
-      MagickMsg(OptionError,UnableToEqualizeImage));
+    {
+      MagickFreeMemory(map);
+      MagickFreeMemory(levels.map);
+      ThrowBinaryException(ResourceLimitError,MemoryAllocationFailed,
+                           MagickMsg(OptionError,UnableToEqualizeImage));
+    }
   /*
     Build histogram.
   */
-  (void) memset(histogram,0,(MaxMap+1)*sizeof(DoublePixelPacket));
-  {
-    PixelIteratorOptions
-      options;
-
-    InitializePixelIteratorOptions(&options,&image->exception);
-    options.max_threads=1;
-    status=PixelIterateMonoRead(BuildHistogram,
-                                &options,
-                                "[%s] Building image histogram...",
-                                histogram,NULL,
-                                0,0,image->columns,image->rows,
-                                image,&image->exception);
-  }
+  histogram=BuildChannelHistograms(image,&image->exception);
+  if (histogram == (DoublePixelPacket *) NULL)
+    {
+      MagickFreeMemory(map);
+      MagickFreeMemory(levels.map);
+      return MagickFail;
+    }
   /*
     Integrate the histogram to get the equalization map.
   */
@@ -453,6 +507,9 @@ MagickExport MagickPassFail GammaImage(Image *image,const char *level)
   levels.level_green = (gamma.green != 0.0);
   levels.level_blue = (gamma.blue != 0.0);
   levels.level_opacity = MagickFalse;
+#if (MaxMap > 256) && defined(HAVE_OPENMP)
+#  pragma omp parallel for schedule(guided)
+#endif
   for (i=0; i <= (long) MaxMap; i++)
   {
     if (levels.level_red)
@@ -525,7 +582,7 @@ MagickExport MagickPassFail GammaImage(Image *image,const char *level)
 %
 %
 */
-#define LevelImageText  "Leveling the image...  "
+#define LevelImageText "Level...  "
 MagickExport MagickPassFail LevelImage(Image *image,const char *levels)
 {
   double
@@ -738,7 +795,7 @@ MagickExport MagickPassFail LevelImageChannel(Image *image,
     {
       status=PixelIterateMonoModify(ApplyLevels,
                                     NULL,
-                                    "[%s] Leveling image...",
+                                    "[%s] Leveling channels...",
                                     NULL,&levels,
                                     0,0,image->columns,image->rows,
                                     image,
@@ -799,7 +856,7 @@ ModulateImagePixels(void *mutable_data,         /* User provided mutable data */
     Modulate image pixels according to options.
   */
   const ModulateImageParameters_t
-    *param = (const ModulateImageParameters_t *) immutable_data;
+    param = *(const ModulateImageParameters_t *) immutable_data;
 
   register long
     i;
@@ -810,15 +867,37 @@ ModulateImagePixels(void *mutable_data,         /* User provided mutable data */
   ARG_NOT_USED(exception);
 
   for (i=0; i < npixels; i++)
-    Modulate(param->percent_hue,param->percent_saturation,param->percent_brightness,
-             &pixels[i].red,&pixels[i].green,&pixels[i].blue);
+    {
+      double
+        brightness,
+        hue,
+        saturation;
+
+      TransformHSL(pixels[i].red,pixels[i].green,pixels[i].blue,
+                   &hue,&saturation,&brightness);
+      brightness*=(0.01+MagickEpsilon)*param.percent_brightness;
+      if (brightness > 1.0)
+        brightness=1.0;
+      saturation*=(0.01+MagickEpsilon)*param.percent_saturation;
+      if (saturation > 1.0)
+        saturation=1.0;
+      
+      hue += (param.percent_hue/200.0 - 0.5);
+      while (hue < 0.0)
+        hue += 1.0;
+      while (hue > 1.0)
+        hue -= 1.0;
+      HSLTransform(hue,saturation,brightness,
+                   &pixels[i].red,&pixels[i].green,&pixels[i].blue);
+    }
 
   return MagickPass;
 }
 
-#define ModulateImageText  "[%s] Modulating image..."
 MagickExport MagickPassFail ModulateImage(Image *image,const char *modulate)
 {
+  char
+    progress_message[MaxTextExtent];
 
   ModulateImageParameters_t
     param;
@@ -847,6 +926,10 @@ MagickExport MagickPassFail ModulateImage(Image *image,const char *modulate)
   param.percent_saturation=AbsoluteValue(param.percent_saturation);
   param.percent_hue=AbsoluteValue(param.percent_hue);
 
+  FormatString(progress_message,"[%%s] Modulate %g/%g/%g...",
+               param.percent_brightness,param.percent_saturation,
+               param.percent_hue);
+  TransformColorspace(image,RGBColorspace);
   if (image->storage_class == PseudoClass)
     {
       (void) ModulateImagePixels(NULL,&param,image,image->colormap,
@@ -856,7 +939,7 @@ MagickExport MagickPassFail ModulateImage(Image *image,const char *modulate)
     }
   else
     {
-      status=PixelIterateMonoModify(ModulateImagePixels,NULL,ModulateImageText,
+      status=PixelIterateMonoModify(ModulateImagePixels,NULL,progress_message,
                                     NULL,&param,0,0,image->columns,image->rows,
                                     image,&image->exception);
     }
@@ -942,7 +1025,7 @@ NegateImagePixels(void *mutable_data,         /* User provided mutable data */
   return MagickPass;
 }
 
-#define NegateImageText  "[%s] Negating the image colors..."
+#define NegateImageText "[%s] Negate..."
 MagickExport MagickPassFail NegateImage(Image *image,const unsigned int grayscale)
 {
   unsigned int
@@ -1031,30 +1114,19 @@ MagickExport MagickPassFail NormalizeImage(Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   is_grayscale=image->is_grayscale;
-  histogram=MagickAllocateMemory(DoublePixelPacket *,
-                                 (MaxMap+1)*sizeof(DoublePixelPacket));
   levels.map=MagickAllocateMemory(PixelPacket *,(MaxMap+1)*sizeof(PixelPacket));
-  if ((histogram == (DoublePixelPacket *) NULL) ||
-      (levels.map == (PixelPacket *) NULL))
+  if (levels.map == (PixelPacket *) NULL)
     ThrowBinaryException3(ResourceLimitError,MemoryAllocationFailed,
                           UnableToNormalizeImage);
   /*
     Form histogram.
   */
-  (void) memset(histogram,0,(MaxMap+1)*sizeof(DoublePixelPacket));
-  {
-    PixelIteratorOptions
-      options;
-
-    InitializePixelIteratorOptions(&options,&image->exception);
-    options.max_threads=1;
-    status=PixelIterateMonoRead(BuildHistogram,
-                                &options,
-                                "[%s] Building image histogram...",
-                                histogram,NULL,
-                                0,0,image->columns,image->rows,
-                                image,&image->exception);
-  }
+  histogram=BuildChannelHistograms(image,&image->exception);
+  if (histogram == (DoublePixelPacket *) NULL)
+    {
+      MagickFreeMemory(levels.map);
+      return MagickFail;
+    }
   /*
     Find the histogram boundaries by locating the 0.1 percent levels.
   */
