@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2008 GraphicsMagick Group
+% Copyright (C) 2008 - 2009 GraphicsMagick Group
 %
 % This program is covered by multiple licenses, which are described in
 % Copyright.txt. You should have received a copy of Copyright.txt with this
@@ -532,6 +532,10 @@ MagickExport CompressionType StringToCompressionType(const char *option)
     compression_type=NoCompression;
   else if (LocaleCompare("BZip",option) == 0)
     compression_type=BZipCompression;
+  else if (LocaleCompare("BZip2",option) == 0)
+    compression_type=BZipCompression;
+  else if (LocaleCompare("BZ2",option) == 0)
+    compression_type=BZipCompression;
   else if (LocaleCompare("Fax",option) == 0)
     compression_type=FaxCompression;
   else if (LocaleCompare("Group4",option) == 0)
@@ -546,13 +550,74 @@ MagickExport CompressionType StringToCompressionType(const char *option)
     compression_type=RLECompression;
   else if (LocaleCompare("Zip",option) == 0)
     compression_type=ZipCompression;
+  else if (LocaleCompare("GZip",option) == 0)
+    compression_type=ZipCompression;
 
   return compression_type;
 }
 
+/*
+  ConfirmAccessMode
+*/
+MagickExport const char *ConfirmAccessModeToString(const ConfirmAccessMode access_mode)
+{
+  const char
+    *mode_string="Undefined";
+
+  switch(access_mode)
+    {
+    case UndefinedConfirmAccessMode:
+      mode_string="Undefined";
+      break;
+    case FileExecuteConfirmAccessMode:
+      mode_string="Execute";
+      break;
+    case FileReadConfirmAccessMode:
+      mode_string="Read";
+      break;
+    case FileWriteConfirmAccessMode:
+      mode_string="Write";
+      break;
+    case URLGetFTPConfirmAccessMode:
+      mode_string="FTP Get";
+      break;
+    case URLGetFileConfirmAccessMode:
+      mode_string="FILE Get";
+      break;
+    case URLGetHTTPConfirmAccessMode:
+      mode_string="HTTP Get";
+      break;
+    }
+
+  return mode_string;
+}
+
 /*
   EndianType
 */
+MagickExport const char *EndianTypeToString(const EndianType endian_type)
+{
+  const char
+    *endian_string="Undefined";
+
+  switch (endian_type)
+    {
+    case UndefinedEndian:
+      endian_string="Undefined";
+      break;
+    case LSBEndian:
+      endian_string="LSB";
+      break;
+    case MSBEndian:
+      endian_string="MSB";
+      break;
+    case NativeEndian:
+      endian_string="Native";
+      break;
+    }
+
+  return endian_string;
+}
 MagickExport EndianType StringToEndianType(const char *option)
 {
   EndianType
@@ -775,6 +840,32 @@ MagickExport ImageType StringToImageType(const char *option)
 /*
   InterlaceType
 */
+MagickExport const char *InterlaceTypeToString(const InterlaceType interlace_type)
+{
+  const char
+    *interlace_string = "Undefined";
+
+  switch (interlace_type)
+    {
+    case UndefinedInterlace:
+      interlace_string = "Undefined";
+      break;
+    case NoInterlace:
+      interlace_string = "No";
+      break;
+    case LineInterlace:
+      interlace_string = "Line";
+      break;
+    case PlaneInterlace:
+      interlace_string = "Plane";
+      break;
+    case PartitionInterlace:
+      interlace_string = "Partition";
+      break;
+    }
+
+  return interlace_string;
+}
 MagickExport InterlaceType StringToInterlaceType(const char *option)
 {
   InterlaceType
@@ -997,39 +1088,50 @@ MagickExport QuantumOperator StringToQuantumOperator(const char *option)
     quantum_operator=AddQuantumOp;
   else if (LocaleCompare("and",option) == 0)
     quantum_operator=AndQuantumOp;
-  else if (LocaleCompare("assign",option) == 0)
+  else if ((LocaleCompare("assign",option) == 0) ||
+           (LocaleCompare("Set",option) == 0))
     quantum_operator=AssignQuantumOp;
   else if (LocaleCompare("divide",option) == 0)
     quantum_operator=DivideQuantumOp;
-  else if (LocaleCompare("lshift",option) == 0)
+  else if ((LocaleCompare("lshift",option) == 0) ||
+           (LocaleCompare("LeftShift",option) == 0))
     quantum_operator=LShiftQuantumOp;
   else if (LocaleCompare("multiply",option) == 0)
     quantum_operator=MultiplyQuantumOp;
   else if (LocaleCompare("or",option) == 0)
     quantum_operator=OrQuantumOp;
-  else if (LocaleCompare("rshift",option) == 0)
+  else if ((LocaleCompare("rshift",option) == 0) ||
+           (LocaleCompare("RightShift",option) == 0))
     quantum_operator=RShiftQuantumOp;
   else if (LocaleCompare("subtract",option) == 0)
     quantum_operator=SubtractQuantumOp;
   else if (LocaleCompare("threshold",option) == 0)
     quantum_operator=ThresholdQuantumOp;
-  else if (LocaleCompare("threshold-black",option) == 0)
+  else if ((LocaleCompare("threshold-black",option) == 0) ||
+           (LocaleCompare("ThresholdBlack",option) == 0))
     quantum_operator=ThresholdBlackQuantumOp;
-  else if (LocaleCompare("threshold-white",option) == 0)
+  else if ((LocaleCompare("threshold-white",option) == 0) ||
+           (LocaleCompare("ThresholdWhite",option) == 0))
     quantum_operator=ThresholdWhiteQuantumOp;
   else if (LocaleCompare("xor",option) == 0)
     quantum_operator=XorQuantumOp;
-  else if (LocaleCompare("noise-gaussian",option) == 0)
+  else if ((LocaleCompare("noise-gaussian",option) == 0) ||
+           (LocaleCompare("GaussianNoise",option) == 0))
     quantum_operator=NoiseGaussianQuantumOp;
-  else if (LocaleCompare("noise-impulse",option) == 0)
+  else if ((LocaleCompare("noise-impulse",option) == 0) ||
+           (LocaleCompare("ImpulseNoise",option) == 0))
     quantum_operator=NoiseImpulseQuantumOp;
-  else if (LocaleCompare("noise-laplacian",option) == 0)
+  else if ((LocaleCompare("noise-laplacian",option) == 0) ||
+           (LocaleCompare("LaplacianNoise",option) == 0))
     quantum_operator=NoiseLaplacianQuantumOp;
-  else if (LocaleCompare("noise-multiplicative",option) == 0)
+  else if ((LocaleCompare("noise-multiplicative",option) == 0) ||
+           (LocaleCompare("MultiplicativeNoise",option) == 0))
     quantum_operator=NoiseMultiplicativeQuantumOp;
-  else if (LocaleCompare("noise-poisson",option) == 0)
+  else if ((LocaleCompare("noise-poisson",option) == 0) ||
+           (LocaleCompare("PoissonNoise",option) == 0))
     quantum_operator=NoisePoissonQuantumOp;
-  else if (LocaleCompare("noise-uniform",option) == 0)
+  else if ((LocaleCompare("noise-uniform",option) == 0) ||
+           (LocaleCompare("UniformNoise",option) == 0))
     quantum_operator=NoiseUniformQuantumOp;
   else if (LocaleCompare("negate",option) == 0)
     quantum_operator=NegateQuantumOp;
@@ -1037,6 +1139,14 @@ MagickExport QuantumOperator StringToQuantumOperator(const char *option)
     quantum_operator=GammaQuantumOp;
   else if (LocaleCompare("depth",option) == 0)
     quantum_operator=DepthQuantumOp;
+  else if (LocaleCompare("log",option) == 0)
+    quantum_operator=LogQuantumOp;
+  else if (LocaleCompare("max",option) == 0)
+    quantum_operator=MaxQuantumOp;
+  else if (LocaleCompare("min",option) == 0)
+    quantum_operator=MinQuantumOp;
+  else if (LocaleCompare("pow",option) == 0)
+    quantum_operator=PowQuantumOp;
 
   return quantum_operator;
 }
@@ -1115,6 +1225,18 @@ MagickExport const char *QuantumOperatorToString(const QuantumOperator quantum_o
     case DepthQuantumOp:
       operator_text="depth";
       break;
+    case LogQuantumOp:
+      operator_text="log";
+      break;
+    case MaxQuantumOp:
+      operator_text="max";
+      break;
+    case MinQuantumOp:
+      operator_text="min";
+      break;
+    case PowQuantumOp:
+      operator_text="pow";
+      break;
     }
 
   return operator_text;
@@ -1195,6 +1317,8 @@ MagickExport ResourceType StringToResourceType(const char *option)
     resource_type=DiskResource;
   else if (LocaleCompare("File",option) == 0)
     resource_type=FileResource;
+  else if (LocaleCompare("Files",option) == 0)
+    resource_type=FileResource;
   else if (LocaleCompare("Map",option) == 0)
     resource_type=MapResource;
   else if (LocaleCompare("Memory",option) == 0)
