@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 GraphicsMagick Group
+% Copyright (C) 2003-2009 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -36,8 +36,8 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/pixel_cache.h"
 #include "magick/gem.h"
+#include "magick/utility.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -137,6 +137,150 @@ MagickExport double ExpandAffine(const AffineMatrix *affine)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e n e r a t e D i f f e r e n t i a l N o i s e                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  Method GenerateDifferentialNoise generates a differential floating-point
+%  noise value which will produce in the final result when added to the
+%  original pixel.  The floating point differential value is useful since
+%  it allows scaling without loss of precision and avoids clipping.
+%
+%  The format of the GenerateDifferentialNoise method is:
+%
+%      double GenerateDifferentialNoise(const Quantum pixel,
+%                                       const NoiseType noise_type,
+%                                       MagickRandomKernel *kernel)
+%
+%  A description of each parameter follows:
+%
+%    o pixel: A structure of type Quantum.
+%
+%    o noise_type:  The type of noise: Uniform, gaussian,
+%      multiplicative Gaussian, impulse, laplacian, or Poisson.
+%
+%    o kernel: Kernel for random number generator.
+%
+*/
+#define NoiseEpsilon   1.0e-5
+#define SigmaUniform   4.0
+#define SigmaGaussian  4.0
+#define SigmaImpulse   0.10
+#define SigmaLaplacian 10.0
+#define SigmaMultiplicativeGaussian  0.5
+#define SigmaPoisson   0.05
+#define TauGaussian    20.0
+
+MagickExport double GenerateDifferentialNoise(const Quantum quantum_pixel,
+					      const NoiseType noise_type,
+					      MagickRandomKernel *kernel)
+{
+  double
+    alpha,
+    beta,
+    pixel,
+    sigma,
+    value;
+
+  pixel=(double) quantum_pixel;
+
+#if QuantumDepth > 8
+  pixel /= MaxRGBDouble/255.0;
+#endif
+
+  alpha=MagickRandomRealInlined(kernel);
+  if (alpha == 0.0)
+    alpha=1.0;
+  switch (noise_type)
+  {
+    case UniformNoise:
+    default:
+    {
+      value=SigmaUniform*(alpha-0.5);
+      break;
+    }
+    case GaussianNoise:
+    {
+      double
+        tau;
+
+      beta=MagickRandomRealInlined(kernel);
+      sigma=sqrt(-2.0*log(alpha))*cos(2.0*MagickPI*beta);
+      tau=sqrt(-2.0*log(alpha))*sin(2.0*MagickPI*beta);
+      value=sqrt((double) pixel)*SigmaGaussian*sigma+TauGaussian*tau;
+      break;
+    }
+    case MultiplicativeGaussianNoise:
+    {
+      if (alpha <= NoiseEpsilon)
+        sigma=255.0;
+      else
+        sigma=sqrt(-2.0*log(alpha));
+      beta=MagickRandomRealInlined(kernel);
+      value=pixel*SigmaMultiplicativeGaussian*sigma*cos(2.0*MagickPI*beta);
+      break;
+    }
+    case ImpulseNoise:
+    {
+      if (alpha < (SigmaImpulse/2.0))
+        value=-pixel;
+       else
+         if (alpha >= (1.0-(SigmaImpulse/2.0)))
+           value=255.0-pixel;
+         else
+           value=0.0;
+      break;
+    }
+    case LaplacianNoise:
+    {
+      if (alpha <= 0.5)
+        {
+          if (alpha <= NoiseEpsilon)
+            value=-255.0;
+          else
+            value=SigmaLaplacian*log(2.0*alpha);
+          break;
+        }
+      beta=1.0-alpha;
+      if (beta <= (0.5*NoiseEpsilon))
+        value=255.0;
+      else
+        value=-(SigmaLaplacian*log(2.0*beta));
+      break;
+    }
+    case PoissonNoise:
+    {
+      double
+        limit;
+
+      register long
+        i;
+
+      limit=exp(-SigmaPoisson*(double) pixel);
+      for (i=0; alpha > limit; i++)
+      {
+        beta=MagickRandomRealInlined(kernel);
+        alpha=alpha*beta;
+      }
+      value=pixel-((double) i/SigmaPoisson);
+      break;
+    }
+  }
+
+#if QuantumDepth > 8
+  value *= (MaxRGBFloat/255.0);
+#endif
+
+  return value;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   G e n e r a t e N o i s e                                                 %
 %                                                                             %
 %                                                                             %
@@ -153,121 +297,19 @@ MagickExport double ExpandAffine(const AffineMatrix *affine)
 %
 %    o pixel: A structure of type Quantum.
 %
-%    o noise_type:  The type of noise: Gaussian, multiplicative Gaussian,
-%      impulse, laplacian, or Poisson.
+%    o noise_type:  The type of noise: Uniform, gaussian,
+%      multiplicative Gaussian, impulse, laplacian, or Poisson.
 %
 %
 */
-#define NoiseEpsilon   1.0e-5
-#define SigmaUniform   4.0
-#define SigmaGaussian  4.0
-#define SigmaImpulse   0.10
-#define SigmaLaplacian 10.0
-#define SigmaMultiplicativeGaussian  0.5
-#define SigmaPoisson   0.05
-#define TauGaussian    20.0
-
-MagickExport Quantum GenerateNoise(const Quantum quantum_pixel,
+MagickExport Quantum GenerateNoise(const Quantum pixel,
                                    const NoiseType noise_type)
 {
   double
-    alpha,
-    beta,
-    pixel,
-    sigma,
     value;
 
-  pixel=(double) quantum_pixel;
-
-#if QuantumDepth > 8
-  pixel /= MaxRGBDouble/255.0;
-#endif
-
-  alpha=(double) rand()/RAND_MAX;
-  if (alpha == 0.0)
-    alpha=1.0;
-  switch (noise_type)
-  {
-    case UniformNoise:
-    default:
-    {
-      value=(double) pixel+SigmaUniform*(alpha-0.5);
-      break;
-    }
-    case GaussianNoise:
-    {
-      double
-        tau;
-
-      beta=(double) rand()/RAND_MAX;
-      sigma=sqrt(-2.0*log(alpha))*cos(2.0*MagickPI*beta);
-      tau=sqrt(-2.0*log(alpha))*sin(2.0*MagickPI*beta);
-      value=(double) pixel+sqrt((double) pixel)*SigmaGaussian*sigma+
-        TauGaussian*tau;
-      break;
-    }
-    case MultiplicativeGaussianNoise:
-    {
-      if (alpha <= NoiseEpsilon)
-        sigma=255.0;
-      else
-        sigma=sqrt(-2.0*log(alpha));
-      beta=(double) rand()/RAND_MAX;
-      value=(double) pixel+pixel*SigmaMultiplicativeGaussian*sigma*
-        cos(2.0*MagickPI*beta);
-      break;
-    }
-    case ImpulseNoise:
-    {
-      if (alpha < (SigmaImpulse/2.0))
-        value=0;
-       else
-         if (alpha >= (1.0-(SigmaImpulse/2.0)))
-           value=255.0;
-         else
-           value=pixel;
-      break;
-    }
-    case LaplacianNoise:
-    {
-      if (alpha <= 0.5)
-        {
-          if (alpha <= NoiseEpsilon)
-            value=(double) pixel-255.0;
-          else
-            value=(double) pixel+SigmaLaplacian*log(2.0*alpha);
-          break;
-        }
-      beta=1.0-alpha;
-      if (beta <= (0.5*NoiseEpsilon))
-        value=(double) pixel+255.0;
-      else
-        value=(double) pixel-SigmaLaplacian*log(2.0*beta);
-      break;
-    }
-    case PoissonNoise:
-    {
-      double
-        limit;
-
-      register long
-        i;
-
-      limit=exp(-SigmaPoisson*(double) pixel);
-      for (i=0; alpha > limit; i++)
-      {
-        beta=(double) rand()/RAND_MAX;
-        alpha=alpha*beta;
-      }
-      value=(double) i/SigmaPoisson;
-      break;
-    }
-  }
-
-#if QuantumDepth > 8
-  value *= (MaxRGBFloat/255.0);
-#endif
-
+  value=(double) pixel+GenerateDifferentialNoise(pixel,noise_type,
+						 AcquireMagickRandomKernel());
   return (RoundDoubleToQuantum(value));
 }
 
@@ -324,7 +366,7 @@ MagickExport int GetOptimalKernelWidth1D(const double radius,const double sigma)
       normalize+=exp(-((double) u*u)/(2.0*sigma*sigma))/(MagickSQ2PI*sigma);
     u=width/2;
     value=exp(-((double) u*u)/(2.0*sigma*sigma))/(MagickSQ2PI*sigma)/normalize;
-    if ((long) (MaxRGB*value) <= 0)
+    if ((long) (MaxRGBDouble*value) <= 0)
       break;
     width+=2;
   }
@@ -360,7 +402,7 @@ MagickExport int GetOptimalKernelWidth2D(const double radius,const double sigma)
     }
     v=width/2;
     value=exp(-((double) v*v)/(2.0*sigma*sigma))/(MagickSQ2PI*sigma)/normalize;
-    if ((long) (MaxRGB*value) <= 0)
+    if ((long) (MaxRGBDouble*value) <= 0)
       break;
     width+=2;
   }
@@ -412,8 +454,8 @@ MagickExport void HSLTransform(const double hue,const double saturation,
   assert(blue != (Quantum *) NULL);
   if (saturation == 0.0)
     {
-      double l = MaxRGB*luminosity;
-      *red=*green=*blue= RoundToQuantum(l);
+      double l = MaxRGBDouble*luminosity;
+      *red=*green=*blue= RoundDoubleToQuantum(l);
     }
   else
     {
@@ -454,12 +496,12 @@ MagickExport void HSLTransform(const double hue,const double saturation,
         case 5: r=v; g=y; b=z; break;
         default: r=v; g=x; b=y; break;
         }
-      r *= MaxRGB;
-      *red=RoundToQuantum(r);
-      g *= MaxRGB;
-      *green=RoundToQuantum(g);
-      b *= MaxRGB;
-      *blue=RoundToQuantum(b);
+      r *= MaxRGBDouble;
+      *red=RoundDoubleToQuantum(r);
+      g *= MaxRGBDouble;
+      *green=RoundDoubleToQuantum(g);
+      b *= MaxRGBDouble;
+      *blue=RoundDoubleToQuantum(b);
     }
 }
 
@@ -518,8 +560,8 @@ MagickExport void HWBTransform(const double hue,const double whiteness,
   v=1.0-blackness;
   if (hue == 0.0)
     {
-      v *= MaxRGB;
-      *red=*green=*blue=RoundSignedToQuantum(v);
+      v *= MaxRGBDouble;
+      *red=*green=*blue=RoundDoubleToQuantum(v);
       return;
     }
   i=(unsigned int) (6.0*hue);
@@ -538,12 +580,12 @@ MagickExport void HWBTransform(const double hue,const double whiteness,
     case 4: r=n; g=whiteness; b=v; break;
     case 5: r=v; g=whiteness; b=n; break;
   }
-  r *= MaxRGB;
-  g *= MaxRGB;
-  b *= MaxRGB;
-  *red=RoundSignedToQuantum(r);
-  *green=RoundSignedToQuantum(g);
-  *blue=RoundSignedToQuantum(b);
+  r *= MaxRGBDouble;
+  g *= MaxRGBDouble;
+  b *= MaxRGBDouble;
+  *red=RoundDoubleToQuantum(r);
+  *green=RoundDoubleToQuantum(g);
+  *blue=RoundDoubleToQuantum(b);
 }
 
 /*
@@ -713,69 +755,6 @@ MagickExport void IdentityAffine(AffineMatrix *affine)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   I n t e r p o l a t e C o l o r                                           %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  Method InterpolateColor applies bi-linear interpolation between a pixel and
-%  it's neighbors.
-%
-%  The format of the InterpolateColor method is:
-%
-%      PixelPacket InterpolateColor(const Image *image,const double x_offset,
-%        const double y_offset,ExceptionInfo *exception)
-%
-%  A description of each parameter follows:
-%
-%    o image: The image.
-%
-%    o x_offset,y_offset: A double representing the current (x,y) position of
-%      the pixel.
-%
-%
-*/
-MagickExport PixelPacket InterpolateColor(const Image *image,
-  const double x_offset,const double y_offset,ExceptionInfo *exception)
-{
-  double
-    alpha,
-    beta,
-    one_minus_alpha,
-    one_minus_beta;
-
-  PixelPacket
-    color;
-
-  register const PixelPacket
-    *p;
-
-  assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  p=AcquireImagePixels(image,(long) x_offset,(long) y_offset,2,2,exception);
-  if (p == (const PixelPacket *) NULL)
-    return(AcquireOnePixel(image,(long) x_offset,(long) y_offset,exception));
-  alpha=x_offset-floor(x_offset);
-  beta=y_offset-floor(y_offset);
-  one_minus_alpha=1.0-alpha;
-  one_minus_beta=1.0-beta;
-  color.red=(Quantum) (one_minus_beta*(one_minus_alpha*p[0].red+
-    alpha*p[1].red)+beta*(one_minus_alpha*p[2].red+alpha*p[3].red)+0.5);
-  color.green=(Quantum) (one_minus_beta*(one_minus_alpha*p[0].green+
-    alpha*p[1].green)+beta*(one_minus_alpha*p[2].green+alpha*p[3].green)+0.5);
-  color.blue=(Quantum) (one_minus_beta*(one_minus_alpha*p[0].blue+
-    alpha*p[1].blue)+beta*(one_minus_alpha*p[2].blue+alpha*p[3].blue)+0.5);
-  color.opacity=(Quantum) (one_minus_beta*(one_minus_alpha*p[0].opacity+
-    alpha*p[1].opacity)+beta*(one_minus_alpha*p[2].opacity+alpha*p[3].opacity)+0.5);
-  return(color);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   M o d u l a t e                                                           %
 %                                                                             %
 %                                                                             %
@@ -885,9 +864,9 @@ MagickExport void TransformHSL(const Quantum red,const Quantum green,
   assert(saturation_result != (double *) NULL);
   assert(luminosity_result != (double *) NULL);
 
-  r=(double) red/MaxRGB;
-  g=(double) green/MaxRGB;
-  b=(double) blue/MaxRGB;
+  r=(double) red/MaxRGBDouble;
+  g=(double) green/MaxRGBDouble;
+  b=(double) blue/MaxRGBDouble;
   max=Max(r,Max(g,b));
   min=Min(r,Min(g,b));
   hue=0.0;
@@ -964,7 +943,7 @@ MagickExport void TransformHWB(const Quantum red,const Quantum green, const Quan
   assert(blackness != (double *) NULL);
   w=(double) Min(red,Min(green,blue));
   v=(double) Max(red,Max(green,blue));
-  *blackness=(double) (MaxRGB-v)/MaxRGB;
+  *blackness=((double) MaxRGBDouble-v)/MaxRGBDouble;
   if (v == w)
     {
       *hue=0.0;
@@ -977,6 +956,6 @@ MagickExport void TransformHWB(const Quantum red,const Quantum green, const Quan
          (double) red-green);
       i=(red == w) ? 3 : ((green == w) ? 5 : 1);
       *hue=((double) i-f/(v-w))/6.0;
-      *whiteness=((double) w/MaxRGB);
+      *whiteness=((double) w/MaxRGBDouble);
     }
 }
