@@ -49,6 +49,7 @@
 #include "magick/magick.h"
 #include "magick/module.h"
 #include "magick/pixel_cache.h"
+#include "magick/random.h"
 #include "magick/registry.h"
 #include "magick/resource.h"
 #include "magick/render.h"
@@ -71,6 +72,9 @@ typedef enum  {
   InitInitialized
 } MagickInitializationState;
 
+static void
+  DestroyMagickInfoList(void);
+
 static SemaphoreInfo
   *magick_semaphore = (SemaphoreInfo *) NULL;
 
@@ -78,6 +82,37 @@ static MagickInfo
   *magick_list = (MagickInfo *) NULL;
 
 static MagickInitializationState MagickInitialized = InitDefault;
+static CoderClass MinimumCoderClass = UnstableCoderClass;
+
+static void DestroyMagickInfo(MagickInfo** magick_info);
+static void DestroyMagickInfoList(void);
+
+/*
+  Block size to use when accessing filesystem.
+
+  Adjust via MAGICK_IOBUF_SIZE environment variable.
+*/
+
+static size_t filesystem_blocksize=16384;
+
+/*
+  Get blocksize to use when accessing the filesystem.
+*/
+size_t
+MagickGetFileSystemBlockSize(void)
+{
+  return filesystem_blocksize;
+}
+
+/*
+  Set blocksize to use when accessing the filesystem.
+*/
+void
+MagickSetFileSystemBlockSize(const size_t block_size)
+{
+  filesystem_blocksize=block_size;
+}
+
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -106,26 +141,31 @@ MagickExport void DestroyMagick(void)
 #if defined(HasX11)
   MagickXDestroyX11Resources();
 #endif
-  DestroyColorInfo();
-  DestroyDelegateInfo();
-  DestroyTypeInfo();
-  DestroyMagicInfo();
-  DestroyMagickInfo();
-  DestroyConstitute();
-  DestroyMagickRegistry();
-  DestroyMagickResources();
-  DestroyTemporaryFiles();
+  DestroyColorInfo();           /* Color database */
+  DestroyDelegateInfo();        /* External delegate information */
+  DestroyTypeInfo();            /* Font information */
+  DestroyMagicInfo();           /* File format detection */
+  DestroyMagickInfoList();      /* Coder registrations + modules */
+  DestroyConstitute();          /* Constitute semaphore */
+  DestroyMagickRegistry();      /* Registered images */
+  DestroyMagickResources();     /* Resource semaphore */
+  DestroyMagickRandomGenerator(); /* Random number generator */
+  DestroyTemporaryFiles();      /* Temporary files */
 #if defined(MSWINDOWS)
-  NTGhostscriptUnLoadDLL();
+  NTGhostscriptUnLoadDLL();     /* Ghostscript DLL */
 #endif /* defined(MSWINDOWS) */
   /*
     Destroy logging last since some components log their destruction.
   */
-  DestroyLogInfo();
-  DestroySemaphore();
+  DestroyLogInfo();             /* Logging configuration */
+  DestroySemaphore();           /* Semaphores framework */
 
   MagickInitialized=InitUninitialized;
+
 }
+/*
+  Destroy MagickInfo structure.
+*/
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -138,14 +178,50 @@ MagickExport void DestroyMagick(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  DestroyMagickInfo() deallocates memory associated with the MagickInfo list.
+%  DestroyMagickInfo() destroys a MagickInfo structure.
 %
-%  The format of the DestroyMagickInfo method is:
+%  The format of the DestroyMagickInfoList method is:
 %
-%      void DestroyMagickInfo(void)
+%      void DestroyMagickInfo(MagickInfo** magick_info)
 %
 */
-MagickExport void DestroyMagickInfo(void)
+static void DestroyMagickInfo(MagickInfo** magick_info)
+{
+  MagickInfo
+    *p;
+
+  p=*magick_info;
+  if (p)
+    {
+      p->name=0;
+      p->description=0;
+      p->version=0;
+      p->note=0;
+      p->module=0;
+      MagickFreeMemory(p);
+      *magick_info=p;
+    }
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   D e s t r o y M a g i c k I n f o L i s t                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  DestroyMagickInfo() deallocates memory associated with the MagickInfo list.
+%
+%  The format of the DestroyMagickInfoList method is:
+%
+%      void DestroyMagickInfoList(void)
+%
+*/
+static void DestroyMagickInfoList(void)
 {
   MagickInfo
     *magick_info;
@@ -166,19 +242,20 @@ MagickExport void DestroyMagickInfo(void)
     entries anyway.
   */
   AcquireSemaphoreInfo(&magick_semaphore);
+#if defined(BuildMagickModules)
+  if (magick_list != (MagickInfo *) NULL)
+    (void) printf("Warning: module registrations are still present!\n");
+#endif /* defined(BuildMagickModules) */
   for (p=magick_list; p != (MagickInfo *) NULL; )
   {
     magick_info=p;
     p=p->next;
 
+#if !defined(BuildMagickModules)
     (void) printf("Warning: module registration for \"%s\" from module \"%s\" still present!\n",
                   magick_info->name, magick_info->module);
-    MagickFreeMemory(magick_info->name);
-    MagickFreeMemory(magick_info->description);
-    MagickFreeMemory(magick_info->version);
-    MagickFreeMemory(magick_info->note);
-    MagickFreeMemory(magick_info->module);
-    MagickFreeMemory(magick_info);
+#endif /* !defined(BuildMagickModules) */
+    DestroyMagickInfo(&magick_info);
   }
   magick_list=(MagickInfo *) NULL;
   LiberateSemaphoreInfo(&magick_semaphore);
@@ -459,6 +536,8 @@ MagickExport MagickInfo **GetMagickInfoArray(ExceptionInfo *exception)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  InitializeMagick() initializes the GraphicsMagick environment.
+%  InitializeMagick() MUST be invoked by the using program before making
+%  use of GraphicsMagick functions or else the library may be unstable.
 %
 %  The format of the InitializeMagick function is:
 %
@@ -543,6 +622,16 @@ static Sigfunc *MagickCondSignal(int signo, Sigfunc *func)
   return (o_handler);
 }
 
+/*
+  Signal handler which does nothing.
+*/
+#if 0
+static RETSIGTYPE MagickIgnoreSignalHandler(int signo)
+{
+  fprintf(stderr,"Caught ignored signal %d\n", signo);
+}
+#endif
+
 /*
   Signal handler to ensure that DestroyMagick is invoked in case the
   user aborts the program.
@@ -727,6 +816,13 @@ MagickExport void InitializeMagickClientPathAndName(const char *ARGUNUSED(path))
 */
 MagickExport void InitializeMagickSignalHandlers(void)
 {
+#if 0
+  /* termination of child process */
+#if defined(SIGCHLD)
+  (void) MagickCondSignal(SIGCHLD,MagickIgnoreSignalHandler);
+#endif
+#endif
+
   /* hangup, default terminate */
 #if defined(SIGHUP)
   (void) MagickCondSignal(SIGHUP,MagickSignalHandler);
@@ -763,49 +859,68 @@ MagickExport void InitializeMagickSignalHandlers(void)
 
 MagickExport void InitializeMagick(const char *path)
 {
+  const char
+    *p;
+
   /* NOTE: This routine sets up the path to the client which needs to
      be determined before almost anything else works right. This also
      includes logging!!! So we can't start logging until the path is
      actually saved. As soon as we know what the path is we make the
      same call to DefineClientSettings to set it up. Please make sure
      that this rule is followed in any future updates the this code!!!
-   */
+  */
   if (MagickInitialized == InitInitialized)
     return;
   SPINLOCK_WAIT;
   MagickInitialized=InitInitialized;
   SPINLOCK_RELEASE;
-
+  
 #if defined(MSWINDOWS)
 # if defined(_DEBUG) && !defined(__BORLANDC__)
   {
     int
       debug;
-
+    
     debug=_CrtSetDbgFlag(_CRTDBG_REPORT_FLAG);
     debug|=_CRTDBG_CHECK_ALWAYS_DF | _CRTDBG_DELAY_FREE_MEM_DF |
-            _CRTDBG_LEAK_CHECK_DF;
+      _CRTDBG_LEAK_CHECK_DF;
     // debug=_CrtSetDbgFlag(debug);
     // _ASSERTE(_CrtCheckMemory());
   }
 # endif /* defined(_DEBUG) */
 #endif /* defined(MSWINDOWS) */
-
+  
   (void) setlocale(LC_ALL,"");
   (void) setlocale(LC_NUMERIC,"C");
 
-  /* Seed the random number generator */
-  srand(time(0));
-
   /* Initialize semaphores */
   InitializeSemaphore();
 
+  /* Seed the random number generator */
+  srand(MagickRandNewSeed());
+
+  /* Initialize our random number generator */
+  InitializeMagickRandomGenerator();
+
   /*
     Set logging flags using the value of MAGICK_DEBUG if it is set in
     the environment.
   */
-  if (getenv("MAGICK_DEBUG"))
-    (void) SetLogEventMask(getenv("MAGICK_DEBUG"));
+  if ((p=getenv("MAGICK_DEBUG")) != (const char *) NULL)
+    (void) SetLogEventMask(p);
+
+  /*
+    Set the filesystem block size.
+  */
+  {
+    size_t
+      block_size=16384;
+    
+    if ((p=getenv("MAGICK_IOBUF_SIZE")) != (const char *) NULL)
+      block_size = (size_t) atol(p);
+    
+    MagickSetFileSystemBlockSize(block_size);
+  }
 
   /*
     Establish the path, filename, and display name of the client app
@@ -822,7 +937,8 @@ MagickExport void InitializeMagick(const char *path)
     Set logging flags using the value of MAGICK_DEBUG if it is set in
     the environment.
   */
-  (void) SetLogEventMask(getenv("MAGICK_DEBUG"));
+  if ((p=getenv("MAGICK_DEBUG")) != (const char *) NULL)
+    (void) SetLogEventMask(p);
 
   /*
     Establish signal handlers for common signals
@@ -837,6 +953,22 @@ MagickExport void InitializeMagick(const char *path)
      to the client is setup */
   InitializeMagickResources();
 
+  /* Initialize magick registry */
+  InitializeMagickRegistry();
+  
+  /*
+    Adjust minimum coder class if requested.
+  */
+  if ((p=getenv("MAGICK_CODER_STABILITY")) != (const char *) NULL)
+    {
+      if (LocaleCompare(p,"UNSTABLE") == 0)
+        MinimumCoderClass=UnstableCoderClass;
+      else if (LocaleCompare(p,"STABLE") == 0)
+        MinimumCoderClass=StableCoderClass;
+      else if (LocaleCompare(p,"PRIMARY") == 0)
+        MinimumCoderClass=PrimaryCoderClass;
+    }
+
   /*
     Initialize module loader
   */
@@ -866,7 +998,7 @@ MagickExport void InitializeMagick(const char *path)
 %
 %  The format of the IsMagickConflict method is:
 %
-%      unsigned int IsMagickConflict(const char *magick)
+%      MagickBool IsMagickConflict(const char *magick)
 %
 %  A description of each parameter follows:
 %
@@ -877,13 +1009,13 @@ MagickExport void InitializeMagick(const char *path)
 %
 %
 */
-MagickExport int unsigned IsMagickConflict(const char *magick)
+MagickExport MagickBool IsMagickConflict(const char *magick)
 {
   assert(magick != (char *) NULL);
 #if defined(MSWINDOWS) || defined(__CYGWIN__)
   return(NTIsMagickConflict(magick));
 #endif
-  return(False);
+  return(MagickFalse);
 }
 
 /*
@@ -901,7 +1033,7 @@ MagickExport int unsigned IsMagickConflict(const char *magick)
 %
 %  The format of the ListMagickInfo method is:
 %
-%      unsigned int ListMagickInfo(FILE *file,ExceptionInfo *exception)
+%      MagickPassFail ListMagickInfo(FILE *file,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -911,7 +1043,7 @@ MagickExport int unsigned IsMagickConflict(const char *magick)
 %
 %
 */
-MagickExport unsigned int ListMagickInfo(FILE *file,ExceptionInfo *exception)
+MagickExport MagickPassFail ListMagickInfo(FILE *file,ExceptionInfo *exception)
 {
 
   MagickInfo
@@ -927,16 +1059,21 @@ MagickExport unsigned int ListMagickInfo(FILE *file,ExceptionInfo *exception)
   if (!magick_array)
     return False;
 
-  (void) fprintf(file,"   Format  Mode  Description\n");
+  (void) fprintf(file,"   Format L  Mode  Description\n");
   (void) fprintf(file,"--------------------------------------------------------"
-    "-----------------------\n");
+    "------------------------\n");
   for (i=0; magick_array[i] != 0; i++)
   {
     if (magick_array[i]->stealth)
       continue;
-    (void) fprintf(file,"%9s%c  %c%c%c",magick_array[i]->name ? magick_array[i]->name : "",
-      magick_array[i]->blob_support ? '*' : ' ',magick_array[i]->decoder ? 'r' : '-',
-      magick_array[i]->encoder ? 'w' : '-',magick_array[i]->encoder && magick_array[i]->adjoin ? '+' : '-');
+    (void) fprintf(file,"%9s %c  %c%c%c",
+                   magick_array[i]->name ? magick_array[i]->name : "",
+                   (magick_array[i]->coder_class == PrimaryCoderClass ? 'P' :
+                    (magick_array[i]->coder_class == StableCoderClass ? 'S' :
+                     'U')),
+                   (magick_array[i]->decoder ? 'r' : '-'),
+                   (magick_array[i]->encoder ? 'w' : '-'),
+                   ((magick_array[i]->encoder && magick_array[i]->adjoin) ? '+' : '-'));
     if (magick_array[i]->description != (char *) NULL)
       (void) fprintf(file,"  %.1024s",magick_array[i]->description);
     if (magick_array[i]->version != (char *) NULL)
@@ -951,21 +1088,21 @@ MagickExport unsigned int ListMagickInfo(FILE *file,ExceptionInfo *exception)
         if (text != (char **) NULL)
           {
             register long
-              i;
+              j;
 
-            for (i=0; text[i] != (char *) NULL; i++)
+            for (j=0; text[j] != (char *) NULL; j++)
               {
-                (void) fprintf(file,"            %.1024s\n",text[i]);
-                MagickFreeMemory(text[i]);
+                (void) fprintf(file,"            %.1024s\n",text[j]);
+                MagickFreeMemory(text[j]);
               }
             MagickFreeMemory(text);
           }
       }
   }
-  (void) fprintf(file,"\n* native blob support\n\n");
+  (void) fprintf(file,"\n Meaning of 'L': P=Primary, S=Stable, U=Unstable\n");
   (void) fflush(file);
   MagickFreeMemory(magick_array);
-  return(True);
+  return(MagickPass);
 }
 
 /*
@@ -984,7 +1121,7 @@ MagickExport unsigned int ListMagickInfo(FILE *file,ExceptionInfo *exception)
 %
 %  The format of the ListModuleMap method is:
 %
-%      unsigned int ListModuleMap(FILE *file,ExceptionInfo *exception)
+%      MagickPassFail ListModuleMap(FILE *file,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -994,7 +1131,7 @@ MagickExport unsigned int ListMagickInfo(FILE *file,ExceptionInfo *exception)
 %
 %
 */
-MagickExport unsigned int ListModuleMap(FILE *file,ExceptionInfo *exception)
+MagickExport MagickPassFail ListModuleMap(FILE *file,ExceptionInfo *exception)
 {
   MagickInfo
     **magick_array;
@@ -1007,7 +1144,7 @@ MagickExport unsigned int ListModuleMap(FILE *file,ExceptionInfo *exception)
 
    magick_array=GetMagickInfoArray(exception);
    if ((!magick_array) || (exception->severity > UndefinedException))
-     return False;
+     return MagickFail;
 
    (void) fprintf(file, "<?xml version=\"1.0\"?>\n");
    (void) fprintf(file, "<!-- %s -->\n",GetMagickCopyright());
@@ -1030,7 +1167,7 @@ MagickExport unsigned int ListModuleMap(FILE *file,ExceptionInfo *exception)
 
    MagickFreeMemory(magick_array);
 
-   return(True);
+   return(MagickPass);
 }
 
 /*
@@ -1152,15 +1289,27 @@ MagickExport MagickInfo *RegisterMagickInfo(MagickInfo *magick_info)
   (void) UnregisterMagickInfo(magick_info->name);
 
   /*
-    Add to front of list.
+    Verify that coder stability level is sufficient.
   */
-  AcquireSemaphoreInfo(&magick_semaphore);
-  magick_info->previous=(MagickInfo *) NULL;
-  magick_info->next=magick_list;
-  if (magick_info->next != (MagickInfo *) NULL)
-    magick_info->next->previous=magick_info;
-  magick_list=magick_info;
-  LiberateSemaphoreInfo(&magick_semaphore);
+  if (magick_info->coder_class >= MinimumCoderClass)
+    {
+      /*
+        Add to front of list.
+      */
+      AcquireSemaphoreInfo(&magick_semaphore);
+      magick_info->previous=(MagickInfo *) NULL;
+      magick_info->next=magick_list;
+      if (magick_info->next != (MagickInfo *) NULL)
+        magick_info->next->previous=magick_info;
+      magick_list=magick_info;
+      LiberateSemaphoreInfo(&magick_semaphore);
+      return(magick_info);
+    }
+
+  /*
+    Discard registration and return NULL.
+  */
+  DestroyMagickInfo(&magick_info);
   return(magick_info);
 }
 
@@ -1203,10 +1352,15 @@ MagickExport MagickInfo *SetMagickInfo(const char *name)
     MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
       UnableToAllocateMagickInfo);
   (void) memset(magick_info,0,sizeof(MagickInfo));
-  magick_info->name=AcquireString(name);
-  magick_info->adjoin=True;
-  magick_info->blob_support=True;
-  magick_info->thread_support=True;
+  magick_info->name=name;
+  magick_info->adjoin=MagickTrue;
+  magick_info->raw=MagickFalse;
+  magick_info->stealth=MagickFalse;
+  magick_info->seekable_stream=MagickFalse;
+  magick_info->blob_support=MagickTrue;
+  magick_info->thread_support=MagickTrue;
+  magick_info->coder_class=StableCoderClass;
+  magick_info->extension_treatment=HintExtensionTreatment;
   magick_info->signature=MagickSignature;
   return(magick_info);
 }
@@ -1223,7 +1377,8 @@ MagickExport MagickInfo *SetMagickInfo(const char *name)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  Method UnregisterMagickInfo removes a name from the magick info list.  It
-%  returns False if the name does not exist in the list otherwise True.
+%  returns MagickFail if the name does not exist in the list otherwise
+%  MagickPass.
 %
 %  The format of the UnregisterMagickInfo method is:
 %
@@ -1238,7 +1393,7 @@ MagickExport MagickInfo *SetMagickInfo(const char *name)
 %      looking for.
 %
 */
-MagickExport unsigned int UnregisterMagickInfo(const char *name)
+MagickExport MagickPassFail UnregisterMagickInfo(const char *name)
 {
   MagickInfo
     *magick_info;
@@ -1250,7 +1405,7 @@ MagickExport unsigned int UnregisterMagickInfo(const char *name)
     status;
 
   assert(name != (const char *) NULL);
-  status=False;
+  status=MagickFail;
   AcquireSemaphoreInfo(&magick_semaphore);
   for (p=magick_list; p != (MagickInfo *) NULL; p=p->next)
   {
@@ -1263,17 +1418,8 @@ MagickExport unsigned int UnregisterMagickInfo(const char *name)
     else
       magick_list=p->next;
     magick_info=p;
-    MagickFreeMemory(magick_info->name);
-    if (magick_info->description != (char *) NULL)
-      MagickFreeMemory(magick_info->description);
-    if (magick_info->version != (char *) NULL)
-      MagickFreeMemory(magick_info->version);
-    if (magick_info->note != (char *) NULL)
-      MagickFreeMemory(magick_info->note);
-    if (magick_info->module != (char *) NULL)
-      MagickFreeMemory(magick_info->module);
-    MagickFreeMemory(magick_info);
-    status=True;
+    DestroyMagickInfo(&magick_info);
+    status=MagickPass;
     break;
   }
   LiberateSemaphoreInfo(&magick_semaphore);
