@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2004 - 2008 GraphicsMagick Group
+% Copyright (C) 2004 - 2009 GraphicsMagick Group
 %
 % This program is covered by multiple licenses, which are described in
 % Copyright.txt. You should have received a copy of Copyright.txt with this
@@ -17,6 +17,7 @@
 #include "magick/enum_strings.h"
 #include "magick/gem.h"
 #include "magick/pixel_iterator.h"
+#include "magick/random.h"
 #include "magick/utility.h"
 #include "magick/operator.h"
 
@@ -571,7 +572,7 @@ QuantumDepthCB(void *mutable_data,
       */
 #if MaxRGB <= MaxMap
 #  if defined(HAVE_OPENMP)
-#    pragma omp critical
+#    pragma omp critical (GM_QuantumDepthCB)
 #  endif
       if (mutable_context->channel_lut == (Quantum *) NULL)
         {
@@ -751,7 +752,7 @@ QuantumGammaCB(void *mutable_data,
   */
 #if MaxRGB <= MaxMap
 #  if defined(HAVE_OPENMP)
-#    pragma omp critical
+#    pragma omp critical (GM_QuantumGammaCB)
 #  endif
   if (mutable_context->channel_lut == (Quantum *) NULL)
     {
@@ -887,6 +888,119 @@ QuantumNegateCB(void *mutable_data,
 
   return (MagickPass);
 }
+/* log(quantum*value+1)/log(value+1) */
+#if MaxRGB > MaxMap
+#  define LogAdjustQuantum(quantum) \
+  ((MaxRGBDouble*log((quantum/MaxRGBDouble)*immutable_context->double_value+1.0)/ \
+    log(immutable_context->double_value+1.0))+0.5)
+#else
+#  define LogAdjustQuantum(quantum) (mutable_context->channel_lut[ScaleQuantumToMap(quantum)])
+#endif
+static MagickPassFail
+QuantumLogCB(void *mutable_data,
+             const void *immutable_data,
+             Image *image,
+             PixelPacket *pixels,
+             IndexPacket *indexes,
+             const long npixels,
+             ExceptionInfo *exception)
+{
+  QuantumMutableContext
+    *mutable_context=(QuantumMutableContext *) mutable_data;
+
+  const QuantumImmutableContext
+    *immutable_context=(const QuantumImmutableContext *) immutable_data;
+  
+  register long
+    i;
+
+  MagickPassFail
+    status=MagickPass;
+  
+  ARG_NOT_USED(image);
+  ARG_NOT_USED(indexes);
+  ARG_NOT_USED(exception);
+
+  /*
+    Build LUT for Q8 and Q16 builds
+  */
+#if MaxRGB <= MaxMap
+#  if defined(HAVE_OPENMP)
+#    pragma omp critical (GM_QuantumLogCB)
+#  endif
+  if (mutable_context->channel_lut == (Quantum *) NULL)
+    {
+      mutable_context->channel_lut=MagickAllocateArray(Quantum *, MaxMap+1,sizeof(Quantum));
+      if (mutable_context->channel_lut == (Quantum *) NULL)
+        status=MagickFail;
+      
+      if (mutable_context->channel_lut != (Quantum *) NULL)
+        {
+          for (i=0; i <= (long) MaxMap; i++)
+            {
+              double
+                value;
+
+              value=MaxRGBDouble*(log((ScaleMapToQuantum(i)/MaxRGBDouble)*
+                                     immutable_context->double_value+1.0)/
+                                  log(immutable_context->double_value+1.0));
+              mutable_context->channel_lut[i] = RoundDoubleToQuantum(value);
+            }
+        }
+    }
+#else
+  ARG_NOT_USED(*mutable_context);
+#endif
+  if (MagickFail == status)
+    return status;
+
+  switch (immutable_context->channel)
+    {
+    case RedChannel:
+    case CyanChannel:
+      for (i=0; i < npixels; i++)
+        pixels[i].red=LogAdjustQuantum(pixels[i].red);
+      break;
+    case GreenChannel:
+    case MagentaChannel:
+      for (i=0; i < npixels; i++)
+        pixels[i].green=LogAdjustQuantum(pixels[i].green);
+      break;
+    case BlueChannel:
+    case YellowChannel:
+      for (i=0; i < npixels; i++)
+        pixels[i].blue=LogAdjustQuantum(pixels[i].blue);
+      break;
+    case BlackChannel:
+    case MatteChannel:
+    case OpacityChannel:
+      for (i=0; i < npixels; i++)
+        pixels[i].opacity=LogAdjustQuantum(pixels[i].opacity);
+      break;
+    case UndefinedChannel:
+    case AllChannels:
+      for (i=0; i < npixels; i++)
+        {
+          pixels[i].red=LogAdjustQuantum(pixels[i].red);
+          pixels[i].green=LogAdjustQuantum(pixels[i].green);
+          pixels[i].blue=LogAdjustQuantum(pixels[i].blue);
+        }
+      break;
+    case GrayChannel:
+      for (i=0; i < npixels; i++)
+        {
+          Quantum
+            intensity;
+
+          intensity = PixelIntensity(&pixels[i]);
+          intensity = LogAdjustQuantum(intensity);
+          pixels[i].red = pixels[i].green = pixels[i].blue = intensity;
+        }
+      break;
+    }
+
+  return status;
+}
 static MagickPassFail
 QuantumLShiftCB(void *mutable_data,
                 const void *immutable_data,
@@ -955,6 +1069,156 @@ QuantumLShiftCB(void *mutable_data,
   return (MagickPass);
 }
 static MagickPassFail
+QuantumMaxCB(void *mutable_data,
+             const void *immutable_data,
+             Image *image,
+             PixelPacket *pixels,
+             IndexPacket *indexes,
+             const long npixels,
+             ExceptionInfo *exception)
+{
+  const QuantumImmutableContext
+    *context=(const QuantumImmutableContext *) immutable_data;
+
+  register long
+    i;
+
+  ARG_NOT_USED(mutable_data);
+  ARG_NOT_USED(image);
+  ARG_NOT_USED(indexes);
+  ARG_NOT_USED(exception);
+
+  switch (context->channel)
+    {
+    case RedChannel:
+    case CyanChannel:
+      for (i=0; i < npixels; i++)
+        if (context->quantum_value > pixels[i].red)
+          pixels[i].red = context->quantum_value;
+      break;
+    case GreenChannel:
+    case MagentaChannel:
+      for (i=0; i < npixels; i++)
+        if (context->quantum_value > pixels[i].green)
+          pixels[i].green = context->quantum_value;
+      break;
+    case BlueChannel:
+    case YellowChannel:
+      for (i=0; i < npixels; i++)
+        if (context->quantum_value > pixels[i].blue)
+          pixels[i].blue = context->quantum_value;
+      break;
+    case BlackChannel:
+    case MatteChannel:
+    case OpacityChannel:
+      for (i=0; i < npixels; i++)
+        if (context->quantum_value > pixels[i].opacity)
+          pixels[i].opacity = context->quantum_value;
+      break;
+    case UndefinedChannel:
+    case AllChannels:
+      for (i=0; i < npixels; i++)
+        {
+          if (context->quantum_value > pixels[i].red)
+            pixels[i].red = context->quantum_value;
+          if (context->quantum_value > pixels[i].green)
+            pixels[i].green = context->quantum_value;
+          if (context->quantum_value > pixels[i].blue)
+            pixels[i].blue = context->quantum_value;
+        }
+      break;
+    case GrayChannel:
+      for (i=0; i < npixels; i++)
+        {
+          Quantum
+            intensity;
+
+          intensity = PixelIntensity(&pixels[i]);
+          if (context->quantum_value > intensity)
+            intensity = context->quantum_value;
+          pixels[i].red = pixels[i].green = pixels[i].blue = intensity;
+        }
+      break;
+    }
+
+  return (MagickPass);
+}
+static MagickPassFail
+QuantumMinCB(void *mutable_data,
+             const void *immutable_data,
+             Image *image,
+             PixelPacket *pixels,
+             IndexPacket *indexes,
+             const long npixels,
+             ExceptionInfo *exception)
+{
+  const QuantumImmutableContext
+    *context=(const QuantumImmutableContext *) immutable_data;
+
+  register long
+    i;
+
+  ARG_NOT_USED(mutable_data);
+  ARG_NOT_USED(image);
+  ARG_NOT_USED(indexes);
+  ARG_NOT_USED(exception);
+
+  switch (context->channel)
+    {
+    case RedChannel:
+    case CyanChannel:
+      for (i=0; i < npixels; i++)
+        if (context->quantum_value < pixels[i].red)
+          pixels[i].red = context->quantum_value;
+      break;
+    case GreenChannel:
+    case MagentaChannel:
+      for (i=0; i < npixels; i++)
+        if (context->quantum_value < pixels[i].green)
+          pixels[i].green = context->quantum_value;
+      break;
+    case BlueChannel:
+    case YellowChannel:
+      for (i=0; i < npixels; i++)
+        if (context->quantum_value < pixels[i].blue)
+          pixels[i].blue = context->quantum_value;
+      break;
+    case BlackChannel:
+    case MatteChannel:
+    case OpacityChannel:
+      for (i=0; i < npixels; i++)
+        if (context->quantum_value < pixels[i].opacity)
+          pixels[i].opacity = context->quantum_value;
+      break;
+    case UndefinedChannel:
+    case AllChannels:
+      for (i=0; i < npixels; i++)
+        {
+          if (context->quantum_value < pixels[i].red)
+            pixels[i].red = context->quantum_value;
+          if (context->quantum_value < pixels[i].green)
+            pixels[i].green = context->quantum_value;
+          if (context->quantum_value < pixels[i].blue)
+            pixels[i].blue = context->quantum_value;
+        }
+      break;
+    case GrayChannel:
+      for (i=0; i < npixels; i++)
+        {
+          Quantum
+            intensity;
+
+          intensity = PixelIntensity(&pixels[i]);
+          if (context->quantum_value < intensity)
+            intensity = context->quantum_value;
+          pixels[i].red = pixels[i].green = pixels[i].blue = intensity;
+        }
+      break;
+    }
+
+  return (MagickPass);
+}
+static MagickPassFail
 QuantumMultiplyCB(void *mutable_data,
                   const void *immutable_data,
                   Image *image,
@@ -1024,13 +1288,13 @@ QuantumMultiplyCB(void *mutable_data,
 
 static inline Quantum
 GenerateQuantumNoise(const Quantum quantum,const NoiseType noise_type,
-                     const double factor,unsigned int *seed)
+                     const double factor,MagickRandomKernel *kernel)
 {
   double
     value;
 
   value = (double) quantum+
-    factor*GenerateDifferentialNoise((double) quantum,noise_type,seed);
+    factor*GenerateDifferentialNoise((double) quantum,noise_type,kernel);
   return RoundDoubleToQuantum(value);
 }
 
@@ -1054,15 +1318,15 @@ QuantumNoiseCB(void *mutable_data,
   double
     factor;
 
-  unsigned int
-    seed;
+  MagickRandomKernel
+    *kernel;
 
   ARG_NOT_USED(mutable_data);
   ARG_NOT_USED(image);
   ARG_NOT_USED(indexes);
   ARG_NOT_USED(exception);
 
-  seed=MagickRandNewSeed();
+  kernel=AcquireMagickRandomKernel();
   factor=context->double_value/MaxRGBDouble;
 
   switch (context->channel)
@@ -1070,31 +1334,31 @@ QuantumNoiseCB(void *mutable_data,
     case RedChannel:
     case CyanChannel:
       for (i=0; i < npixels; i++)
-        pixels[i].red = GenerateQuantumNoise(pixels[i].red,noise_type,factor,&seed);
+        pixels[i].red = GenerateQuantumNoise(pixels[i].red,noise_type,factor,kernel);
       break;
     case GreenChannel:
     case MagentaChannel:
       for (i=0; i < npixels; i++)
-        pixels[i].green = GenerateQuantumNoise(pixels[i].green,noise_type,factor,&seed);
+        pixels[i].green = GenerateQuantumNoise(pixels[i].green,noise_type,factor,kernel);
       break;
     case BlueChannel:
     case YellowChannel:
       for (i=0; i < npixels; i++)
-        pixels[i].blue = GenerateQuantumNoise(pixels[i].blue,noise_type,factor,&seed);
+        pixels[i].blue = GenerateQuantumNoise(pixels[i].blue,noise_type,factor,kernel);
       break;
     case BlackChannel:
     case MatteChannel:
     case OpacityChannel:
       for (i=0; i < npixels; i++)
-        pixels[i].opacity = GenerateQuantumNoise(pixels[i].opacity,noise_type,factor,&seed);
+        pixels[i].opacity = GenerateQuantumNoise(pixels[i].opacity,noise_type,factor,kernel);
       break;
     case UndefinedChannel:
     case AllChannels:
       for (i=0; i < npixels; i++)
         {
-          pixels[i].red   = GenerateQuantumNoise(pixels[i].red,noise_type,factor,&seed);
-          pixels[i].green = GenerateQuantumNoise(pixels[i].green,noise_type,factor,&seed);
-          pixels[i].blue  = GenerateQuantumNoise(pixels[i].blue,noise_type,factor,&seed);
+          pixels[i].red   = GenerateQuantumNoise(pixels[i].red,noise_type,factor,kernel);
+          pixels[i].green = GenerateQuantumNoise(pixels[i].green,noise_type,factor,kernel);
+          pixels[i].blue  = GenerateQuantumNoise(pixels[i].blue,noise_type,factor,kernel);
         }
       break;
     case GrayChannel:
@@ -1105,7 +1369,7 @@ QuantumNoiseCB(void *mutable_data,
 
           intensity = PixelIntensity(&pixels[i]);
           pixels[i].red = pixels[i].green = pixels[i].blue =
-            GenerateQuantumNoise(intensity,noise_type,factor,&seed);
+            GenerateQuantumNoise(intensity,noise_type,factor,kernel);
         }
       break;
     }
@@ -1252,6 +1516,110 @@ QuantumOrCB(void *mutable_data,
 
   return (MagickPass);
 }
+#if MaxRGB > MaxMap
+#  define PowAdjustQuantum(quantum) (MaxRGBDouble*pow(quantum/MaxRGBDouble,immutable_context->double_value)+0.5)
+#else
+#  define PowAdjustQuantum(quantum) (mutable_context->channel_lut[ScaleQuantumToMap(quantum)])
+#endif
+static MagickPassFail
+QuantumPowCB(void *mutable_data,
+             const void *immutable_data,
+             Image *image,
+             PixelPacket *pixels,
+             IndexPacket *indexes,
+             const long npixels,
+             ExceptionInfo *exception)
+{
+  QuantumMutableContext
+    *mutable_context=(QuantumMutableContext *) mutable_data;
+
+  const QuantumImmutableContext
+    *immutable_context=(const QuantumImmutableContext *) immutable_data;
+  
+  register long
+    i;
+
+  MagickPassFail
+    status=MagickPass;
+  
+  ARG_NOT_USED(image);
+  ARG_NOT_USED(indexes);
+  ARG_NOT_USED(exception);
+
+  /*
+    Build LUT for Q8 and Q16 builds
+  */
+#if MaxRGB <= MaxMap
+#  if defined(HAVE_OPENMP)
+#    pragma omp critical (GM_QuantumPowCB)
+#  endif
+  if (mutable_context->channel_lut == (Quantum *) NULL)
+    {
+      mutable_context->channel_lut=MagickAllocateArray(Quantum *, MaxMap+1,sizeof(Quantum));
+      if (mutable_context->channel_lut == (Quantum *) NULL)
+        status=MagickFail;
+      
+      if (mutable_context->channel_lut != (Quantum *) NULL)
+        {
+          for (i=0; i <= (long) MaxMap; i++)
+            mutable_context->channel_lut[i] =
+              ScaleMapToQuantum(MaxMap*pow((double) i/MaxMap,
+                                           immutable_context->double_value));
+        }
+    }
+#else
+  ARG_NOT_USED(*mutable_context);
+#endif
+  if (MagickFail == status)
+    return status;
+
+  switch (immutable_context->channel)
+    {
+    case RedChannel:
+    case CyanChannel:
+      for (i=0; i < npixels; i++)
+        pixels[i].red=PowAdjustQuantum(pixels[i].red);
+      break;
+    case GreenChannel:
+    case MagentaChannel:
+      for (i=0; i < npixels; i++)
+        pixels[i].green=PowAdjustQuantum(pixels[i].green);
+      break;
+    case BlueChannel:
+    case YellowChannel:
+      for (i=0; i < npixels; i++)
+        pixels[i].blue=PowAdjustQuantum(pixels[i].blue);
+      break;
+    case BlackChannel:
+    case MatteChannel:
+    case OpacityChannel:
+      for (i=0; i < npixels; i++)
+        pixels[i].opacity=PowAdjustQuantum(pixels[i].opacity);
+      break;
+    case UndefinedChannel:
+    case AllChannels:
+      for (i=0; i < npixels; i++)
+        {
+          pixels[i].red=PowAdjustQuantum(pixels[i].red);
+          pixels[i].green=PowAdjustQuantum(pixels[i].green);
+          pixels[i].blue=PowAdjustQuantum(pixels[i].blue);
+        }
+      break;
+    case GrayChannel:
+      for (i=0; i < npixels; i++)
+        {
+          Quantum
+            intensity;
+
+          intensity = PixelIntensity(&pixels[i]);
+          intensity = PowAdjustQuantum(intensity);
+          pixels[i].red = pixels[i].green = pixels[i].blue = intensity;
+        }
+      break;
+    }
+
+  return status;
+}
 static MagickPassFail
 QuantumRShiftCB(void *mutable_data,
                 const void *immutable_data,
@@ -1813,6 +2181,18 @@ QuantumOperatorRegionImage(Image *image,
     case DepthQuantumOp:
       call_back=QuantumDepthCB;
       break;
+    case LogQuantumOp:
+      call_back=QuantumLogCB;
+      break;
+    case MaxQuantumOp:
+      call_back=QuantumMaxCB;
+      break;
+    case MinQuantumOp:
+      call_back=QuantumMinCB;
+      break;
+    case PowQuantumOp:
+      call_back=QuantumPowCB;
+      break;
     }
 
   if (call_back)
