@@ -185,7 +185,7 @@ PixelIterateMonoRead(PixelIteratorMonoReadCallback call_back,
   (void) SetRegionThreads(max_threads,options,columns,rows);
 
 #if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
+#  pragma omp parallel for schedule(static,1) shared(row_count, status)
 #endif
   for (row=y; row < (long) (y+rows); row++)
     {
@@ -211,7 +211,7 @@ PixelIterateMonoRead(PixelIteratorMonoReadCallback call_back,
         thread_status=(call_back)(mutable_data,immutable_data,image,pixels,indexes,columns,exception);
 
 #if defined(HAVE_OPENMP)
-#  pragma omp critical
+#  pragma omp critical (GM_PixelIterateMonoRead)
 #endif
       {
         row_count++;
@@ -316,7 +316,7 @@ PixelIterateMonoModify(PixelIteratorMonoModifyCallback call_back,
   (void) SetRegionThreads(max_threads,options,columns,rows);
 
 #if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
+#  pragma omp parallel for schedule(static,1) shared(row_count, status)
 #endif
   for (row=y; row < (long) (y+rows); row++)
     {
@@ -346,7 +346,7 @@ PixelIterateMonoModify(PixelIteratorMonoModifyCallback call_back,
           thread_status=MagickFail;
 
 #if defined(HAVE_OPENMP)
-#  pragma omp critical
+#  pragma omp critical (GM_PixelIterateMonoModify)
 #endif
       {
         row_count++;
@@ -463,7 +463,7 @@ PixelIterateDualRead(PixelIteratorDualReadCallback call_back,
   (void) SetRegionThreads(max_threads,options,columns,rows);
 
 #if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
+#  pragma omp parallel for schedule(static,1) shared(row_count, status)
 #endif
   for (row=0; row < (long) rows; row++)
     {
@@ -508,7 +508,7 @@ PixelIterateDualRead(PixelIteratorDualReadCallback call_back,
                                   columns, exception);
 
 #if defined(HAVE_OPENMP)
-#  pragma omp critical
+#  pragma omp critical (GM_PixelIterateDualRead)
 #endif
       {
         row_count++;
@@ -627,7 +627,7 @@ PixelIterateDualImplementation(PixelIteratorDualModifyCallback call_back,
   (void) SetRegionThreads(max_threads,options,columns,rows);
 
 #if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
+#  pragma omp parallel for schedule(static,1) shared(row_count, status)
 #endif
   for (row=0; row < (long) rows; row++)
     {
@@ -684,7 +684,7 @@ PixelIterateDualImplementation(PixelIteratorDualModifyCallback call_back,
           thread_status=MagickFail;
 
 #if defined(HAVE_OPENMP)
-#  pragma omp critical
+#  pragma omp critical (GM_PixelIterateDualImplementation)
 #endif
       {
         row_count++;
@@ -923,7 +923,7 @@ PixelIterateTripleImplementation(PixelIteratorTripleModifyCallback call_back,
   (void) SetRegionThreads(max_threads,options,columns,rows);
 
 #if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
+#  pragma omp parallel for schedule(static,1) shared(row_count, status)
 #endif
   for (row=0; row < (long) rows; row++)
     {
@@ -1000,7 +1000,7 @@ PixelIterateTripleImplementation(PixelIteratorTripleModifyCallback call_back,
         thread_status=MagickFail;
 
 #if defined(HAVE_OPENMP)
-#  pragma omp critical
+#  pragma omp critical (GM_PixelIterateTripleImplementation)
 #endif
       {
         row_count++;
