@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003, 2008 GraphicsMagick Group
+% Copyright (C) 2003, 2009 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -91,7 +91,7 @@ static ResourceInfo
   {
     { "",       "",  0, ResourceInfinity, SummationLimit },
     { "disk",   "B", 0, ResourceInfinity, SummationLimit },
-    { "file",   "",  0, 256,              SummationLimit },
+    { "files",  "",  0, 256,              SummationLimit },
     { "map",    "B", 0, ResourceInfinity, SummationLimit },
     { "memory", "B", 0, ResourceInfinity, SummationLimit },
     { "pixels", "P", 0, ResourceInfinity, AbsoluteLimit  }
@@ -338,33 +338,26 @@ MagickExport void InitializeMagickResources(void)
   */
 #if defined(POSIX)
   {
-    long
-      files=-1;
-
     unsigned long
       total_memory=0;
 
-#  if defined(HAVE_SYSCONF) && defined(_SC_OPEN_MAX)
-    files=sysconf(_SC_OPEN_MAX);
-#  endif
-
 #  if  defined(HAVE_SYSCONF) && defined(_SC_PHYS_PAGES)
     {
-    long
-      pagesize=-1,
-      pages=-1;
-    /*
-      Compute total physical memory based on number of memory pages,
-      and page size.
-    */
-    pages=sysconf(_SC_PHYS_PAGES);
-    pagesize = MagickGetMMUPageSize();
+      long
+        pagesize=-1,
+        pages=-1;
+      /*
+        Compute total physical memory based on number of memory pages,
+        and page size.
+      */
+      pages=sysconf(_SC_PHYS_PAGES);
+      pagesize = MagickGetMMUPageSize();
 
-    if (pages > 0 && pagesize > 0)
-      total_memory=((pages+512)/1024)*((pagesize+512)/1024);
-    (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
-      "Total physical memory %ld MB (%ld pages and %ld bytes per page)",
-        total_memory, pages, pagesize);
+      if (pages > 0 && pagesize > 0)
+        total_memory=((pages+512)/1024)*((pagesize+512)/1024);
+      (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
+                            "Total physical memory %ld MB (%ld pages and %ld bytes per page)",
+                            total_memory, pages, pagesize);
     }
 #  elif defined(MAGICK_PHYSICAL_MEMORY_COMMAND) && defined(HAVE_POPEN)
     {
@@ -385,14 +378,11 @@ MagickExport void InitializeMagickResources(void)
             total_memory=(bytes/(1024*1024));
           (void) pclose(command);
           (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
-            "Total physical memory %ld MB",total_memory);
+                                "Total physical memory %ld MB",total_memory);
         }
     }
 #  endif
 
-    if (files > 0)
-      max_files=files/2;
-
     if (total_memory > 0)
       {
         /*
@@ -435,32 +425,47 @@ MagickExport void InitializeMagickResources(void)
         MEMORYSTATUSEX
           stat_ex;
 
+        (void) memset(&stat_ex,0,sizeof(stat_ex));
+        stat_ex.dwLength=sizeof(stat_ex);
         if (GlobalMemoryStatusEx(&stat_ex))
           {
             total_physical_memory=(long)(stat_ex.ullTotalPhys/1048576UL);
             total_virtual_memory=(long)(stat_ex.ullTotalVirtual/1048576UL);
           }
+        else
+          {
+            (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
+                                  "GlobalMemoryStatusEx() call failed! (error %ld)",GetLastError());
+          }
       }
 #endif
 
     if (total_physical_memory == 0)
-    {
-      MEMORYSTATUS
-        stat;
+      {
+        MEMORYSTATUS
+          stat;
 
-      GlobalMemoryStatus(&stat);
-      total_physical_memory=stat.dwTotalPhys/1048576;
-      total_virtual_memory=stat.dwTotalVirtual/1048576;
-    }
-
-    if (total_virtual_memory > 3*total_physical_memory)
-      max_memory=2*total_physical_memory;
-    else
-      max_memory=(long)(0.7*total_virtual_memory);
-    max_map=(long)(8*total_physical_memory);
+        GlobalMemoryStatus(&stat);
+        total_physical_memory=stat.dwTotalPhys/1048576;
+        total_virtual_memory=stat.dwTotalVirtual/1048576;
+      }
     (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
-      "Total physical memory %ld MB, Total virtual memory %ld MB",
-        total_physical_memory, total_virtual_memory);
+                          "Total physical memory %ld MB, Total virtual memory %ld MB",
+                          total_physical_memory, total_virtual_memory);
+
+    max_memory=Min(total_physical_memory,total_virtual_memory);
+    /*
+      We will default to using at most 80% of available memory for image data.
+    */
+    max_memory=(long)(0.8*max_memory);
+    /*
+      Use the maximum memory as our default memory map limit.  Memory
+      mapping files larger than available RAM can cause VM thrashing.
+      This implies that memory mapping won't get used much.  However,
+      it is possible for heap memory allocations to fail (due to heap
+      fragmentation) whereas memory mapping still succeeds.
+    */
+    max_map=max_memory;
 
     /*
       Windows lowio level supports up to 2048 open files.
@@ -511,6 +516,76 @@ MagickExport void InitializeMagickResources(void)
       max_pixels=MagickSizeStrToInt64(envp,1024);
   }
 
+#if defined(POSIX)
+#  if defined(HAVE_GETRLIMIT) && defined(RLIMIT_NOFILE)
+    {
+      struct rlimit
+	rlimits;
+
+      rlim_t
+	margin,
+	target;
+
+      margin=128;
+      target=(rlim_t) max_files+margin;
+      if (getrlimit(RLIMIT_NOFILE, &rlimits) != -1)
+	{
+	  (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
+				"System file open limits are %lu soft,"
+				" %lu hard",
+				(unsigned long) rlimits.rlim_cur,
+				(unsigned long) rlimits.rlim_max);
+#    if defined(HAVE_SETRLIMIT)
+	  if (rlimits.rlim_max < target)
+	    rlimits.rlim_cur=rlimits.rlim_max;
+	  if (rlimits.rlim_cur < target)
+	    {
+	      (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
+				    "Increasing file open soft limit from %lu "
+				    "to %lu",
+				    (unsigned long) rlimits.rlim_cur,
+				    (unsigned long) target);
+	      rlimits.rlim_cur=target;
+	      (void) setrlimit(RLIMIT_NOFILE, &rlimits);
+	    }
+	  if (getrlimit(RLIMIT_NOFILE, &rlimits) != -1)
+	    {
+	      if (rlimits.rlim_cur < target)
+		{
+		  if (rlimits.rlim_cur > margin*2)
+		    max_files=rlimits.rlim_cur-margin;
+		  else
+		    max_files=rlimits.rlim_cur/2;
+		}
+	    }
+#    endif
+	}
+    }
+#  elif defined(HAVE_SYSCONF) && defined(_SC_OPEN_MAX)
+    {
+      long
+	margin,
+	open_max,
+	target;
+
+      margin=128;
+      target=max_files+margin;
+      open_max=sysconf(_SC_OPEN_MAX);
+      (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
+			    "System file open limit is %lu",
+			    (unsigned long) open_max);
+      if (open_max < target)
+	{
+	  if (open_max > margin*2)
+	    max_files=open_max-margin;
+	  else
+	    max_files=open_max/2;
+	}
+    }
+#  endif
+
+#endif
+
   if (max_disk >= 0)
     (void) SetMagickResourceLimit(DiskResource,max_disk);
   if (max_files >= 0)
@@ -689,6 +764,15 @@ MagickExport MagickPassFail ListMagickResourceInfo(FILE *file,
 
       fprintf(file,"%8s: %10s (MAGICK_LIMIT_%s)\n", heading, limit, environment);
     }
+#if defined(HAVE_OPENMP)
+  {
+    char
+      limit[MaxTextExtent];
+    
+    FormatString(limit,"%d",omp_get_max_threads());
+    fprintf(file,"%8s: %10s (OMP_NUM_THREADS)\n", "Threads", limit);
+  }
+#endif /* defined(HAVE_OPENMP) */
   (void) fflush(file);
 
   LiberateSemaphoreInfo(&resource_semaphore);
