@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 GraphicsMagick Group
+% Copyright (C) 2003, 2009 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -37,14 +37,23 @@
 */
 #include "magick/studio.h"
 #include "magick/utility.h"
+
 #if defined(HAVE_PTHREAD)
-#include <pthread.h>
+#  define USE_POSIX_THREADS 1
+#elif defined(MSWINDOWS)
+#  define USE_WIN32_THREADS 1
 #endif
-#if defined(MSWINDOWS)
-#include <windows.h>
-#define USE_SPINLOCKS
-#define SPINLOCK_DELAY_MILLI_SECS 10
+
+#if defined(USE_POSIX_THREADS)
+#  include <pthread.h>
 #endif
+
+#if defined(USE_WIN32_THREADS)
+#  include <windows.h>
+#  define USE_SPINLOCKS
+#  define SPINLOCK_DELAY_MILLI_SECS 10
+#endif
+
 #include "magick/semaphore.h"
 
 /*
@@ -52,24 +61,15 @@
 */
 struct _SemaphoreInfo
 {
-#if defined(HAVE_PTHREAD)
+#if defined(USE_POSIX_THREADS)
   pthread_mutex_t
     mutex;		/* POSIX thread mutex */
-
-  pthread_t
-    owner_thread_id;    /* ID of thread which holds the lock */
 #endif
-#if defined(MSWINDOWS)
+#if defined(USE_WIN32_THREADS)
   CRITICAL_SECTION
     mutex;		/* Windows critical section */
-
-  DWORD
-    owner_thread_id;    /* ID of thread which holds the lock */
 #endif
 
-  unsigned int
-    lock_depth;         /* Number of times lock is held by same thread */
-
   unsigned long
     signature;		/* Used to validate structure */
 };
@@ -77,12 +77,12 @@ struct _SemaphoreInfo
 /*
   Static declaractions.
 */
-#if defined(HAVE_PTHREAD)
+#if defined(USE_POSIX_THREADS)
 static pthread_mutex_t
   semaphore_mutex = PTHREAD_MUTEX_INITIALIZER;
 #endif
 
-#if defined(MSWINDOWS)
+#if defined(USE_WIN32_THREADS)
 #if !defined(USE_SPINLOCKS)
 static CRITICAL_SECTION
   semaphore_mutex;
@@ -148,10 +148,10 @@ static void spinlock_release (LONG volatile *sl)
 MagickExport void AcquireSemaphoreInfo(SemaphoreInfo **semaphore_info)
 {
   assert(semaphore_info != (SemaphoreInfo **) NULL);
-#if defined(HAVE_PTHREAD)
+#if defined(USE_POSIX_THREADS)
   (void) pthread_mutex_lock(&semaphore_mutex);
 #endif
-#if defined(MSWINDOWS)
+#if defined(USE_WIN32_THREADS)
 #if !defined(USE_SPINLOCKS)
   if (!active_semaphore)
     InitializeCriticalSection(&semaphore_mutex);
@@ -163,10 +163,10 @@ MagickExport void AcquireSemaphoreInfo(SemaphoreInfo **semaphore_info)
 #endif
   if (*semaphore_info == (SemaphoreInfo *) NULL)
     *semaphore_info=AllocateSemaphoreInfo();
-#if defined(HAVE_PTHREAD)
+#if defined(USE_POSIX_THREADS)
   (void) pthread_mutex_unlock(&semaphore_mutex);
 #endif
-#if defined(MSWINDOWS)
+#if defined(USE_WIN32_THREADS)
 #if !defined(USE_SPINLOCKS)
   LeaveCriticalSection(&semaphore_mutex);
 #else
@@ -216,7 +216,7 @@ MagickExport SemaphoreInfo *AllocateSemaphoreInfo(void)
   /*
     Initialize the semaphore.
   */
-#if defined(HAVE_PTHREAD)
+#if defined(USE_POSIX_THREADS)
   {
     int
       status;
@@ -230,7 +230,7 @@ MagickExport SemaphoreInfo *AllocateSemaphoreInfo(void)
         errno=status;
         return((SemaphoreInfo *) NULL);
       }
-#if 0
+#if 1
 #if defined(PTHREAD_MUTEX_RECURSIVE)
     (void) pthread_mutexattr_settype(&mutexattr, PTHREAD_MUTEX_RECURSIVE);
 #endif /* PTHREAD_MUTEX_RECURSIVE */
@@ -244,14 +244,11 @@ MagickExport SemaphoreInfo *AllocateSemaphoreInfo(void)
       }
 
     (void) pthread_mutexattr_destroy(&mutexattr);
-
-    semaphore_info->owner_thread_id=0;
   }
 #endif
-#if defined(MSWINDOWS)
+#if defined(USE_WIN32_THREADS)
   InitializeCriticalSection(&semaphore_info->mutex);
 #endif
-  semaphore_info->lock_depth=0;
   
   semaphore_info->signature=MagickSignature;
   return(semaphore_info);
@@ -278,10 +275,10 @@ MagickExport SemaphoreInfo *AllocateSemaphoreInfo(void)
 */
 MagickExport void DestroySemaphore(void)
 {
-#if defined(HAVE_PTHREAD)
+#if defined(USE_POSIX_THREADS)
   (void) pthread_mutex_destroy(&semaphore_mutex);
 #endif
-#if defined(MSWINDOWS)
+#if defined(USE_WIN32_THREADS)
 #if !defined(USE_SPINLOCKS)
   if (active_semaphore)
     DeleteCriticalSection(&semaphore_mutex);
@@ -319,10 +316,10 @@ MagickExport void DestroySemaphoreInfo(SemaphoreInfo **semaphore_info)
   if (*semaphore_info == (SemaphoreInfo *) NULL)
     return;
   assert((*semaphore_info)->signature == MagickSignature);
-#if defined(HAVE_PTHREAD)
+#if defined(USE_POSIX_THREADS)
   (void) pthread_mutex_lock(&semaphore_mutex);
 #endif
-#if defined(MSWINDOWS)
+#if defined(USE_WIN32_THREADS)
 #if !defined(USE_SPINLOCKS)
   if (!active_semaphore)
     InitializeCriticalSection(&semaphore_mutex);
@@ -332,17 +329,17 @@ MagickExport void DestroySemaphoreInfo(SemaphoreInfo **semaphore_info)
   spinlock_wait(&semaphore_mutex);
 #endif
 #endif
-#if defined(HAVE_PTHREAD)
+#if defined(USE_POSIX_THREADS)
   (void) pthread_mutex_destroy(&(*semaphore_info)->mutex);
 #endif
-#if defined(MSWINDOWS)
+#if defined(USE_WIN32_THREADS)
   DeleteCriticalSection(&(*semaphore_info)->mutex);
 #endif
   MagickFreeMemory((*semaphore_info));
-#if defined(HAVE_PTHREAD)
+#if defined(USE_POSIX_THREADS)
   (void) pthread_mutex_unlock(&semaphore_mutex);
 #endif
-#if defined(MSWINDOWS)
+#if defined(USE_WIN32_THREADS)
 #if !defined(USE_SPINLOCKS)
   LeaveCriticalSection(&semaphore_mutex);
 #else
@@ -372,11 +369,16 @@ MagickExport void DestroySemaphoreInfo(SemaphoreInfo **semaphore_info)
 */
 MagickExport void InitializeSemaphore(void)
 {
-#if defined(HAVE_PTHREAD)
-  (void) pthread_mutex_init(&semaphore_mutex,
-    (const pthread_mutexattr_t *) NULL);
+#if defined(USE_POSIX_THREADS)
+  /*
+    We use static pthread mutex initialization with
+    PTHREAD_MUTEX_INITIALIZER so explicit runtime initialization is
+    not required.
+  */
+/*   (void) pthread_mutex_init(&semaphore_mutex, */
+/*     (const pthread_mutexattr_t *) NULL); */
 #endif
-#if defined(MSWINDOWS)
+#if defined(USE_WIN32_THREADS)
 #if !defined(USE_SPINLOCKS)
   if (!active_semaphore)
     InitializeCriticalSection(&semaphore_mutex);
@@ -411,10 +413,11 @@ MagickExport void InitializeSemaphore(void)
 MagickExport void LiberateSemaphoreInfo(SemaphoreInfo **semaphore_info)
 {
   assert(semaphore_info != (SemaphoreInfo **) NULL);
-  if (*semaphore_info == (SemaphoreInfo *) NULL)
-    return;
-  assert((*semaphore_info)->signature == MagickSignature);
-  (void) UnlockSemaphoreInfo(*semaphore_info);
+  if (*semaphore_info != (SemaphoreInfo *) NULL)
+    {
+      assert((*semaphore_info)->signature == MagickSignature);
+      (void) UnlockSemaphoreInfo(*semaphore_info);
+    }
 }
 
 /*
@@ -445,44 +448,27 @@ MagickExport void LiberateSemaphoreInfo(SemaphoreInfo **semaphore_info)
 */
 MagickExport MagickPassFail LockSemaphoreInfo(SemaphoreInfo *semaphore_info)
 {
+  MagickPassFail
+    status=MagickPass;
+
   assert(semaphore_info != (SemaphoreInfo *) NULL);
-  assert(semaphore_info->signature == MagickSignature);
-  /* fprintf(stderr,"lock %p\n", semaphore_info); */
-#if defined(HAVE_PTHREAD)
+/*   assert(semaphore_info->signature == MagickSignature); */
+#if defined(USE_POSIX_THREADS)
   {
     int
-      status;
+      err_status;
 
-    pthread_t
-      self;
-
-    self=pthread_self();
-
-    if ((semaphore_info->lock_depth > 0) &&
-        (semaphore_info->owner_thread_id == self))
+    if ((err_status = pthread_mutex_lock(&semaphore_info->mutex)) != 0)
       {
-        fprintf(stderr,
-                "Warning: recursive semaphore lock detected (depth=%u p=%p)!\n",
-                semaphore_info->lock_depth,semaphore_info);
-        fflush(stderr);
+        errno=err_status;
+        status=MagickFail;
       }
-    
-    if ((status = pthread_mutex_lock(&semaphore_info->mutex)) != 0)
-      {
-        errno=status;
-        return(MagickFail);
-      }
-    /* Record the thread ID of the locking thread */
-    semaphore_info->owner_thread_id=self;
   }
 #endif
-#if defined(MSWINDOWS)
+#if defined(USE_WIN32_THREADS)
   EnterCriticalSection(&semaphore_info->mutex);
-  /* Record the thread ID of the locking thread */
-  semaphore_info->owner_thread_id=GetCurrentThreadId();
 #endif
-  semaphore_info->lock_depth++;
-  return(MagickPass);
+  return(status);
 }
 
 /*
@@ -513,33 +499,18 @@ MagickExport MagickPassFail LockSemaphoreInfo(SemaphoreInfo *semaphore_info)
 */
 MagickExport MagickPassFail UnlockSemaphoreInfo(SemaphoreInfo *semaphore_info)
 {
+  MagickPassFail
+    status=MagickPass;
+
   assert(semaphore_info != (SemaphoreInfo *) NULL);
-  assert(semaphore_info->signature == MagickSignature);
-  if (semaphore_info->lock_depth == 0)
-    {
-      fprintf(stderr,
-              "Warning: unlock on unlocked semaphore (p=%p)!\n",
-              semaphore_info);
-      fflush(stderr);
-      return (MagickFail);
-    }
+  /* assert(semaphore_info->signature == MagickSignature); */
 
-  /* fprintf(stderr,"unlock %p\n", semaphore_info); */
-
-  semaphore_info->lock_depth--;
-
-#if defined(HAVE_PTHREAD)
-  /* Enforce that unlocking thread is the same as the locking thread */
-  assert(pthread_equal(semaphore_info->owner_thread_id,pthread_self()));
-  if (semaphore_info->lock_depth == 0)
-    semaphore_info->owner_thread_id=-1;
+#if defined(USE_POSIX_THREADS)
   if (pthread_mutex_unlock(&semaphore_info->mutex))
-    return(MagickFail);
+    status=MagickFail;
 #endif
-#if defined(MSWINDOWS)
-  /* Enforce that unlocking thread is the same as the locking thread */
-  assert(GetCurrentThreadId() == semaphore_info->owner_thread_id);
+#if defined(USE_WIN32_THREADS)
   LeaveCriticalSection(&semaphore_info->mutex);
 #endif
-  return(MagickPass);
+  return(status);
 }
