@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003, 2007 GraphicsMagick Group
+% Copyright (C) 2003 - 2009 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -281,6 +281,11 @@ static void CropToFitImage(Image **image,const double x_shear,
 %
 %
 */
+#if 1
+#if !defined(DisableSlowOpenMP)
+#  define IntegralRotateImageUseOpenMP
+#endif
+#endif
 static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
                                   ExceptionInfo *exception)
 {
@@ -294,8 +299,8 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
     page;
 
   long
-    tile_width_max=128,
-    tile_height_max=128;
+    tile_width_max,
+    tile_height_max;
 
   MagickPassFail
     status=MagickPass;
@@ -306,37 +311,68 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
   assert(image != (Image *) NULL);
   page=image->page;
   rotations%=4;
+
+  {
+    /*
+      Clone appropriately to create rotate image.
+    */
+    unsigned long
+      clone_columns=0,
+      clone_rows=0;
+    
+    switch (rotations)
+      {
+      case 0:
+	clone_columns=0;
+	clone_rows=0;
+	break;
+      case 2:
+	clone_columns=image->columns;
+	clone_rows=image->rows;
+	break;
+      case 1:
+      case 3:
+	clone_columns=image->rows;
+	clone_rows=image->columns;
+	break;
+      }
+    rotate_image=CloneImage(image,clone_columns,clone_rows,True,exception);
+    if (rotate_image == (Image *) NULL)
+      return((Image *) NULL);
+    if (rotations != 0)
+      if (ModifyCache(rotate_image,exception) != MagickPass)
+	{
+	  DestroyImage(rotate_image);
+	  return (Image *) NULL;
+	}
+  }
+
+  tile_height_max=tile_width_max=2048/sizeof(PixelPacket); /* 2k x 2k = 4MB */
   if ((rotations == 1) || (rotations == 3))
     {
-      rotate_image=CloneImage(image,image->rows,image->columns,True,exception);
+      /*
+	Allow override of tile geometry for testing.
+      */
+      const char *
+	value;
 
-      {
-        const char *
-          value;
+      if (!GetPixelCacheInCore(image) || !GetPixelCacheInCore(rotate_image))
+	tile_height_max=tile_width_max=8192/sizeof(PixelPacket); /* 8k x 8k = 64MB */
 
-        /*
-          Allow override of tile geometry for testing.
-        */
-        if ((value=getenv("MAGICK_ROTATE_TILE_GEOMETRY")))
-          {
-            double
-              width,
-              height;
-            
-            if (GetMagickDimension(value,&width,&height,NULL,NULL) == 2)
-              {
-                tile_height_max=(unsigned long) height;
-                tile_width_max=(unsigned long) width;
-              }
-          }
-      }
+      if ((value=getenv("MAGICK_ROTATE_TILE_GEOMETRY")))
+	{
+	  double
+	    width,
+	    height;
+	  
+	  if (GetMagickDimension(value,&width,&height,NULL,NULL) == 2)
+	    {
+	      tile_height_max=(unsigned long) height;
+	      tile_width_max=(unsigned long) width;
+	    }
+	}
     }
-  else
-    {
-      rotate_image=CloneImage(image,image->columns,image->rows,True,exception);
-    }
-  if (rotate_image == (Image *) NULL)
-    return((Image *) NULL);
+
   /*
     Integral rotate the image.
   */
@@ -345,76 +381,18 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
     case 0:
       {
         /*
-          Rotate 0 degrees.
+          Rotate 0 degrees (nothing more to do).
         */
-        long
-          y;
-
-        unsigned long
-          row_count=0;
-
-        (void) strlcpy(message,"[%s] Rotate image 0 degrees...",sizeof(message));
-#if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(static,16) shared(row_count, status)
-#endif
-        for (y=0; y < (long) image->rows; y++)
-          {
-            register const PixelPacket
-              *p;
-            
-            register PixelPacket
-              *q;
-            
-            register const IndexPacket
-              *indexes;
-            
-            IndexPacket
-              *rotate_indexes;
-
-            MagickPassFail
-              thread_status;
-
-            thread_status=status;
-            if (thread_status == MagickFail)
-              continue;
-
-            p=AcquireImagePixels(image,0,y,image->columns,1,exception);
-            q=SetImagePixelsEx(rotate_image,0,y,rotate_image->columns,1,exception);
-            if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
-              thread_status=MagickFail;
-            if (thread_status != MagickFail)
-              {
-                (void) memcpy(q,p,image->columns*sizeof(PixelPacket));
-                indexes=AccessImmutableIndexes(image);
-                rotate_indexes=AccessMutableIndexes(rotate_image);
-                if ((indexes != (IndexPacket *) NULL) &&
-                    (rotate_indexes != (IndexPacket *) NULL))
-                  (void) memcpy(rotate_indexes,indexes,image->columns*
-                                sizeof(IndexPacket));
-
-                if (!SyncImagePixelsEx(rotate_image,exception))
-                  thread_status=MagickFail;
-              }
-#if defined(HAVE_OPENMP)
-#  pragma omp critical
-#endif
-            {
-              row_count++;
-              if (QuantumTick(row_count,image->rows))
-                if (!MagickMonitorFormatted(row_count,image->rows,exception,
-                                            message,image->filename))
-                  thread_status=MagickFail;
-                  
-              if (thread_status == MagickFail)
-                status=MagickFail;
-            }
-          }
+	(void) strlcpy(message,"[%s] Rotate: 0 degrees...",sizeof(message));
+	if (!MagickMonitorFormatted(image->rows-1,image->rows,exception,
+				    message,image->filename))
+	  status=MagickFail;
         break;
       }
     case 1:
       {
         /*
-          Rotate 90 degrees (not a candidate for OpenMP).
+          Rotate 90 degrees.
         */
         magick_int64_t
           tile;
@@ -425,15 +403,26 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
         long
           tile_y;
 
-        (void) strlcpy(message,"[%s] Rotate image 90 degrees...",sizeof(message));
+        (void) strlcpy(message,"[%s] Rotate: 90 degrees...",sizeof(message));
         total_tiles=(((image->rows/tile_height_max)+1)*
                      ((image->columns/tile_width_max)+1));        
         tile=0;
+
+#if defined(IntegralRotateImageUseOpenMP)
+#  pragma omp parallel for schedule(static,1) shared(status, tile)
+#endif
         for (tile_y=0; tile_y < (long) image->rows; tile_y+=tile_height_max)
           {
             long
               tile_x;
 
+            MagickPassFail
+              thread_status;
+
+            thread_status=status;
+            if (thread_status == MagickFail)
+              continue;
+
             for (tile_x=0; tile_x < (long) image->columns; tile_x+=tile_width_max)
               {
                 long
@@ -468,7 +457,7 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
                                                tile_width,tile_height,exception);
                 if (tile_pixels == (const PixelPacket *) NULL)
                   {
-                    status=MagickFail;
+                    thread_status=MagickFail;
                     break;
                   }
                 /*
@@ -496,11 +485,11 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
                     register long
                       x;
 
-                    q=SetImagePixels(rotate_image,dest_tile_x,dest_tile_y+y,
-                                     tile_height,1);
+                    q=SetImagePixelsEx(rotate_image,dest_tile_x,dest_tile_y+y,
+                                       tile_height,1,exception);
                     if (q == (PixelPacket *) NULL)
                       {
-                        status=MagickFail;
+                        thread_status=MagickFail;
                         break;
                       }
                     /*
@@ -538,21 +527,26 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
                               }
                           }
                       }
-                    if (!SyncImagePixels(rotate_image))
+                    if (!SyncImagePixelsEx(rotate_image,exception))
                       {
-                        status=MagickFail;
+                        thread_status=MagickFail;
                         break;
                       }
                   }
 
-                tile++;
-                if (QuantumTick(tile,total_tiles))
-                  if (!MagickMonitorFormatted(tile,total_tiles,exception,
-                                              message,image->filename))
+#if defined(IntegralRotateImageUseOpenMP)
+#  pragma omp critical (GM_IntegralRotateImage)
+#endif
+                {
+                  tile++;
+                  if (QuantumTick(tile,total_tiles))
+                    if (!MagickMonitorFormatted(tile,total_tiles,exception,
+                                                message,image->filename))
+                      thread_status=MagickFail;
+                  
+                  if (thread_status == MagickFail)
                     status=MagickFail;
- 
-                if (status == MagickFail)
-                  break;
+                }
               }
           }
         Swap(page.width,page.height);
@@ -571,9 +565,9 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
         unsigned long
           row_count=0;
 
-        (void) strlcpy(message,"[%s] Rotate image 180 degrees...",sizeof(message));
-#if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(static,16) shared(row_count, status)
+        (void) strlcpy(message,"[%s] Rotate: 180 degrees...",sizeof(message));
+#if defined(IntegralRotateImageUseOpenMP)
+#  pragma omp parallel for schedule(static,8) shared(row_count, status)
 #endif
         for (y=0; y < (long) image->rows; y++)
           {
@@ -618,8 +612,8 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
                 if (!SyncImagePixelsEx(rotate_image,exception))
                   thread_status=MagickFail;
               }
-#if defined(HAVE_OPENMP)
-#  pragma omp critical
+#if defined(IntegralRotateImageUseOpenMP)
+#  pragma omp critical (GM_IntegralRotateImage)
 #endif
             {
               row_count++;
@@ -639,7 +633,7 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
     case 3:
       {
         /*
-          Rotate 270 degrees (not a candidate for OpenMP).
+          Rotate 270 degrees.
         */
 
         magick_int64_t
@@ -651,16 +645,25 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
         long
           tile_y;
 
-        (void) strlcpy(message,"[%s] Rotate image 270 degrees...",sizeof(message));
+        (void) strlcpy(message,"[%s] Rotate: 270 degrees...",sizeof(message));
         total_tiles=(((image->rows/tile_height_max)+1)*
                      ((image->columns/tile_width_max)+1));
         tile=0;
-        status=MagickPass;
+#if defined(IntegralRotateImageUseOpenMP)
+#  pragma omp parallel for schedule(static,1) shared(status, tile)
+#endif
         for (tile_y=0; tile_y < (long) image->rows; tile_y+=tile_height_max)
           {
             long
               tile_x;
 
+            MagickPassFail
+              thread_status;
+
+            thread_status=status;
+            if (thread_status == MagickFail)
+              continue;
+
             for (tile_x=0; tile_x < (long) image->columns; tile_x+=tile_width_max)
               {
                 long
@@ -677,7 +680,6 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
                 const PixelPacket
                   *tile_pixels;
                     
-                tile++;
                 /*
                   Compute image region corresponding to tile.
                 */
@@ -696,7 +698,7 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
                                                tile_width,tile_height,exception);
                 if (tile_pixels == (const PixelPacket *) NULL)
                   {
-                    status=MagickFail;
+                    thread_status=MagickFail;
                     break;
                   }
                 /*
@@ -724,10 +726,11 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
                     IndexPacket
                       *rotate_indexes;
 
-                    q=SetImagePixels(rotate_image,dest_tile_x,dest_tile_y+y,tile_height,1);
+                    q=SetImagePixelsEx(rotate_image,dest_tile_x,dest_tile_y+y,
+                                       tile_height,1,exception);
                     if (q == (PixelPacket *) NULL)
                       {
-                        status=MagickFail;
+                        thread_status=MagickFail;
                         break;
                       }
                     /*
@@ -765,21 +768,30 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
                               }
                           }
                       }
-                    if (!SyncImagePixels(rotate_image))
+                    if (!SyncImagePixelsEx(rotate_image,exception))
                       {
-                        status=MagickFail;
+                        thread_status=MagickFail;
                         break;
                       }
                   }
-                if (QuantumTick(tile,total_tiles))
-                  if (!MagickMonitorFormatted(tile,total_tiles,exception,
-                                              message,image->filename))
+
+#if defined(IntegralRotateImageUseOpenMP)
+#  pragma omp critical (GM_IntegralRotateImage)
+#endif
+                {
+                  tile++;
+                  if (QuantumTick(tile,total_tiles))
+                    if (!MagickMonitorFormatted(tile,total_tiles,exception,
+                                                message,image->filename))
+                      thread_status=MagickFail;
+                }
+
+                if (thread_status == MagickFail)
+                  {
                     status=MagickFail;
-                if (status == MagickFail)
-                  break;
+                    break;
+                  }
               }
-            if (status == MagickFail)
-              break;
           }
         Swap(page.width,page.height);
         Swap(page.x,page.y);
@@ -832,7 +844,7 @@ static void XShearImage(Image *image,const double degrees,
                         const unsigned long width,const unsigned long height,
                         const long x_offset,long y_offset)
 {
-#define XShearImageText  "[%s] X Shear image...  "
+#define XShearImageText  "[%s] X Shear: %+g degrees, region %lux%lu%+ld%+ld...  "
 
   long
     y;
@@ -850,7 +862,7 @@ static void XShearImage(Image *image,const double degrees,
   is_grayscale=image->is_grayscale;
 
 #if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(static,16) shared(row_count, status)
+#  pragma omp parallel for schedule(dynamic,8) shared(row_count, status)
 #endif
   for (y=0; y < (long) height; y++)
     {
@@ -895,7 +907,7 @@ static void XShearImage(Image *image,const double degrees,
           direction=LEFT;
         }
       step=(long) floor(displacement);
-      alpha=(double) MaxRGB*(displacement-step);
+      alpha=MaxRGBDouble*(displacement-step);
       if (alpha == 0.0)
         {
           /*
@@ -948,13 +960,15 @@ static void XShearImage(Image *image,const double degrees,
             thread_status=MagickFail;
 
 #if defined(HAVE_OPENMP)
-#  pragma omp critical
+#  pragma omp critical (GM_XShearImage)
 #endif
           {
             row_count++;
             if (QuantumTick(row_count,height))
               if (!MagickMonitorFormatted(row_count,height,&image->exception,
-                                          XShearImageText,image->filename))
+                                          XShearImageText,image->filename,
+					  degrees,width,height,
+					  x_offset,y_offset))
                 thread_status=MagickFail;
             
             if (thread_status == MagickFail)
@@ -1035,13 +1049,15 @@ static void XShearImage(Image *image,const double degrees,
       if (!SyncImagePixelsEx(image,&image->exception))
         thread_status=MagickFail;
 #if defined(HAVE_OPENMP)
-#  pragma omp critical
+#  pragma omp critical (GM_XShearImage)
 #endif
       {
         row_count++;
         if (QuantumTick(row_count,height))
           if (!MagickMonitorFormatted(row_count,height,&image->exception,
-                                      XShearImageText,image->filename))
+                                          XShearImageText,image->filename,
+					  degrees,width,height,
+					  x_offset,y_offset))
             thread_status=MagickFail;
         
         if (thread_status == MagickFail)
@@ -1090,7 +1106,7 @@ static void YShearImage(Image *image,const double degrees,
                         const unsigned long width,const unsigned long height,long x_offset,
                         const long y_offset)
 {
-#define YShearImageText  "[%s] Y Shear image..."
+#define YShearImageText  "[%s] Y Shear: %+g degrees, region %lux%lu%+ld%+ld...  "
 
   long
     y;
@@ -1108,7 +1124,7 @@ static void YShearImage(Image *image,const double degrees,
   is_grayscale=image->is_grayscale;
 
 #if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(static,16) shared(row_count, status)
+#  pragma omp parallel for schedule(dynamic,8) shared(row_count, status)
 #endif
   for (y=0; y < (long) width; y++)
     {
@@ -1206,13 +1222,15 @@ static void YShearImage(Image *image,const double degrees,
             thread_status=MagickFail;
 
 #if defined(HAVE_OPENMP)
-#  pragma omp critical
+#  pragma omp critical (GM_YShearImage)
 #endif
           {
             row_count++;
             if (QuantumTick(row_count,width))
               if (!MagickMonitorFormatted(row_count,width,&image->exception,
-                                          YShearImageText,image->filename))
+                                          YShearImageText,image->filename,
+					  degrees,width,height,
+					  x_offset,y_offset))
                 thread_status=MagickFail;
             
             if (thread_status == MagickFail)
@@ -1294,13 +1312,15 @@ static void YShearImage(Image *image,const double degrees,
         thread_status=MagickFail;
 
 #if defined(HAVE_OPENMP)
-#  pragma omp critical
+#  pragma omp critical (GM_YShearImage)
 #endif
       {
         row_count++;
         if (QuantumTick(row_count,width))
           if (!MagickMonitorFormatted(row_count,width,&image->exception,
-                                      YShearImageText,image->filename))
+                                      YShearImageText,image->filename,
+				      degrees,width,height,
+				      x_offset,y_offset))
             thread_status=MagickFail;
         
         if (thread_status == MagickFail)
