@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2003, 2007 GraphicsMagick Group
+  Copyright (C) 2003 - 2009 GraphicsMagick Group
   Copyright (C) 2002 ImageMagick Studio
  
   This program is covered by multiple licenses, which are described in
@@ -15,6 +15,18 @@
 extern "C" {
 #endif
 
+/*
+  This define is not used by GraphicsMagick and it causes some headers
+  from other installed packages (e.g. MinGW libpthread) to misbehave.
+*/
+#undef HAVE_CONFIG_H
+
+/*
+  Allow configuration of cache line size.  If smaller than actual
+  cache line size, then performance may suffer.
+*/
+#define MAGICK_CACHE_LINE_SIZE 128
+
 /*
   Note that the WIN32 and WIN64 definitions are provided by the build
   configuration rather than the compiler.  Definitions available from
@@ -140,7 +152,20 @@ extern "C" {
 #endif
 
 #define MagickSignature  0xabacadabUL
-#define MaxTextExtent  2053
+
+/*
+  This size is the default minimum string allocation size (heap or
+  stack) for a C string in GraphicsMagick.  The weird size is claimed
+  to be based on 2*FILENAME_MAX (not including terminating NULL) on
+  some antique system.  Linux has a FILENAME_MAX definition, but it is
+  4096 bytes.  Many OSs have path limits of 1024 bytes.
+
+  The FormatString() function assumes that the buffer it is writing to
+  has at least this many bytes remaining.
+*/
+#if !defined(MaxTextExtent)
+#  define MaxTextExtent  2053
+#endif
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -219,6 +244,9 @@ extern "C" {
 #    define NAMLEN(dirent) strlen((dirent)->d_name)
 #  endif
 #  include <sys/wait.h>
+#  if defined(HAVE_SYS_RESOURCE_H)
+#    include <sys/resource.h>
+#  endif /* defined(HAVE_SYS_RESOURCE_H)  */
 #  include <pwd.h>
 #endif
 
@@ -230,6 +258,16 @@ extern "C" {
 #  define S_ISREG(mode) (((mode) & S_IFMT) == S_IFREG)
 #endif
 
+/*
+  Avoid shadowing system library globals and functions.
+*/
+#undef gamma
+#define gamma gamma_magick
+#undef swab
+#define swab swab_magick
+#undef y1
+#define y1 y1_magick
+
 #include "magick/magick_types.h"
 #include "magick/image.h"
 #include "magick/list.h"
@@ -242,7 +280,6 @@ extern "C" {
 #  endif
 #endif
 
-
 #if defined(POSIX)
 # include "magick/unix_port.h"
 #endif /* defined(POSIX) */
@@ -420,11 +457,32 @@ extern int vsnprintf(char *s, size_t n, const char *format, va_list ap);
 */
 #if defined(MSWINDOWS) && !defined(Windows95) && !defined(__BORLANDC__)
   /* Windows '95 and Borland C do not support _lseeki64 */
-#  define MagickSeek(file,offset,whence)  _lseeki64(file,offset,whence)
-#  define MagickTell(file) _telli64(file)
+#  define MagickSeek(fildes,offset,whence)  _lseeki64(fildes,/* __int64 */ offset,whence)
+#  define MagickTell(fildes) /* __int64 */ _telli64(fildes)
 #else
-#  define MagickSeek(file,offset,whence)  lseek(file,offset,whence)
-#  define MagickTell(file) tell(file)
+#  define MagickSeek(fildes,offset,whence)  lseek(fildes,offset,whence)
+#  define MagickTell(fildes) tell(filedes)
+#endif
+
+#if defined(MSWINDOWS) && !defined(Windows95) && !defined(__BORLANDC__) && \
+  !(defined(_MSC_VER) && _MSC_VER < 1400) && \
+  !(defined(__MINGW32__) && __MSVCRT_VERSION__ < 0x800)
+  /*
+    Windows '95 and Borland C do not support _lseeki64
+    Visual Studio does not support _fseeki64 and _ftelli64 until the 2005 release.
+    Without these interfaces, files over 2GB in size are not supported for Windows.
+  */
+#  define MagickFseek(stream,offset,whence) _fseeki64(stream,/* __int64 */ offset,whence)
+#  define MagickFstat(fildes,stat_buff) _fstati64(fildes,/* struct _stati64 */ stat_buff)
+#  define MagickFtell(stream) /* __int64 */ _ftelli64(stream)
+#  define MagickStatStruct_t struct _stati64
+#  define MagickStat(path,stat_buff) _stati64(path,/* struct _stati64 */ stat_buff)
+#else
+#  define MagickFseek(stream,offset,whence) fseek(stream,offset,whence)
+#  define MagickFstat(fildes,stat_buff) fstat(fildes,stat_buff)
+#  define MagickFtell(stream) ftell(stream)
+#  define MagickStatStruct_t struct stat
+#  define MagickStat(path,stat_buff) stat(path,stat_buff)
 #endif
 
 #if !defined(HAVE_POPEN) && defined(HAVE__POPEN)
@@ -469,13 +527,14 @@ extern MagickExport const char
   *DefaultTileLabel,
   *ForegroundColor,
   *HighlightColor,
-  *LoadImageText,
-  *LoadImagesText,
   *MatteColor,
   *PSDensityGeometry,
-  *PSPageGeometry,
-  *SaveImageText,
-  *SaveImagesText;
+  *PSPageGeometry;
+
+#define LoadImageText "[%s] Loading image: %lux%lu...  "
+#define SaveImageText "[%s] Saving image: %lux%lu...  "
+#define LoadImagesText "[%s] Loading images...  "
+#define SaveImagesText "[%s] Saving images...  "
 
 extern MagickExport const unsigned long
   DefaultCompressionQuality;
