@@ -78,7 +78,7 @@
 MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
                               ExceptionInfo *exception)
 {
-#define ChopImageText  "[%s] Chop image..."
+#define ChopImageText "[%s] Chop..."
 
   Image
     *chop_image;
@@ -135,8 +135,8 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
   /*
     Extract chop image.
   */
-#if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(static,16) shared(row_count, status)
+#if defined(HAVE_OPENMP)  && !defined(DisableSlowOpenMP)
+#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
 #endif
   for (y=0; y < (long) clone_info.y; y++)
     {
@@ -186,8 +186,8 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
           if (!SyncImagePixelsEx(chop_image,exception))
             thread_status=MagickFail;
         }
-#if defined(HAVE_OPENMP)
-#  pragma omp critical
+#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
+#  pragma omp critical (GM_ChopImage)
 #endif
       {
         row_count++;
@@ -203,8 +203,8 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
   /*
     Extract chop image.
   */
-#if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(static,16) shared(row_count, status)
+#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
+#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
 #endif
   for (y=0; y < (long) (image->rows-(clone_info.y+clone_info.height)); y++)
     {
@@ -254,8 +254,8 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
           if (!SyncImagePixelsEx(chop_image,exception))
             thread_status=MagickFail;
         }
-#if defined(HAVE_OPENMP)
-#  pragma omp critical
+#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
+#  pragma omp critical (GM_ChopImage)
 #endif
       {
         row_count++;
@@ -500,8 +500,8 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
   crop_image->page=page;
   if ((geometry->width == 0) || (geometry->height == 0))
     (void) memset(&crop_image->page,0,sizeof(RectangleInfo));
-#if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(static,16) shared(row_count, status)
+#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
+#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
 #endif
   for (y=0; y < (long) crop_image->rows; y++)
     {
@@ -541,14 +541,14 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
           if (!SyncImagePixelsEx(crop_image,exception))
             thread_status=MagickFail;
         }
-#if defined(HAVE_OPENMP)
-#  pragma omp critical
+#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
+#  pragma omp critical (GM_CropImage)
 #endif
       {
         row_count++;
         if (QuantumTick(row_count,crop_image->rows))
-          if (!MagickMonitorFormatted(row_count,crop_image->rows-1,exception,
-                                      "[%s] Crop %lux%lu+%ld+%ld image...",
+          if (!MagickMonitorFormatted(row_count,crop_image->rows,exception,
+                                      "[%s] Crop: %lux%lu+%ld+%ld...",
                                       crop_image->filename,
                                       crop_image->columns,crop_image->rows,
                                       page.x,page.y))
@@ -844,7 +844,7 @@ MagickExport Image *FlattenImages(const Image *image,ExceptionInfo *exception)
 */
 MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
 {
-#define FlipImageText  "[%s] Flip image..."
+#define FlipImageText "[%s] Flip..."
 
   Image
     *flip_image;
@@ -871,8 +871,8 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
   /*
     Flip each row.
   */
-#if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(static,16) shared(row_count, status)
+#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
+#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
 #endif
   for (y=0; y < (long) flip_image->rows; y++)
     {
@@ -912,8 +912,8 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
           if (!SyncImagePixelsEx(flip_image,exception))
             thread_status=MagickFail;
         }
-#if defined(HAVE_OPENMP)
-#  pragma omp critical
+#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
+#  pragma omp critical (GM_FlipImage)
 #endif
       {
         row_count++;
@@ -963,7 +963,7 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
 */
 MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
 {
-#define FlopImageText  "[%s] Flop image..."
+#define FlopImageText "[%s] Flop..."
 
   Image
     *flop_image;
@@ -990,8 +990,8 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
   /*
     Flop each row.
   */
-#if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(static,16) shared(row_count, status)
+#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
+#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
 #endif
   for (y=0; y < (long) flop_image->rows; y++)
     {
@@ -1039,8 +1039,8 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
           if (!SyncImagePixelsEx(flop_image,exception))
             thread_status=MagickFail;
         }
-#if defined(HAVE_OPENMP)
-#  pragma omp critical
+#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
+#  pragma omp critical (GM_FlopImage)
 #endif
       {
         row_count++;
@@ -1091,7 +1091,7 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
 */
 MagickExport Image *MosaicImages(const Image *image,ExceptionInfo *exception)
 {
-#define MosaicImageText  "[%s] Create image mosaic..."
+#define MosaicImageText "[%s] Create mosaic..."
 
   Image
     *mosaic_image;
@@ -1332,18 +1332,19 @@ MagickExport Image *ShaveImage(const Image *image,
 %
 */
 MagickExport void TransformImage(Image **image,const char *crop_geometry,
-  const char *image_geometry)
+				 const char *image_geometry)
 {
   Image
+    *previous,
     *resize_image,
     *transform_image;
 
-  int
-    flags;
-
   RectangleInfo
     geometry;
 
+  int
+    flags;
+
   assert(image != (Image **) NULL);
   assert((*image)->signature == MagickSignature);
   transform_image=(*image);
@@ -1352,9 +1353,6 @@ MagickExport void TransformImage(Image **image,const char *crop_geometry,
       Image
         *crop_image;
 
-      RectangleInfo
-        geometry;
-
       /*
         Crop image to a user specified size.
       */
@@ -1363,7 +1361,9 @@ MagickExport void TransformImage(Image **image,const char *crop_geometry,
       if ((geometry.width == 0) || (geometry.height == 0) ||
           ((flags & XValue) != 0) || ((flags & YValue) != 0) ||
           (flags & PercentValue))
-        crop_image=CropImage(transform_image,&geometry,&(*image)->exception);
+	{
+	  crop_image=CropImage(transform_image,&geometry,&(*image)->exception);
+	}
       else
         if ((transform_image->columns > geometry.width) ||
             (transform_image->rows > geometry.height))
@@ -1386,56 +1386,65 @@ MagickExport void TransformImage(Image **image,const char *crop_geometry,
             height=geometry.height;
             next=(Image *) NULL;
             for (y=0; y < (long) transform_image->rows; y+=height)
-            {
-              for (x=0; x < (long) transform_image->columns; x+=width)
-              {
-                geometry.width=width;
-                geometry.height=height;
-                geometry.x=x;
-                geometry.y=y;
-                next=CropImage(transform_image,&geometry,&(*image)->exception);
-                if (next == (Image *) NULL)
-                  break;
-                if (crop_image == (Image *) NULL)
-                  crop_image=next;
-                else
-                  {
-                    next->previous=crop_image;
-                    crop_image->next=next;
-                    crop_image=crop_image->next;
-                  }
-              }
-              if (next == (Image *) NULL)
-                break;
-            }
+	      {
+		for (x=0; x < (long) transform_image->columns; x+=width)
+		  {
+		    geometry.width=width;
+		    geometry.height=height;
+		    geometry.x=x;
+		    geometry.y=y;
+		    next=CropImage(transform_image,&geometry,&(*image)->exception);
+		    if (next == (Image *) NULL)
+		      break;
+		    if (crop_image == (Image *) NULL)
+		      crop_image=next;
+		    else
+		      {
+			next->previous=crop_image;
+			crop_image->next=next;
+			crop_image=crop_image->next;
+		      }
+		  }
+		if (next == (Image *) NULL)
+		  break;
+	      }
           }
       if (crop_image != (Image *) NULL)
         {
+	  previous=transform_image->previous;
+	  crop_image->next=transform_image->next;
           DestroyImage(transform_image);
+	  transform_image=(Image *) NULL;
           while (crop_image->previous != (Image *) NULL)
             crop_image=crop_image->previous;
+	  crop_image->previous=previous;
           transform_image=crop_image;
         }
       *image=transform_image;
     }
   if (image_geometry == (const char *) NULL)
     return;
+
   /*
     Scale image to a user specified size.
   */
   SetGeometry(transform_image,&geometry);
   flags=GetMagickGeometry(image_geometry,&geometry.x,&geometry.y,
-    &geometry.width,&geometry.height);
+			  &geometry.width,&geometry.height);
   if ((transform_image->columns == geometry.width) &&
       (transform_image->rows == geometry.height))
     return;
+  
   /*
     Resize image.
   */
   resize_image=ZoomImage(transform_image,geometry.width,geometry.height,
-    &(*image)->exception);
+			 &(*image)->exception);
   if (resize_image == (Image *) NULL)
     return;
+
+  previous=transform_image->previous;
+  resize_image->next=transform_image->next;
   DestroyImage(transform_image);
   transform_image=resize_image;
   *image=transform_image;
